@@ -72,7 +72,7 @@ public class HDFSAuditDestination extends AuditDestination {
 		}
 		if (isStopped) {
 			addDeferredCount(events.size());
-			logError("log() called after stop was requested. name=" + getName());
+			logError("log() called after stop was requested. name={}", getName());
 			return false;
 		}
 		try {
@@ -86,7 +86,9 @@ public class HDFSAuditDestination extends AuditDestination {
 			logError("Error writing to log file.", t);
 			return false;
 		} finally {
-			logger.info("Flushing HDFS audit. Event Size:" + events.size());
+			if (logger.isDebugEnabled()) {
+				logger.debug("Flushing HDFS audit. Event Size:{}", events.size());
+			}
 			if (auditWriter != null) {
 				flush();
 			}
@@ -102,7 +104,7 @@ public class HDFSAuditDestination extends AuditDestination {
 			return false;
 		}
 		if (isStopped) {
-			logError("log() called after stop was requested. name=" + getName());
+			logError("log() called after stop was requested. name={}", getName());
 			return false;
 		}
 
@@ -115,7 +117,7 @@ public class HDFSAuditDestination extends AuditDestination {
 			logError("Error writing to log file.", t);
 			return false;
 		} finally {
-			logger.info("Flushing HDFS audit. File:" + file.getAbsolutePath() + file.getName());
+			logger.info("Flushing HDFS audit. File:{}{}", file.getAbsolutePath(), file.getName());
 			if (auditWriter != null) {
 				flush();
 			}
@@ -125,7 +127,9 @@ public class HDFSAuditDestination extends AuditDestination {
 
 	@Override
 	public void flush() {
-		logger.info("Flush called. name=" + getName());
+		if (logger.isDebugEnabled()) {
+			logger.debug("==> HDFSAuditDestination.flush() called. name={}", getName());
+		}
 		MiscUtil.executePrivilegedAction(new PrivilegedAction<Void>() {
 			@Override
 			public Void run() {
@@ -133,6 +137,9 @@ public class HDFSAuditDestination extends AuditDestination {
 				return null;
 			}
 		});
+		if (logger.isDebugEnabled()) {
+			logger.debug("<== HDFSAuditDestination.flush() called. name={}", getName());
+		}
 	}
 
 	/*
@@ -147,7 +154,7 @@ public class HDFSAuditDestination extends AuditDestination {
 			logStatusIfRequired();
 			addTotalCount(events.size());
 			addDeferredCount(events.size());
-			logError("log() called after stop was requested. name=" + getName());
+			logError("log() called after stop was requested. name={}", getName());
 			return false;
 		}
 		List<String> jsonList = new ArrayList<String>();
@@ -155,7 +162,7 @@ public class HDFSAuditDestination extends AuditDestination {
 			try {
 				jsonList.add(MiscUtil.stringify(event));
 			} catch (Throwable t) {
-				logger.error("Error converting to JSON. event=" + event);
+				logger.error("Error converting to JSON. event={}", event);
 				addTotalCount(1);
 				addFailedCount(1);
 				logFailedEvent(event);
