@@ -19,10 +19,10 @@
 
 package org.apache.ranger.audit.queue;
 
-import com.google.gson.Gson;
-import com.google.gson.GsonBuilder;
 import org.apache.ranger.audit.model.AuditEventBase;
+import org.apache.ranger.audit.model.AuditIndexRecord;
 import org.apache.ranger.audit.model.AuthzAuditEvent;
+import org.apache.ranger.audit.model.SPOOL_FILE_STATUS;
 import org.apache.ranger.audit.provider.AuditHandler;
 import org.apache.ranger.audit.provider.MiscUtil;
 import org.slf4j.Logger;
@@ -60,10 +60,6 @@ import java.util.concurrent.TimeUnit;
 public class AuditFileQueueSpool implements Runnable {
     private static final Logger logger = LoggerFactory.getLogger(AuditFileQueueSpool.class);
 
-    public enum SPOOL_FILE_STATUS {
-        pending, write_inprogress, read_inprogress, done
-    }
-
     public static final String PROP_FILE_SPOOL_LOCAL_DIR			   = "filespool.dir";
     public static final String PROP_FILE_SPOOL_LOCAL_FILE_NAME 		   = "filespool.filename.format";
     public static final String PROP_FILE_SPOOL_ARCHIVE_DIR 			   = "filespool.archive.dir";
@@ -114,8 +110,6 @@ public class AuditFileQueueSpool implements Runnable {
     boolean isDestDown 	= false;
     boolean isSpoolingSuccessful = true;
 
-    private Gson gson = null;
-
     public AuditFileQueueSpool(AuditHandler consumerProvider) {
         this.consumerProvider = consumerProvider;
     }
@@ -139,8 +133,6 @@ public class AuditFileQueueSpool implements Runnable {
         }
 
         try {
-            gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss.SSS")
-                    .create();
             // Initial folder and file properties
             String logFolderProp = MiscUtil.getStringProperty(props, propPrefix
                     + "." + PROP_FILE_SPOOL_LOCAL_DIR);
@@ -256,17 +248,17 @@ public class AuditFileQueueSpool implements Runnable {
             // Load index file
             loadIndexFile();
             for (AuditIndexRecord auditIndexRecord : indexRecords) {
-                if (!auditIndexRecord.status.equals(SPOOL_FILE_STATUS.done)) {
+                if (!auditIndexRecord.getStatus().equals(SPOOL_FILE_STATUS.done)) {
                     isPending = true;
                 }
-                if (auditIndexRecord.status
+                if (auditIndexRecord.getStatus()
                         .equals(SPOOL_FILE_STATUS.write_inprogress)) {
                     currentWriterIndexRecord = auditIndexRecord;
                     logger.info("currentWriterIndexRecord="
-                            + currentWriterIndexRecord.filePath
+                            + currentWriterIndexRecord.getFilePath()
                             + ", queueName=" + FILE_QUEUE_PROVIDER_NAME);
                 }
-                if (auditIndexRecord.status
+                if (auditIndexRecord.getStatus()
                         .equals(SPOOL_FILE_STATUS.read_inprogress)) {
                     indexQueue.add(auditIndexRecord);
                 }
@@ -274,8 +266,8 @@ public class AuditFileQueueSpool implements Runnable {
             printIndex();
             for (int i = 0; i < indexRecords.size(); i++) {
                 AuditIndexRecord auditIndexRecord = indexRecords.get(i);
-                if (auditIndexRecord.status.equals(SPOOL_FILE_STATUS.pending)) {
-                    File consumerFile = new File(auditIndexRecord.filePath);
+                if (auditIndexRecord.getStatus().equals(SPOOL_FILE_STATUS.pending)) {
+                    File consumerFile = new File(auditIndexRecord.getFilePath());
                     if (!consumerFile.exists()) {
                         logger.error("INIT: Consumer file="
                                 + consumerFile.getPath() + " not found.");
@@ -525,11 +517,11 @@ public class AuditFileQueueSpool implements Runnable {
 
             AuditIndexRecord tmpIndexRecord = new AuditIndexRecord();
 
-            tmpIndexRecord.id = MiscUtil.generateUniqueId();
-            tmpIndexRecord.filePath = outLogFile.getPath();
-            tmpIndexRecord.status = SPOOL_FILE_STATUS.write_inprogress;
-            tmpIndexRecord.fileCreateTime = currentTime;
-            tmpIndexRecord.lastAttempt = true;
+            tmpIndexRecord.setId(MiscUtil.generateUniqueId());
+            tmpIndexRecord.setFilePath(outLogFile.getPath());
+            tmpIndexRecord.setStatus(SPOOL_FILE_STATUS.write_inprogress);
+            tmpIndexRecord.setFileCreateTime(currentTime);
+            tmpIndexRecord.setLastAttempt(true);
             currentWriterIndexRecord = tmpIndexRecord;
             indexRecords.add(currentWriterIndexRecord);
             saveIndexFile();
@@ -540,9 +532,9 @@ public class AuditFileQueueSpool implements Runnable {
                 // in append mode.
                 logger.info("Opening existing file for append. queueName="
                         + FILE_QUEUE_PROVIDER_NAME + ", fileName="
-                        + currentWriterIndexRecord.filePath);
+                        + currentWriterIndexRecord.getFilePath());
                 logWriter = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(
-                        currentWriterIndexRecord.filePath, true),"UTF-8")));
+                        currentWriterIndexRecord.getFilePath(), true),"UTF-8")));
             }
         }
         return logWriter;
@@ -564,12 +556,12 @@ public class AuditFileQueueSpool implements Runnable {
                     logWriter = null;
                     closeFile = false;
                 }
-                currentWriterIndexRecord.status = SPOOL_FILE_STATUS.pending;
-                currentWriterIndexRecord.writeCompleteTime = new Date();
+                currentWriterIndexRecord.setStatus(SPOOL_FILE_STATUS.pending);
+                currentWriterIndexRecord.setWriteCompleteTime(new Date());
                 saveIndexFile();
                 logger.info("Adding file to queue. queueName="
                         + FILE_QUEUE_PROVIDER_NAME + ", fileName="
-                        + currentWriterIndexRecord.filePath);
+                        + currentWriterIndexRecord.getFilePath());
                 indexQueue.add(currentWriterIndexRecord);
                 currentWriterIndexRecord = null;
             }
@@ -578,11 +570,11 @@ public class AuditFileQueueSpool implements Runnable {
 
     private void rollOverSpoolFileByTime() {
         if (System.currentTimeMillis()
-                - currentWriterIndexRecord.fileCreateTime.getTime() > fileRolloverSec * 1000) {
+                - currentWriterIndexRecord.getFileCreateTime().getTime() > fileRolloverSec * 1000) {
             closeFile = true;
             logger.info("Closing file. Rolling over. queueName="
                     + FILE_QUEUE_PROVIDER_NAME + ", fileName="
-                    + currentWriterIndexRecord.filePath);
+                    + currentWriterIndexRecord.getFilePath());
         }
     }
 
@@ -600,9 +592,13 @@ public class AuditFileQueueSpool implements Runnable {
             String line;
             while ((line = br.readLine()) != null) {
                 if (!line.isEmpty() && !line.startsWith("#")) {
-                    AuditIndexRecord record = gson.fromJson(line,
-                            AuditIndexRecord.class);
-                    indexRecords.add(record);
+			try {
+	                    AuditIndexRecord record = MiscUtil.fromJson(line,
+	                            AuditIndexRecord.class);
+	                    indexRecords.add(record);
+			} catch (Exception e) {
+						logger.error("Error parsing following JSON: "+line, e);
+					}
                 }
             }
         } finally {
@@ -618,7 +614,7 @@ public class AuditFileQueueSpool implements Runnable {
         while (iter.hasNext()) {
             AuditIndexRecord record = iter.next();
             logger.info("INDEX=" + record + ", isFileExist="
-                    + (new File(record.filePath).exists()));
+                    + (new File(record.getFilePath()).exists()));
         }
         logger.info("INDEX printIndex() ==== END");
     }
@@ -628,8 +624,8 @@ public class AuditFileQueueSpool implements Runnable {
         Iterator<AuditIndexRecord> iter = indexRecords.iterator();
         while (iter.hasNext()) {
             AuditIndexRecord record = iter.next();
-            if (record.id.equals(indexRecord.id)) {
-                logger.info("Removing file from index. file=" + record.filePath
+            if (record.getId().equals(indexRecord.getId())) {
+                logger.info("Removing file from index. file=" + record.getFilePath()
                         + ", queueName=" + FILE_QUEUE_PROVIDER_NAME
                         + ", consumer=" + consumerProvider.getName());
 
@@ -646,21 +642,19 @@ public class AuditFileQueueSpool implements Runnable {
     }
 
     synchronized void saveIndexFile() throws FileNotFoundException, IOException {
-        PrintWriter out = new PrintWriter(indexFile,"UTF-8");
-        for (AuditIndexRecord auditIndexRecord : indexRecords) {
-            out.println(gson.toJson(auditIndexRecord));
+        try (PrintWriter out = new PrintWriter(indexFile, "UTF-8")) {
+            for (AuditIndexRecord auditIndexRecord : indexRecords) {
+                out.println(MiscUtil.stringify(auditIndexRecord));
+            }
         }
-        out.close();
-        // printIndex();
-
     }
 
     void appendToDoneFile(AuditIndexRecord indexRecord)
             throws FileNotFoundException, IOException {
-        logger.info("Moving to done file. " + indexRecord.filePath
+        logger.info("Moving to done file. " + indexRecord.getFilePath()
                 + ", queueName=" + FILE_QUEUE_PROVIDER_NAME + ", consumer="
                 + consumerProvider.getName());
-        String line = gson.toJson(indexRecord);
+        String line = MiscUtil.stringify(indexRecord);
         PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(
                 indexDoneFile, true),"UTF-8")));
         out.println(line);
@@ -674,7 +668,7 @@ public class AuditFileQueueSpool implements Runnable {
         File logFile = null;
         File archiveFile = null;
         try {
-            logFile = new File(indexRecord.filePath);
+            logFile = new File(indexRecord.getFilePath());
             String fileName = logFile.getName();
             archiveFile = new File(archiveFolder, fileName);
             logger.info("Moving logFile " + logFile + " to " + archiveFile);
@@ -708,9 +702,10 @@ public class AuditFileQueueSpool implements Runnable {
                     int filesDeletedCount = 0;
                     while ((line = br.readLine()) != null) {
                         if (!line.isEmpty() && !line.startsWith("#")) {
-                            AuditIndexRecord record = gson.fromJson(line,
+				try {
+	                            AuditIndexRecord record = MiscUtil.fromJson(line,
                                     AuditIndexRecord.class);
-                            logFile = new File(record.filePath);
+                            logFile = new File(record.getFilePath());
                             String fileName = logFile.getName();
                             archiveFile = new File(archiveFolder, fileName);
                             if (archiveFile.exists()) {
@@ -728,6 +723,9 @@ public class AuditFileQueueSpool implements Runnable {
                                     break;
                                 }
                             }
+				} catch (Exception e) {
+								logger.error("Error parsing following JSON: "+line, e);
+							}
                         }
                     }
                 } finally {
@@ -749,34 +747,6 @@ public class AuditFileQueueSpool implements Runnable {
         }
     }
 
-    class AuditIndexRecord {
-        String id;
-        String filePath;
-        int linePosition = 0;
-        SPOOL_FILE_STATUS status = SPOOL_FILE_STATUS.write_inprogress;
-        Date fileCreateTime;
-        Date writeCompleteTime;
-        Date doneCompleteTime;
-        Date lastSuccessTime;
-        Date lastFailedTime;
-        int failedAttemptCount = 0;
-        boolean lastAttempt = false;
-
-        @Override
-        public String toString() {
-            return "AuditIndexRecord [id=" + id + ", filePath=" + filePath
-                    + ", linePosition=" + linePosition + ", status=" + status
-                    + ", fileCreateTime=" + fileCreateTime
-                    + ", writeCompleteTime=" + writeCompleteTime
-                    + ", doneCompleteTime=" + doneCompleteTime
-                    + ", lastSuccessTime=" + lastSuccessTime
-                    + ", lastFailedTime=" + lastFailedTime
-                    + ", failedAttemptCount=" + failedAttemptCount
-                    + ", lastAttempt=" + lastAttempt + "]";
-        }
-
-    }
-
     /*
      * (non-Javadoc)
      *
@@ -825,7 +795,7 @@ public class AuditFileQueueSpool implements Runnable {
 
                 boolean isRemoveIndex = false;
                 File consumerFile = new File(
-                        currentConsumerIndexRecord.filePath);
+                        currentConsumerIndexRecord.getFilePath());
                 if (!consumerFile.exists()) {
                     logger.error("Consumer file=" + consumerFile.getPath()
                             + " not found.");
@@ -834,12 +804,12 @@ public class AuditFileQueueSpool implements Runnable {
                 } else {
                     // Let's open the file to write
                     BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(
-                            currentConsumerIndexRecord.filePath),"UTF-8"));
+                            currentConsumerIndexRecord.getFilePath()),"UTF-8"));
                     try {
                         if (auditFileType.equalsIgnoreCase(DEFAULT_AUDIT_FILE_TYPE)) {
                             // if Audit File format is JSON each audit file in the Local Spool Location will be copied
                             // to HDFS location as JSON
-                            File srcFile = new File(currentConsumerIndexRecord.filePath);
+                            File srcFile = new File(currentConsumerIndexRecord.getFilePath());
                             logFile(srcFile);
                         } else {
                             // If Audit File format is ORC, each records in audit files in the Local Spool Location will be
@@ -847,13 +817,13 @@ public class AuditFileQueueSpool implements Runnable {
                             logEvent(br);
                         }
                         logger.info("Done reading file. file="
-                                + currentConsumerIndexRecord.filePath
+                                + currentConsumerIndexRecord.getFilePath()
                                 + ", queueName=" + FILE_QUEUE_PROVIDER_NAME
                                 + ", consumer=" + consumerProvider.getName());
                         // The entire file is read
-                        currentConsumerIndexRecord.status = SPOOL_FILE_STATUS.done;
-                        currentConsumerIndexRecord.doneCompleteTime = new Date();
-                        currentConsumerIndexRecord.lastAttempt = true;
+                        currentConsumerIndexRecord.setStatus(SPOOL_FILE_STATUS.done);
+                        currentConsumerIndexRecord.setDoneCompleteTime(new Date());
+                        currentConsumerIndexRecord.setLastAttempt(true);
 
                         isRemoveIndex = true;
                     } catch (Exception ex) {
@@ -863,9 +833,9 @@ public class AuditFileQueueSpool implements Runnable {
                                 + consumerProvider.getName());
                         lastAttemptTime = System.currentTimeMillis();
                         // Update the index file
-                        currentConsumerIndexRecord.lastFailedTime = new Date();
-                        currentConsumerIndexRecord.failedAttemptCount++;
-                        currentConsumerIndexRecord.lastAttempt = false;
+                        currentConsumerIndexRecord.setLastFailedTime(new Date());
+                        currentConsumerIndexRecord.setFailedAttemptCount(currentConsumerIndexRecord.getFailedAttemptCount() + 1);
+                        currentConsumerIndexRecord.setLastAttempt(false);
                         saveIndexFile();
                     } finally {
                         br.close();
@@ -890,7 +860,7 @@ public class AuditFileQueueSpool implements Runnable {
     private void logEvent(BufferedReader br) throws Exception {
         String line;
         int    currLine  = 0;
-        int    startLine = currentConsumerIndexRecord.linePosition;
+        int    startLine = currentConsumerIndexRecord.getLinePosition();
         List<AuditEventBase> events = new ArrayList<>();
         while ((line = br.readLine()) != null) {
             currLine++;
@@ -931,15 +901,15 @@ public class AuditFileQueueSpool implements Runnable {
                         + consumerProvider.getName());
             } else {
                 // Update index and save
-                indexRecord.linePosition = currLine;
-                indexRecord.status = SPOOL_FILE_STATUS.read_inprogress;
-                indexRecord.lastSuccessTime = new Date();
-                indexRecord.lastAttempt = true;
+                indexRecord.setLinePosition(currLine);
+                indexRecord.setStatus(SPOOL_FILE_STATUS.read_inprogress);
+                indexRecord.setLastSuccessTime(new Date());
+                indexRecord.setLastAttempt(true);
                 saveIndexFile();
 
                 if (isDestDown) {
                     isDestDown = false;
-                    logger.info("Destination up now. " + indexRecord.filePath
+                    logger.info("Destination up now. " + indexRecord.getFilePath()
                             + ", queueName=" + FILE_QUEUE_PROVIDER_NAME
                             + ", consumer=" + consumerProvider.getName());
                 }
@@ -958,7 +928,7 @@ public class AuditFileQueueSpool implements Runnable {
             logger.debug("==> AuditFileQueueSpool.logFile()");
         }
         int    currLine  = 0;
-        int    startLine = currentConsumerIndexRecord.linePosition;
+        int    startLine = currentConsumerIndexRecord.getLinePosition();
 
         if (currLine < startLine) {
             currLine++;
@@ -990,15 +960,15 @@ public class AuditFileQueueSpool implements Runnable {
                         + consumerProvider.getName()+ ", logFile=" + file.getName());
             } else {
                 // Update index and save
-                indexRecord.linePosition = currLine;
-                indexRecord.status = SPOOL_FILE_STATUS.read_inprogress;
-                indexRecord.lastSuccessTime = new Date();
-                indexRecord.lastAttempt = true;
+                indexRecord.setLinePosition(currLine);
+                indexRecord.setStatus(SPOOL_FILE_STATUS.read_inprogress);
+                indexRecord.setLastSuccessTime(new Date());
+                indexRecord.setLastAttempt(true);
                 saveIndexFile();
 
                 if (isDestDown) {
                     isDestDown = false;
-                    logger.info("Destination up now. " + indexRecord.filePath
+                    logger.info("Destination up now. " + indexRecord.getFilePath()
                             + ", queueName=" + FILE_QUEUE_PROVIDER_NAME
                             + ", consumer=" + consumerProvider.getName());
                 }
