@@ -19,12 +19,11 @@
 
 package org.apache.ranger.plugin.contextenricher;
 
-import com.google.gson.Gson;
-import com.google.gson.GsonBuilder;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.ranger.authorization.hadoop.config.RangerPluginConfig;
+import org.apache.ranger.authorization.utils.JsonUtils;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.model.RangerServiceDef;
 import org.apache.ranger.plugin.model.RangerServiceResource;
@@ -76,10 +75,10 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 	private static final Logger PERF_CONTEXTENRICHER_INIT_LOG = RangerPerfTracer.getPerfLogger("contextenricher.init");
 	private static final Logger PERF_TRIE_OP_LOG              = RangerPerfTracer.getPerfLogger("resourcetrie.retrieval");
 	private static final Logger PERF_SET_SERVICETAGS_LOG      = RangerPerfTracer.getPerfLogger("tagenricher.setservicetags");
-
+	private static final Logger PERF_SERVICETAGS_RETRIEVAL_LOG = RangerPerfTracer.getPerfLogger("tagenricher.tags.retrieval");
 
 	private static final String TAG_REFRESHER_POLLINGINTERVAL_OPTION = "tagRefresherPollingInterval";
-	public  static final String TAG_RETRIEVER_CLASSNAME_OPTION       = "tagRetrieverClassName";
+	public static final String TAG_RETRIEVER_CLASSNAME_OPTION        = "tagRetrieverClassName";
 	private static final String TAG_DISABLE_TRIE_PREFILTER_OPTION    = "disableTrieLookupPrefilter";
 
 	private RangerTagRefresher                 tagRefresher;
@@ -105,7 +104,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 
 		String propertyPrefix        = getPropertyPrefix();
 		String tagRetrieverClassName = getOption(TAG_RETRIEVER_CLASSNAME_OPTION);
-		long   pollingIntervalMs     = getLongOption(TAG_REFRESHER_POLLINGINTERVAL_OPTION, 60 * 1000);
+		long   pollingIntervalMs     = getLongOption(TAG_REFRESHER_POLLINGINTERVAL_OPTION, 60 * 1000L);
 
 		dedupStrings               = getBooleanConfig(propertyPrefix + ".dedup.strings", true);
 		disableTrieLookupPrefilter = getBooleanOption(TAG_DISABLE_TRIE_PREFILTER_OPTION, false);
@@ -310,13 +309,18 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				}
 
 				if (!serviceTags.getIsDelta()) {
+					if (serviceTags.getIsTagsDeduped()) {
+						final int countOfDuplicateTags = serviceTags.dedupTags();
+
+						LOG.info("Number of duplicate tags removed from the received serviceTags:[" + countOfDuplicateTags + "]. Number of tags in the de-duplicated serviceTags :[" + serviceTags.getTags().size() + "].");
+					}
 					processServiceTags(serviceTags);
 				} else {
 					if (LOG.isDebugEnabled()) {
 						LOG.debug("Received service-tag deltas:" + serviceTags);
 					}
 					ServiceTags oldServiceTags = enrichedServiceTags != null ? enrichedServiceTags.getServiceTags() : new ServiceTags();
-					ServiceTags allServiceTags = rebuildOnlyIndex ? oldServiceTags : RangerServiceTagsDeltaUtil.applyDelta(oldServiceTags, serviceTags);
+					ServiceTags allServiceTags = rebuildOnlyIndex ? oldServiceTags : RangerServiceTagsDeltaUtil.applyDelta(oldServiceTags, serviceTags, serviceTags.getIsTagsDeduped());
 
 					if (serviceTags.getTagsChangeExtent() == ServiceTags.TagsChangeExtent.NONE) {
 						if (LOG.isDebugEnabled()) {
@@ -385,6 +389,9 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		this.tagRefresher = null;
 
 		if (tagRefresher != null) {
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("Trying to clean up RangerTagRefresher(" + tagRefresher.getName() + ")");
+			}
 			tagRefresher.cleanup();
 		}
 
@@ -473,23 +480,27 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		List<RangerServiceResource> changedServiceResources = deltas.getServiceResources();
 
 		for (RangerServiceResource serviceResource : changedServiceResources) {
-
 			final boolean removedOldServiceResource = MapUtils.isEmpty(serviceResource.getResourceElements()) || removeOldServiceResource(serviceResource, resourceMatchers, serviceResourceTrie);
+
 			if (removedOldServiceResource) {
-
 				if (!StringUtils.isEmpty(serviceResource.getResourceSignature())) {
-
 					RangerServiceResourceMatcher resourceMatcher = createRangerServiceResourceMatcher(serviceResource, serviceDefHelper, hierarchies);
 
 					if (resourceMatcher != null) {
 						for (RangerServiceDef.RangerResourceDef resourceDef : serviceDef.getResources()) {
+							RangerPolicy.RangerPolicyResource                policyResource = serviceResource.getResourceElements().get(resourceDef.getName());
+							RangerResourceTrie<RangerServiceResourceMatcher> trie           = serviceResourceTrie.get(resourceDef.getName());
 
-							RangerResourceTrie<RangerServiceResourceMatcher> trie = serviceResourceTrie.get(resourceDef.getName());
+							if (LOG.isDebugEnabled()) {
+								LOG.debug("Trying to add resource-matcher to " + (trie == null ? "new" : "existing") + " trie for " + resourceDef.getName());
+							}
 
 							if (trie != null) {
-								trie.add(serviceResource.getResourceElements().get(resourceDef.getName()), resourceMatcher);
+								trie.add(policyResource, resourceMatcher);
+								trie.wrapUpUpdate();
+
 								if (LOG.isDebugEnabled()) {
-									LOG.debug("Added resource-matcher for service-resource:[" + serviceResource + "]");
+									LOG.debug("Added resource-matcher for policy-resource:[" + policyResource + "]");
 								}
 							} else {
 								trie = new RangerResourceTrie<>(resourceDef, Collections.singletonList(resourceMatcher), getPolicyEngineOptions().optimizeTagTrieForRetrieval, getPolicyEngineOptions().optimizeTagTrieForSpace, null);
@@ -514,6 +525,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				break;
 			}
 		}
+
 		if (isInError) {
 			LOG.error("Error in processing tag-deltas. Will continue to use old tags");
 			deltas.setTagVersion(-1L);
@@ -523,44 +535,61 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			}
 			enrichedServiceTags = new EnrichedServiceTags(allServiceTags, resourceMatchers, serviceResourceTrie);
 		}
-
 	}
 
 	private boolean removeOldServiceResource(RangerServiceResource serviceResource, List<RangerServiceResourceMatcher> resourceMatchers, Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> resourceTries) {
 		boolean ret = true;
 
 		if (enrichedServiceTags != null) {
-
 			if (LOG.isDebugEnabled()) {
 				LOG.debug("Removing service-resource:[" + serviceResource + "] from trie-map");
 			}
 
-			// Remove existing serviceResource from the copy
-
 			RangerAccessResourceImpl accessResource = new RangerAccessResourceImpl();
 
 			for (Map.Entry<String, RangerPolicy.RangerPolicyResource> entry : serviceResource.getResourceElements().entrySet()) {
-				accessResource.setValue(entry.getKey(), entry.getValue());
+				accessResource.setValue(entry.getKey(), entry.getValue().getValues());
 			}
+
 			if (LOG.isDebugEnabled()) {
 				LOG.debug("RangerAccessResource:[" + accessResource + "] created to represent service-resource[" + serviceResource + "] to find evaluators from trie-map");
 			}
 
-			RangerAccessRequestImpl  request = new RangerAccessRequestImpl();
+			RangerAccessRequestImpl request = new RangerAccessRequestImpl();
 			request.setResource(accessResource);
 
 			Collection<RangerServiceResourceMatcher> oldMatchers = getEvaluators(request, enrichedServiceTags);
 
 			if (LOG.isDebugEnabled()) {
-				LOG.debug("Found [" + oldMatchers.size() + "] matchers for service-resource[" + serviceResource + "]");
+				LOG.debug("Found [" + oldMatchers + "] matchers for service-resource[" + serviceResource + "]");
+			}
+
+			if (CollectionUtils.isNotEmpty(oldMatchers)) {
+				List<RangerServiceResourceMatcher> notMatched = new ArrayList<>();
+
+				for (RangerServiceResourceMatcher resourceMatcher : oldMatchers) {
+					final RangerPolicyResourceMatcher.MatchType matchType = resourceMatcher.getMatchType(accessResource, request.getResourceElementMatchingScopes(), request.getContext());
+
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("resource:[" + accessResource + ", MatchType:[" + matchType + "]");
+					}
+
+					if (matchType != RangerPolicyResourceMatcher.MatchType.SELF) {
+						notMatched.add(resourceMatcher);
+					}
+				}
+
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("oldMatchers : [" + notMatched + "] do not match resource:[" + accessResource + "] exactly and will be discarded");
+				}
+
+				oldMatchers.removeAll(notMatched);
 			}
 
 			for (RangerServiceResourceMatcher matcher : oldMatchers) {
-
 				for (RangerServiceDef.RangerResourceDef resourceDef : serviceDef.getResources()) {
-					String resourceDefName = resourceDef.getName();
-
-					RangerResourceTrie<RangerServiceResourceMatcher> trie = resourceTries.get(resourceDefName);
+					String                                           resourceDefName = resourceDef.getName();
+					RangerResourceTrie<RangerServiceResourceMatcher> trie            = resourceTries.get(resourceDefName);
 
 					if (trie != null) {
 						trie.delete(serviceResource.getResourceElements().get(resourceDefName), matcher);
@@ -573,12 +602,11 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				}
 			}
 
-			// Remove old resource matchers
 			if (ret) {
 				resourceMatchers.removeAll(oldMatchers);
 
 				if (LOG.isDebugEnabled()) {
-					LOG.debug("Found and removed [" + oldMatchers.size() + "] matchers for service-resource[" + serviceResource + "] from trie-map");
+					LOG.debug("Found and removed [" + oldMatchers + "] matchers for service-resource[" + serviceResource + "] from trie-map");
 				}
 			}
 		}
@@ -666,6 +694,12 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 
 		RangerAccessResource resource = request.getResource();
 
+		RangerPerfTracer perf = null;
+
+		if (RangerPerfTracer.isPerfTraceEnabled(PERF_SERVICETAGS_RETRIEVAL_LOG)) {
+			perf = RangerPerfTracer.getPerfTracer(PERF_SERVICETAGS_RETRIEVAL_LOG, "RangerTagEnricher.findMatchingTags=" + resource.getAsString() + ")");
+		}
+
 		if ((resource == null || resource.getKeys() == null || resource.getKeys().isEmpty()) && request.isAccessTypeAny()) {
 			ret = enrichedServiceTags.getTagsForEmptyResourceAndAnyAccess();
 		} else {
@@ -673,10 +707,13 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			final Collection<RangerServiceResourceMatcher> serviceResourceMatchers = getEvaluators(request, enrichedServiceTags);
 
 			if (CollectionUtils.isNotEmpty(serviceResourceMatchers)) {
-
 				for (RangerServiceResourceMatcher resourceMatcher : serviceResourceMatchers) {
 
-					final RangerPolicyResourceMatcher.MatchType matchType = resourceMatcher.getMatchType(resource, request.getContext());
+					final RangerPolicyResourceMatcher.MatchType matchType = resourceMatcher.getMatchType(resource, request.getResourceElementMatchingScopes(), request.getContext());
+
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("resource:[" + resource + ", MatchType:[" + matchType + "]");
+					}
 
 					final boolean isMatched;
 
@@ -685,7 +722,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 					} else if (request.getResourceMatchingScope() == RangerAccessRequest.ResourceMatchingScope.SELF_OR_DESCENDANTS) {
 						isMatched = matchType != RangerPolicyResourceMatcher.MatchType.NONE;
 					} else {
-						isMatched = matchType == RangerPolicyResourceMatcher.MatchType.SELF || matchType == RangerPolicyResourceMatcher.MatchType.ANCESTOR;
+						isMatched = matchType == RangerPolicyResourceMatcher.MatchType.SELF || matchType == RangerPolicyResourceMatcher.MatchType.SELF_AND_ALL_DESCENDANTS || matchType == RangerPolicyResourceMatcher.MatchType.ANCESTOR;
 					}
 
 					if (isMatched) {
@@ -699,6 +736,8 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			}
 		}
 
+		RangerPerfTracer.logAlways(perf);
+
 		if (CollectionUtils.isEmpty(ret)) {
 			if (LOG.isDebugEnabled()) {
 				LOG.debug("RangerTagEnricher.findMatchingTags(" + resource + ") - No tags Found ");
@@ -735,13 +774,16 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				perf = RangerPerfTracer.getPerfTracer(PERF_TRIE_OP_LOG, "RangerTagEnricher.getEvaluators(resource=" + resource.getAsString() + ")");
 			}
 
-			ret = RangerResourceEvaluatorsRetriever.getEvaluators(serviceResourceTrie, resource.getAsMap(), request.getResourceMatchingScope());
+			ret = RangerResourceEvaluatorsRetriever.getEvaluators(serviceResourceTrie, resource.getAsMap(), request.getResourceElementMatchingScopes());
 
 			RangerPerfTracer.logAlways(perf);
 		}
+		if (ret == null) {
+			ret = new ArrayList<>();
+		}
 
 		if(LOG.isDebugEnabled()) {
-			LOG.debug("<== RangerTagEnricher.getEvaluators(request=" + request + "): evaluatorCount=" + ret.size());
+			LOG.debug("<== RangerTagEnricher.getEvaluators(request=" + request + "): evaluators=" + ret);
 		}
 
 		return ret;
@@ -842,8 +884,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		private long lastActivationTimeInMillis;
 
 		private final String cacheFile;
-		private boolean hasProvidedTagsToReceiver;
-		private Gson gson;
+		private boolean      hasProvidedTagsToReceiver;
 
 		RangerTagRefresher(RangerTagRetriever tagRetriever, RangerTagEnricher tagEnricher, long lastKnownVersion, BlockingQueue<DownloadTrigger> tagDownloadQueue, String cacheFile) {
 			this.tagRetriever = tagRetriever;
@@ -851,11 +892,6 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			this.lastKnownVersion = lastKnownVersion;
 			this.tagDownloadQueue = tagDownloadQueue;
 			this.cacheFile = cacheFile;
-			try {
-				gson = new GsonBuilder().setDateFormat("yyyyMMdd-HH:mm:ss.SSS-Z").create();
-			} catch(Throwable excp) {
-				LOG.error("failed to create GsonBuilder object", excp);
-			}
 			setName("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ")-" + getId());
 		}
 
@@ -987,6 +1023,9 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 					try {
 						super.join();
 						isJoined = true;
+						if (LOG.isDebugEnabled()) {
+							LOG.debug("RangerTagRefresher(" + getName() + ") is stopped");
+						}
 					} catch (InterruptedException excp) {
 						LOG.warn("RangerTagRefresher(" + getName() + ").stopRefresher(): Error while waiting for thread to exit", excp);
 						LOG.warn("Retrying Thread.join(). Current thread will be marked as 'interrupted' after Thread.join() returns");
@@ -1015,7 +1054,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				try {
 					reader = new FileReader(cacheFile);
 
-					serviceTags = gson.fromJson(reader, ServiceTags.class);
+						serviceTags = JsonUtils.jsonToObject(reader, ServiceTags.class);
 
 					if (serviceTags != null && !StringUtils.equals(tagEnricher.getServiceName(), serviceTags.getServiceName())) {
 						LOG.warn("ignoring unexpected serviceName '" + serviceTags.getServiceName() + "' in cache file '" + cacheFile.getAbsolutePath() + "'");
@@ -1058,7 +1097,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 					try {
 						writer = new FileWriter(cacheFile);
 
-						gson.toJson(serviceTags, writer);
+						JsonUtils.objectToWriter(writer, serviceTags);
 					} catch (Exception excp) {
 						LOG.error("failed to save service-tags to cache file '" + cacheFile.getAbsolutePath() + "'", excp);
 					} finally {
