@@ -19,6 +19,7 @@
 
 package org.apache.ranger.plugin.policyengine;
 
+import java.util.Collections;
 import java.util.Date;
 import java.util.List;
 import java.util.Map;
@@ -65,5 +66,13 @@ public interface RangerAccessRequest {
 
 	ResourceMatchingScope getResourceMatchingScope();
 
-	enum ResourceMatchingScope {SELF, SELF_OR_DESCENDANTS}
+	default Map<String, ResourceElementMatchingScope> getResourceElementMatchingScopes() {
+		return Collections.emptyMap();
+	}
+
+	enum ResourceMatchingScope { SELF, SELF_OR_DESCENDANTS }
+
+	enum ResourceElementMatchingScope { SELF, SELF_OR_CHILD, SELF_OR_PREFIX }
+
+	enum ResourceElementMatchType { NONE, SELF, CHILD, PREFIX }
 }
