@@ -567,6 +567,22 @@ public class RangerPolicyRepository {
         return policyEvaluators;
     }
 
+    public int getPolicyEvaluatorCount() {
+        return policyEvaluators.size();
+    }
+
+    public int getDataMaskPolicyEvaluatorCount() {
+        return dataMaskPolicyEvaluators.size();
+    }
+
+    public int getRowFilterPolicyEvaluatorCount() {
+        return rowFilterPolicyEvaluators.size();
+    }
+
+    public int getAuditPolicyEvaluatorCount() {
+        return auditPolicyEvaluators.size();
+    }
+
     List<RangerPolicyEvaluator> getDataMaskPolicyEvaluators() {
         return dataMaskPolicyEvaluators;
     }
@@ -616,7 +632,7 @@ public class RangerPolicyRepository {
                         Collections.sort(ret, PolicyEvaluatorForTag.EVAL_ORDER_COMPARATOR);
                         break;
                     case RangerPolicy.POLICY_TYPE_DATAMASK:
-                        Collections.sort(ret, PolicyEvaluatorForTag.NAME_COMPARATOR);
+                        Collections.sort(ret, PolicyEvaluatorForTag.MATCH_TYPE_COMPARATOR);
                         break;
                     case RangerPolicy.POLICY_TYPE_ROWFILTER:
                         Collections.sort(ret, PolicyEvaluatorForTag.NAME_COMPARATOR);
@@ -703,7 +719,7 @@ public class RangerPolicyRepository {
             perf = RangerPerfTracer.getPerfTracer(PERF_TRIE_OP_LOG, "RangerPolicyRepository.getLikelyMatchEvaluators(resource=" + resource.getAsString() + ")");
         }
 
-        Collection<RangerPolicyResourceEvaluator> smallestList = RangerResourceEvaluatorsRetriever.getEvaluators(resourceTrie, resource.getAsMap(), request.getResourceMatchingScope());
+        Collection<RangerPolicyResourceEvaluator> smallestList = RangerResourceEvaluatorsRetriever.getEvaluators(resourceTrie, resource.getAsMap(), request.getResourceElementMatchingScopes());
 
         if (smallestList != null) {
             if (smallestList.size() == 0) {
@@ -915,14 +931,13 @@ public class RangerPolicyRepository {
             }
         }
         LOG.info("This policy engine contains " + (policyEvaluators.size()+dataMaskPolicyEvaluators.size()+rowFilterPolicyEvaluators.size()) + " policy evaluators");
-        RangerPolicyEvaluator.PolicyEvalOrderComparator comparator = new RangerPolicyEvaluator.PolicyEvalOrderComparator();
-        Collections.sort(policyEvaluators, comparator);
+        Collections.sort(policyEvaluators, RangerPolicyEvaluator.EVAL_ORDER_COMPARATOR);
         this.policyEvaluators = policyEvaluators;
 
-        Collections.sort(dataMaskPolicyEvaluators, comparator);
+        Collections.sort(dataMaskPolicyEvaluators, RangerPolicyEvaluator.NAME_COMPARATOR);
         this.dataMaskPolicyEvaluators = dataMaskPolicyEvaluators;
 
-        Collections.sort(rowFilterPolicyEvaluators, comparator);
+        Collections.sort(rowFilterPolicyEvaluators, RangerPolicyEvaluator.NAME_COMPARATOR);
         this.rowFilterPolicyEvaluators = rowFilterPolicyEvaluators;
 
         this.policyEvaluatorsMap = createPolicyEvaluatorsMap();
@@ -1159,7 +1174,7 @@ public class RangerPolicyRepository {
 
         if (CollectionUtils.isNotEmpty(evaluators)) {
             ret = new ArrayList<>(evaluators);
-            Collections.sort(ret, new RangerPolicyEvaluator.PolicyEvalOrderComparator());
+            Collections.sort(ret, RangerPolicyEvaluator.EVAL_ORDER_COMPARATOR);
         }
 
         return ret;
