@@ -33,6 +33,7 @@ import org.apache.ranger.authorization.hadoop.config.RangerPluginConfig;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.apache.ranger.plugin.contextenricher.RangerAdminUserStoreRetriever;
 import org.apache.ranger.plugin.contextenricher.RangerUserStoreEnricher;
+import org.apache.ranger.plugin.model.RangerBaseModelObject;
 import org.apache.ranger.plugin.policyengine.RangerRequestScriptEvaluator;
 import org.apache.ranger.plugin.contextenricher.RangerContextEnricher;
 import org.apache.ranger.plugin.contextenricher.RangerTagEnricher;
@@ -70,7 +71,6 @@ public class RangerBasePlugin {
 	private       RangerAccessResultProcessor resultProcessor;
 	private       RangerRoles                 roles;
 	private final List<RangerChainedPlugin>   chainedPlugins;
-	private final boolean                     enableImplicitUserStoreEnricher;
 	private final boolean                     dedupStrings;
 	private       boolean                     isUserStoreEnricherAddedImplcitly = false;
 
@@ -87,6 +87,17 @@ public class RangerBasePlugin {
 		this.pluginConfig  = pluginConfig;
 		this.pluginContext = new RangerPluginContext(pluginConfig);
 
+		boolean usePerfDataRecorder  = pluginConfig.getBoolean("ranger.perf.aggregate.data", false);
+		int     perfDataDumpInterval = pluginConfig.getInt("ranger.perf.aggregate.data.dump.interval", 0);
+		boolean usePerfDataLock      = pluginConfig.getBoolean("ranger.perf.aggregate.data.lock.enabled", false);
+		String  nullSafeSupplier     = pluginConfig.get(pluginConfig.getPropertyPrefix() + ".null_safe.supplier", RangerBaseModelObject.NULL_SAFE_SUPPLIER_V2);
+
+		LOG.info(pluginConfig.getPropertyPrefix() + ".null_safe.supplier=" + nullSafeSupplier);
+
+		RangerBaseModelObject.setNullSafeSupplier(nullSafeSupplier);
+
+		PerfDataRecorder.initialize(usePerfDataRecorder, perfDataDumpInterval, usePerfDataLock, null);
+
 		Set<String> superUsers         = toSet(pluginConfig.get(pluginConfig.getPropertyPrefix() + ".super.users"));
 		Set<String> superGroups        = toSet(pluginConfig.get(pluginConfig.getPropertyPrefix() + ".super.groups"));
 		Set<String> auditExcludeUsers  = toSet(pluginConfig.get(pluginConfig.getPropertyPrefix() + ".audit.exclude.users"));
@@ -101,9 +112,7 @@ public class RangerBasePlugin {
 
 		RangerRequestScriptEvaluator.init(pluginConfig);
 
-		this.enableImplicitUserStoreEnricher = pluginConfig.getBoolean(pluginConfig.getPropertyPrefix() + ".enable.implicit.userstore.enricher", false);
-		this.dedupStrings                    = pluginConfig.getBoolean(pluginConfig.getPropertyPrefix() + ".dedup.strings", true);
-
+		this.dedupStrings   = pluginConfig.getBoolean(pluginConfig.getPropertyPrefix() + ".dedup.strings", true);
 		this.chainedPlugins = initChainedPlugins();
 	}
 
@@ -283,13 +292,15 @@ public class RangerBasePlugin {
 			LOG.debug("==> setPolicies(" + policies + ")");
 		}
 
-		if (enableImplicitUserStoreEnricher && policies != null && !ServiceDefUtil.isUserStoreEnricherPresent(policies)) {
-			String retrieverClassName = getConfig().get(RangerUserStoreEnricher.USERSTORE_RETRIEVER_CLASSNAME_OPTION, RangerAdminUserStoreRetriever.class.getCanonicalName());
-			String retrieverPollIntMs = getConfig().get(RangerUserStoreEnricher.USERSTORE_REFRESHER_POLLINGINTERVAL_OPTION, Integer.toString(60 * 1000));
+		if (pluginConfig.isEnableImplicitUserStoreEnricher() && policies != null && !ServiceDefUtil.isUserStoreEnricherPresent(policies)) {
+			String retrieverClassName = pluginConfig.get(RangerUserStoreEnricher.USERSTORE_RETRIEVER_CLASSNAME_OPTION, RangerAdminUserStoreRetriever.class.getCanonicalName());
+			String retrieverPollIntMs = pluginConfig.get(RangerUserStoreEnricher.USERSTORE_REFRESHER_POLLINGINTERVAL_OPTION, Integer.toString(60 * 1000));
 
 			// in case of delta, policies will only have changes; hence add userStoreEnricher if it was implicitly added previous calls to setPolicies()
 			if (RangerPolicyDeltaUtil.hasPolicyDeltas(policies) == Boolean.TRUE && isUserStoreEnricherAddedImplcitly) {
 				ServiceDefUtil.addUserStoreEnricher(policies, retrieverClassName, retrieverPollIntMs);
+			} else if (pluginConfig.isUseRangerGroups() || pluginConfig.isConvertEmailToUsername()) {
+				isUserStoreEnricherAddedImplcitly = ServiceDefUtil.addUserStoreEnricher(policies, retrieverClassName, retrieverPollIntMs);
 			} else {
 				isUserStoreEnricherAddedImplcitly = ServiceDefUtil.addUserStoreEnricherIfNeeded(policies, retrieverClassName, retrieverPollIntMs);
 			}
@@ -354,6 +365,13 @@ public class RangerBasePlugin {
 						if (policies.getPolicies() == null) {
 							policies.setPolicies(new ArrayList<>());
 						}
+						if (MapUtils.isNotEmpty(policies.getSecurityZones())) {
+							for (ServicePolicies.SecurityZoneInfo element : policies.getSecurityZones().values()) {
+								if (element.getPolicies() == null) {
+									element.setPolicies(new ArrayList<>());
+								}
+							}
+						}
 					}
 				}
 			}
@@ -411,7 +429,9 @@ public class RangerBasePlugin {
 						newPolicyEngine.setTrustedProxyAddresses(pluginConfig.getTrustedProxyAddresses());
 					}
 
+					LOG.info("Switching policy engine from [" + getPolicyVersion() + "]");
 					this.policyEngine       = newPolicyEngine;
+					LOG.info("Switched policy engine to [" + getPolicyVersion() + "]");
 					this.currentAuthContext = pluginContext.getAuthContext();
 
 					pluginContext.notifyAuthContextChanged();
@@ -421,7 +441,17 @@ public class RangerBasePlugin {
 					}
 
 					if (this.refresher != null) {
-						this.refresher.saveToCache(usePolicyDeltas ? servicePolicies : policies);
+						boolean doPreserveDeltas = pluginConfig.getBoolean (pluginConfig.getPropertyPrefix() + ".preserve.deltas", false);
+						if (!doPreserveDeltas) {
+							this.refresher.saveToCache(usePolicyDeltas ? servicePolicies : policies);
+						} else {
+							// Save both deltas and all policies to cache for verification
+							this.refresher.saveToCache(policies);
+
+							if (usePolicyDeltas) {
+								this.refresher.saveToCache(servicePolicies);
+							}
+						}
 					}
 				}
 
@@ -479,13 +509,19 @@ public class RangerBasePlugin {
 			ret = policyEngine.evaluatePolicies(request, RangerPolicy.POLICY_TYPE_ACCESS, null);
 		}
 
-		if (ret != null) {
+		if (ret != null && !RangerAccessRequestUtil.getIsSkipChainedPlugins(request.getContext())) {
 			for (RangerChainedPlugin chainedPlugin : chainedPlugins) {
 				if (LOG.isDebugEnabled()) {
 					LOG.debug("BasePlugin.isAccessAllowed result=[" + ret + "]");
 					LOG.debug("Calling chainedPlugin.isAccessAllowed for service:[" + chainedPlugin.plugin.pluginConfig.getServiceName() + "]");
 				}
-				RangerAccessResult chainedResult = chainedPlugin.isAccessAllowed(request);
+				RangerAccessResult chainedResult;
+
+				if (ret.getIsAccessDetermined() && chainedPlugin.skipAccessCheckIfAlreadyDetermined) {
+					chainedResult = null;
+				} else {
+					chainedResult = chainedPlugin.isAccessAllowed(request);
+				}
 
 				if (chainedResult != null) {
 					if (LOG.isDebugEnabled()) {
@@ -507,7 +543,6 @@ public class RangerBasePlugin {
 		if (resultProcessor != null) {
 			resultProcessor.processResult(ret);
 		}
-
 		return ret;
 	}
 
@@ -1137,6 +1172,10 @@ public class RangerBasePlugin {
 		return baseACLs;
 	}
 
+	protected RangerPolicyEngine getPolicyEngine() {
+		return policyEngine;
+	}
+
 	private RangerAdminClient getAdminClient() throws Exception {
 		PolicyRefresher   refresher = this.refresher;
 		RangerAdminClient admin     = refresher == null ? null : refresher.getRangerAdminClient();
@@ -1183,12 +1222,12 @@ public class RangerBasePlugin {
 		int     policyType     = result.getPolicyType();
 
 		if (chainedResult.getIsAccessDetermined()) { // only if chained-result is definitive
-			// override if result is not definitive or chained-result is by a higher priority policy
-			overrideResult = !result.getIsAccessDetermined() || chainedResult.getPolicyPriority() > result.getPolicyPriority();
+			// override if chained-result is by a higher priority policy or result is not definitive or the result is not-allowed and no matching Ranger policy found
+			overrideResult = chainedResult.getPolicyPriority() > result.getPolicyPriority() || !result.getIsAccessDetermined() || (!result.getIsAllowed() && result.getPolicyId() == -1L);
 
 			if (!overrideResult) {
-				// override if chained-result is from the same policy priority, and if denies access
-				if (chainedResult.getPolicyPriority() == result.getPolicyPriority() && !chainedResult.getIsAllowed()) {
+				// override if chained-result is from the same policy priority, and if denies access with a specific policy id
+				if (chainedResult.getPolicyPriority() == result.getPolicyPriority() && (!chainedResult.getIsAllowed() && chainedResult.getPolicyId() != -1L)) {
 					// let's not override if result is already denied
 					if (result.getIsAllowed()) {
 						overrideResult = true;
@@ -1314,4 +1353,8 @@ public class RangerBasePlugin {
 
 		return ret;
 	}
+
+	public Long getPolicyVersion() {
+		return this.policyEngine == null ? -1L : this.policyEngine.getPolicyVersion();
+	}
 }
