@@ -23,15 +23,19 @@ import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.ranger.plugin.conditionevaluator.RangerScriptConditionEvaluator;
 import org.apache.ranger.plugin.contextenricher.RangerUserStoreEnricher;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItem;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItemCondition;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItemRowFilterInfo;
+import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItemDataMaskInfo;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyResource;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerRowFilterPolicyItem;
+import org.apache.ranger.plugin.model.RangerPolicy.RangerDataMaskPolicyItem;
 import org.apache.ranger.plugin.model.RangerPolicyDelta;
 import org.apache.ranger.plugin.model.RangerServiceDef;
+import org.apache.ranger.plugin.model.RangerServiceDef.RangerPolicyConditionDef;
 import org.apache.ranger.plugin.model.RangerServiceDef.RangerAccessTypeDef;
 import org.apache.ranger.plugin.model.RangerServiceDef.RangerContextEnricherDef;
 import org.apache.ranger.plugin.model.RangerServiceDef.RangerDataMaskTypeDef;
@@ -48,15 +52,44 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Set;
 
 public class ServiceDefUtil {
     private static final Logger LOG = LoggerFactory.getLogger(ServiceDefUtil.class);
 
+    public static final String IMPLICIT_CONDITION_EXPRESSION_EVALUATOR = RangerScriptConditionEvaluator.class.getCanonicalName();
+    public static final String IMPLICIT_CONDITION_EXPRESSION_NAME      = "_expression";
+    public static final String IMPLICIT_CONDITION_EXPRESSION_LABEL     = "Enter boolean expression";
+    public static final String IMPLICIT_CONDITION_EXPRESSION_DESC      = "Boolean expression";
+
     private static final String USER_STORE_ENRICHER = RangerUserStoreEnricher.class.getCanonicalName();
 
+
+    public static final String ACCESS_TYPE_MARKER_CREATE = "_CREATE";
+    public static final String ACCESS_TYPE_MARKER_READ   = "_READ";
+    public static final String ACCESS_TYPE_MARKER_UPDATE = "_UPDATE";
+    public static final String ACCESS_TYPE_MARKER_DELETE = "_DELETE";
+    public static final String ACCESS_TYPE_MARKER_MANAGE = "_MANAGE";
+    public static final String ACCESS_TYPE_MARKER_ALL    = "_ALL";
+    public static final Set<String> ACCESS_TYPE_MARKERS;
+
+    static {
+        Set<String> typeMarkers = new LinkedHashSet<>();
+
+        typeMarkers.add(ACCESS_TYPE_MARKER_CREATE);
+        typeMarkers.add(ACCESS_TYPE_MARKER_READ);
+        typeMarkers.add(ACCESS_TYPE_MARKER_UPDATE);
+        typeMarkers.add(ACCESS_TYPE_MARKER_DELETE);
+        typeMarkers.add(ACCESS_TYPE_MARKER_MANAGE);
+        typeMarkers.add(ACCESS_TYPE_MARKER_ALL);
+
+        ACCESS_TYPE_MARKERS = Collections.unmodifiableSet(typeMarkers);
+    }
+
     public static boolean getOption_enableDenyAndExceptionsInPolicies(RangerServiceDef serviceDef, RangerPluginContext pluginContext) {
         boolean ret = false;
 
@@ -71,6 +104,17 @@ public class ServiceDefUtil {
         return ret;
     }
 
+    public static boolean getOption_enableTagBasedPolicies(RangerServiceDef serviceDef, Configuration config) {
+        boolean ret = false;
+
+        if(serviceDef != null) {
+            boolean defaultValue = config == null || config.getBoolean("ranger.servicedef.enableTagBasedPolicies", true);
+            ret = ServiceDefUtil.getBooleanValue(serviceDef.getOptions(), RangerServiceDef.OPTION_ENABLE_TAG_BASED_POLICIES, defaultValue);
+        }
+
+        return ret;
+    }
+
     public static RangerDataMaskTypeDef getDataMaskType(RangerServiceDef serviceDef, String typeName) {
         RangerDataMaskTypeDef ret = null;
 
@@ -97,6 +141,22 @@ public class ServiceDefUtil {
         return serviceDef;
     }
 
+    public static RangerPolicyConditionDef getConditionDef(RangerServiceDef serviceDef, String conditionName) {
+        RangerPolicyConditionDef ret = null;
+
+        if (serviceDef != null && serviceDef.getPolicyConditions() != null) {
+            for (RangerPolicyConditionDef conditionDef : serviceDef.getPolicyConditions()) {
+                if (StringUtils.equals(conditionDef.getName(), conditionName)) {
+                    ret = conditionDef;
+
+                    break;
+                }
+            }
+        }
+
+        return ret;
+    }
+
     public static RangerResourceDef getResourceDef(RangerServiceDef serviceDef, String resource) {
         RangerResourceDef ret = null;
 
@@ -204,67 +264,69 @@ public class ServiceDefUtil {
     }
 
     public static RangerServiceDef normalizeAccessTypeDefs(RangerServiceDef serviceDef, final String componentType) {
-
         if (serviceDef != null && StringUtils.isNotBlank(componentType)) {
+            normalizeAccessTypeDefs(serviceDef.getAccessTypes(), componentType);
+            normalizeAccessTypeDefs(serviceDef.getMarkerAccessTypes(), componentType);
 
-            List<RangerServiceDef.RangerAccessTypeDef> accessTypeDefs = serviceDef.getAccessTypes();
+            if (serviceDef.getDataMaskDef() != null) {
+                normalizeAccessTypeDefs(serviceDef.getDataMaskDef().getAccessTypes(), componentType);
+            }
 
-            if (CollectionUtils.isNotEmpty(accessTypeDefs)) {
-
-                String prefix = componentType + AbstractServiceStore.COMPONENT_ACCESSTYPE_SEPARATOR;
-
-                List<RangerServiceDef.RangerAccessTypeDef> unneededAccessTypeDefs = null;
-
-                for (RangerServiceDef.RangerAccessTypeDef accessTypeDef : accessTypeDefs) {
-
-                    String accessType = accessTypeDef.getName();
-
-                    if (StringUtils.startsWith(accessType, prefix)) {
-
-                        String newAccessType = StringUtils.removeStart(accessType, prefix);
-
-                        accessTypeDef.setName(newAccessType);
-
-                        Collection<String> impliedGrants = accessTypeDef.getImpliedGrants();
-
-                        if (CollectionUtils.isNotEmpty(impliedGrants)) {
-
-                            Collection<String> newImpliedGrants = null;
-
-                            for (String impliedGrant : impliedGrants) {
-
-                                if (StringUtils.startsWith(impliedGrant, prefix)) {
-
-                                    String newImpliedGrant = StringUtils.removeStart(impliedGrant, prefix);
-
-                                    if (newImpliedGrants == null) {
-                                        newImpliedGrants = new ArrayList<>();
-                                    }
-
-                                    newImpliedGrants.add(newImpliedGrant);
-                                }
-                            }
-                            accessTypeDef.setImpliedGrants(newImpliedGrants);
-
-                        }
-                    } else if (StringUtils.contains(accessType, AbstractServiceStore.COMPONENT_ACCESSTYPE_SEPARATOR)) {
-                        if(unneededAccessTypeDefs == null) {
-                            unneededAccessTypeDefs = new ArrayList<>();
-                        }
-
-                        unneededAccessTypeDefs.add(accessTypeDef);
-                    }
-                }
-
-                if(unneededAccessTypeDefs != null) {
-                    accessTypeDefs.removeAll(unneededAccessTypeDefs);
-                }
+            if (serviceDef.getRowFilterDef() != null) {
+                normalizeAccessTypeDefs(serviceDef.getRowFilterDef().getAccessTypes(), componentType);
             }
         }
 
         return serviceDef;
     }
 
+    private static void normalizeAccessTypeDefs(List<RangerAccessTypeDef> accessTypeDefs, String componentType) {
+        if (CollectionUtils.isNotEmpty(accessTypeDefs)) {
+            String                    prefix                 = componentType + AbstractServiceStore.COMPONENT_ACCESSTYPE_SEPARATOR;
+            List<RangerAccessTypeDef> unneededAccessTypeDefs = null;
+
+            for (RangerAccessTypeDef accessTypeDef : accessTypeDefs) {
+                String accessType = accessTypeDef.getName();
+
+                if (StringUtils.startsWith(accessType, prefix)) {
+                    String newAccessType = StringUtils.removeStart(accessType, prefix);
+
+                    accessTypeDef.setName(newAccessType);
+                } else if (StringUtils.contains(accessType, AbstractServiceStore.COMPONENT_ACCESSTYPE_SEPARATOR)) {
+                    if (unneededAccessTypeDefs == null) {
+                        unneededAccessTypeDefs = new ArrayList<>();
+                    }
+
+                    unneededAccessTypeDefs.add(accessTypeDef);
+
+                    continue;
+                }
+
+                Collection<String> impliedGrants = accessTypeDef.getImpliedGrants();
+
+                if (CollectionUtils.isNotEmpty(impliedGrants)) {
+                    Set<String> newImpliedGrants = new HashSet<>();
+
+                    for (String impliedGrant : impliedGrants) {
+                        if (StringUtils.startsWith(impliedGrant, prefix)) {
+                            String newImpliedGrant = StringUtils.removeStart(impliedGrant, prefix);
+
+                            newImpliedGrants.add(newImpliedGrant);
+                        } else if (!StringUtils.contains(impliedGrant, AbstractServiceStore.COMPONENT_ACCESSTYPE_SEPARATOR)) {
+                            newImpliedGrants.add(impliedGrant);
+                        }
+                    }
+
+                    accessTypeDef.setImpliedGrants(newImpliedGrants);
+                }
+            }
+
+            if (unneededAccessTypeDefs != null) {
+                accessTypeDefs.removeAll(unneededAccessTypeDefs);
+            }
+        }
+    }
+
     private static void normalizeDataMaskDef(RangerServiceDef serviceDef) {
         if(serviceDef != null && serviceDef.getDataMaskDef() != null) {
             List<RangerResourceDef>   dataMaskResources   = serviceDef.getDataMaskDef().getResources();
@@ -533,6 +595,8 @@ public class ServiceDefUtil {
 
             serviceDef.setContextEnrichers(enricherDefs);
 
+            ret = true;
+
             LOG.info("addUserStoreEnricher(serviceName={}): added userStoreEnricher {}", policies.getServiceName(), userStoreEnricher);
         }
 
@@ -585,6 +649,102 @@ public class ServiceDefUtil {
         return ret;
     }
 
+    public static List<RangerAccessTypeDef> getMarkerAccessTypes(List<RangerAccessTypeDef> accessTypeDefs) {
+        List<RangerAccessTypeDef> ret              = new ArrayList<>();
+        Map<String, Set<String>>  markerTypeGrants = getMarkerAccessTypeGrants(accessTypeDefs);
+        long                      maxItemId        = getMaxItemId(accessTypeDefs);
+
+        for (String accessTypeMarker : ACCESS_TYPE_MARKERS) {
+            RangerAccessTypeDef accessTypeDef = new RangerAccessTypeDef(++maxItemId, accessTypeMarker, accessTypeMarker, null, markerTypeGrants.get(accessTypeMarker));
+
+            ret.add(accessTypeDef);
+        }
+
+        return ret;
+    }
+
+    public static RangerPolicyConditionDef createImplicitExpressionConditionDef(Long itemId) {
+        RangerPolicyConditionDef ret = new RangerPolicyConditionDef(itemId, IMPLICIT_CONDITION_EXPRESSION_NAME, IMPLICIT_CONDITION_EXPRESSION_EVALUATOR, new HashMap<>());
+
+        ret.getEvaluatorOptions().put("engineName", "JavaScript");
+        ret.getEvaluatorOptions().put("ui.isMultiline", "true");
+        ret.setLabel(IMPLICIT_CONDITION_EXPRESSION_LABEL);
+        ret.setDescription(IMPLICIT_CONDITION_EXPRESSION_DESC);
+        ret.setUiHint("{ \"isMultiline\":true }");
+
+        return ret;
+    }
+
+    private static Map<String, Set<String>> getMarkerAccessTypeGrants(List<RangerAccessTypeDef> accessTypeDefs) {
+        Map<String, Set<String>> ret = new HashMap<>();
+
+        for (String accessTypeMarker : ACCESS_TYPE_MARKERS) {
+            ret.put(accessTypeMarker, new HashSet<>());
+        }
+
+        if (CollectionUtils.isNotEmpty(accessTypeDefs)) {
+            for (RangerAccessTypeDef accessTypeDef : accessTypeDefs) {
+                if (accessTypeDef == null || StringUtils.isBlank(accessTypeDef.getName()) || ACCESS_TYPE_MARKERS.contains(accessTypeDef.getName())) {
+                    continue;
+                }
+
+                addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_ALL));
+
+                if (accessTypeDef.getCategory() == null) {
+                    continue;
+                } else if (accessTypeDef.getCategory() == RangerAccessTypeDef.AccessTypeCategory.CREATE) {
+                    addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_CREATE));
+                } else if (accessTypeDef.getCategory() == RangerAccessTypeDef.AccessTypeCategory.READ) {
+                    addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_READ));
+                } else if (accessTypeDef.getCategory() == RangerAccessTypeDef.AccessTypeCategory.UPDATE) {
+                    addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_UPDATE));
+                } else if (accessTypeDef.getCategory() == RangerAccessTypeDef.AccessTypeCategory.DELETE) {
+                    addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_DELETE));
+                } else if (accessTypeDef.getCategory() == RangerAccessTypeDef.AccessTypeCategory.MANAGE) {
+                    addToMarkerGrants(accessTypeDef, ret.get(ACCESS_TYPE_MARKER_MANAGE));
+                }
+            }
+        }
+
+        return ret;
+    }
+
+    private static void addToMarkerGrants(RangerAccessTypeDef accessTypeDef, Set<String> markerGrants) {
+        markerGrants.add(accessTypeDef.getName());
+
+        if (CollectionUtils.isNotEmpty(accessTypeDef.getImpliedGrants())) {
+            markerGrants.addAll(accessTypeDef.getImpliedGrants());
+        }
+    }
+
+    private static long getMaxItemId(List<RangerAccessTypeDef> accessTypeDefs) {
+        long ret = -1;
+
+        if (CollectionUtils.isNotEmpty(accessTypeDefs)) {
+            for (RangerAccessTypeDef accessTypeDef : accessTypeDefs) {
+                if (accessTypeDef.getItemId() != null && ret < accessTypeDef.getItemId()) {
+                    ret = accessTypeDef.getItemId();
+                }
+            }
+        }
+
+        return ret;
+    }
+
+    public static long getConditionsMaxItemId(List<RangerPolicyConditionDef> conditions) {
+        long ret = 0;
+
+        if (conditions != null) {
+            for (RangerPolicyConditionDef condition : conditions) {
+                if (condition != null && condition.getItemId() != null && ret < condition.getItemId()) {
+                    ret = condition.getItemId();
+                }
+            }
+        }
+
+        return ret;
+    }
+
     private static boolean anyPolicyHasUserGroupAttributeExpression(List<RangerPolicy> policies) {
         boolean ret = false;
 
@@ -687,6 +847,20 @@ public class ServiceDefUtil {
 
                 ret = RangerRequestExprResolver.hasUserGroupAttributeInExpression(filterExpr);
             }
+
+            if (!ret && policyItem instanceof RangerDataMaskPolicyItem) {
+                RangerDataMaskPolicyItem     dataMaskPolicyItem = (RangerDataMaskPolicyItem) policyItem;
+                RangerPolicyItemDataMaskInfo dataMaskInfo       = dataMaskPolicyItem.getDataMaskInfo();
+                String                       maskedValue        = dataMaskInfo != null ? dataMaskInfo.getValueExpr() : null;
+
+                ret = RangerRequestExprResolver.hasUserGroupAttributeInExpression(maskedValue);
+
+                if (!ret) {
+                    String maskCondition = dataMaskInfo != null ? dataMaskInfo.getConditionExpr() : null;
+
+                    ret = RangerRequestExprResolver.hasUserGroupAttributeInExpression(maskCondition);
+                }
+            }
         }
 
         return ret;
