@@ -54,6 +54,10 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.PrepareBul
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.hbase.wal.WALEdit;
+import org.apache.hadoop.thirdparty.com.google.common.collect.Lists;
+import org.apache.hadoop.thirdparty.com.google.common.collect.Sets;
+import org.apache.hadoop.thirdparty.com.google.common.base.MoreObjects;
+import org.apache.hadoop.thirdparty.com.google.common.collect.MapMaker;
 import org.apache.hadoop.security.AccessControlException;
 import org.apache.ranger.audit.model.AuthzAuditEvent;
 import org.apache.ranger.audit.provider.AuditProviderFactory;
@@ -75,10 +79,6 @@ import org.apache.ranger.plugin.util.GrantRevokeRequest;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.base.MoreObjects;
-import com.google.common.collect.Lists;
-import com.google.common.collect.MapMaker;
-import com.google.common.collect.Sets;
 import com.google.protobuf.RpcCallback;
 import com.google.protobuf.RpcController;
 import org.apache.ranger.plugin.util.RangerPerfTracer;
@@ -211,7 +211,10 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		try {
 			remoteAddr = RpcServer.getRemoteAddress().get();
 		} catch (NoSuchElementException e) {
-			LOG.info("Unable to get remote Address");
+			// HBase services will sometimes make calls as a part of
+			// internal operations. It is not worth logging when we do
+			// not have a remote address (a client's remote address).
+			LOG.trace("Unable to get remote Address");
 		}
 
 		if(remoteAddr == null) {
@@ -320,10 +323,15 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		String access = _authUtils.getAccess(action);
 		User user = getActiveUser(ctx);
 		String userName = _userUtils.getUserAsString(user);
+		final Map<String, Set<String>> colFamiliesForDebugLoggingOnly;
 
 		if (LOG.isDebugEnabled()) {
+			colFamiliesForDebugLoggingOnly = getColumnFamilies(familyMap);
 			LOG.debug(String.format("evaluateAccess: entered: user[%s], Operation[%s], access[%s], families[%s]",
-					userName, operation, access, getColumnFamilies(familyMap).toString()));
+					userName, operation, access, colFamiliesForDebugLoggingOnly));
+		}
+		else{
+			colFamiliesForDebugLoggingOnly = Collections.emptyMap();
 		}
 
 		byte[] tableBytes = getTableName(env);
@@ -340,8 +348,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			LOG.debug("evaluateAccess: exiting: isKnownAccessPattern returned true: access allowed, not audited");
 			result = new ColumnFamilyAccessResult(true, true, null, null, null, null, null);
 			if (LOG.isDebugEnabled()) {
-				Map<String, Set<String>> families = getColumnFamilies(familyMap);
-				String message = String.format(messageTemplate, userName, operation, access, families.toString(), result.toString());
+				String message = String.format(messageTemplate, userName, operation, access, colFamiliesForDebugLoggingOnly, result.toString());
 				LOG.debug(message);
 			}
 			return result;
@@ -357,11 +364,10 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 				.user(user)
 				.access(access)
 				.table(table);
-		Map<String, Set<String>> families = getColumnFamilies(familyMap);
 		if (LOG.isDebugEnabled()) {
-			LOG.debug("evaluateAccess: families to process: " + families.toString());
+			LOG.debug("evaluateAccess: families to process: " + colFamiliesForDebugLoggingOnly);
 		}
-		if (families == null || families.isEmpty()) {
+		if (familyMap == null || familyMap.isEmpty()) {
 			LOG.debug("evaluateAccess: Null or empty families collection, ok.  Table level access is desired");
 			session.buildRequest()
 				.authorize();
@@ -380,7 +386,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 						authorized ? Collections.singletonList(event) : null,
 						null, authorized ? null : event, reason, null);
 			if (LOG.isDebugEnabled()) {
-				String message = String.format(messageTemplate, userName, operation, access, families.toString(), result.toString());
+				String message = String.format(messageTemplate, userName, operation, access, colFamiliesForDebugLoggingOnly, result.toString());
 				LOG.debug(message);
 			}
 			return result;
@@ -404,13 +410,13 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		Set<String> familesAccessDenied = new HashSet<String>();
 		Set<String> familesAccessIndeterminate = new HashSet<String>();
 
-		for (Map.Entry<String, Set<String>> anEntry : families.entrySet()) {
-			String family = anEntry.getKey();
+		for (Map.Entry<byte[], ? extends Collection<?>> anEntry : familyMap.entrySet()) {
+			String family = Bytes.toString(anEntry.getKey());
 			session.columnFamily(family);
 			if (LOG.isDebugEnabled()) {
 				LOG.debug("evaluateAccess: Processing family: " + family);
 			}
-			Set<String> columns = anEntry.getValue();
+			Collection<?> columns = anEntry.getValue();
 			if (columns == null || columns.isEmpty()) {
 				LOG.debug("evaluateAccess: columns collection null or empty, ok.  Family level access is desired.");
 
@@ -485,8 +491,10 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			} else {
 				LOG.debug("evaluateAccess: columns collection not empty.  Skipping Family level check, will do finer level access check.");
 				Set<String> accessibleColumns = new HashSet<String>(); // will be used in to populate our results cache for the filter
- 				for (String column : columns) {
- 					if (LOG.isDebugEnabled()) {
+				Iterator<String> columnIterator = new ColumnIterator(columns);
+				while (columnIterator.hasNext()) {
+					String column = columnIterator.next();
+					 if (LOG.isDebugEnabled()) {
  						LOG.debug("evaluateAccess: Processing column: " + column);
  					}
  					session.column(column)
@@ -526,7 +534,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		RangerAuthorizationFilter filter = new RangerAuthorizationFilter(session, familesAccessAllowed, familesAccessDenied, familesAccessIndeterminate, columnsAccessAllowed);
 		result = new ColumnFamilyAccessResult(everythingIsAccessible, somethingIsAccessible, authorizedEvents, familyLevelAccessEvents, deniedEvent, denialReason, filter);
 		if (LOG.isDebugEnabled()) {
-			String message = String.format(messageTemplate, userName, operation, access, families.toString(), result.toString());
+			String message = String.format(messageTemplate, userName, operation, access, colFamiliesForDebugLoggingOnly, result.toString());
 			LOG.debug(message);
 		}
 		return result;
@@ -832,7 +840,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		requirePermission(c, "assign", regionInfo.getTable().getName(), null, null, Action.ADMIN);
 	}
 	@Override
-	public void preBalance(ObserverContext<MasterCoprocessorEnvironment> c) throws IOException {
+	public void preBalance(ObserverContext<MasterCoprocessorEnvironment> c, BalanceRequest request) throws IOException {
 		requirePermission(c,"balance", Permission.Action.ADMIN);
 	}
 	@Override
