@@ -23,33 +23,23 @@ import java.nio.file.Files;
 import java.util.List;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.hadoop.conf.Configuration;
 import org.apache.ranger.admin.client.AbstractRangerAdminClient;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
 import org.apache.ranger.plugin.util.ServicePolicies;
 import org.apache.ranger.plugin.util.ServiceTags;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
-import com.google.gson.Gson;
-import com.google.gson.GsonBuilder;
 
 /**
  * A test implementation of the RangerAdminClient interface that just reads policies in from a file and returns them
  */
 public class RangerAdminClientImpl extends AbstractRangerAdminClient {
-    private static final Logger LOG = LoggerFactory.getLogger(RangerAdminClientImpl.class);
     private final static String cacheFilename = "hdfs-policies.json";
     private final static String tagFilename = "hdfs-policies-tag.json";
-    private Gson gson;
+    private String hdfsVersion = null;
 
-    public void init(String serviceName, String appId, String configPropertyPrefix) {
-        Gson gson = null;
-        try {
-            gson = new GsonBuilder().setDateFormat("yyyyMMdd-HH:mm:ss.SSS-Z").setPrettyPrinting().create();
-        } catch(Throwable excp) {
-            LOG.error("RangerAdminClientImpl: failed to create GsonBuilder object", excp);
-        }
-        this.gson = gson;
+    public void init(String serviceName, String appId, String configPropertyPrefix, Configuration config) {
+        super.init(serviceName, appId, configPropertyPrefix, config);
+
+        this.hdfsVersion = config.get("hdfs.version", "");
     }
 
     public ServicePolicies getServicePoliciesIfUpdated(long lastKnownVersion, long lastActivationTimeInMillis) throws Exception {
@@ -58,7 +48,6 @@ public class RangerAdminClientImpl extends AbstractRangerAdminClient {
         if (basedir == null) {
             basedir = new File(".").getCanonicalPath();
         }
-	    String hdfsVersion = RangerConfiguration.getInstance().get("hdfs.version", "");
 
         final String relativePath;
         if (StringUtils.isNotBlank(hdfsVersion)) {
@@ -78,7 +67,6 @@ public class RangerAdminClientImpl extends AbstractRangerAdminClient {
         if (basedir == null) {
             basedir = new File(".").getCanonicalPath();
         }
-        String hdfsVersion = RangerConfiguration.getInstance().get("hdfs.version", "");
 
         final String relativePath;
         if (StringUtils.isNotBlank(hdfsVersion)) {
