@@ -21,8 +21,6 @@
 
 import java.util.List;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
 import org.apache.hadoop.hive.ql.security.HiveAuthenticationProvider;
@@ -37,14 +35,15 @@ import org.apache.hadoop.hive.ql.security.authorization.plugin.HivePolicyProvide
 import org.apache.hadoop.hive.ql.security.authorization.plugin.HivePrincipal;
 import org.apache.hadoop.hive.ql.security.authorization.plugin.HivePrivilegeInfo;
 import org.apache.hadoop.hive.ql.security.authorization.plugin.HivePrivilegeObject;
-import org.apache.hadoop.hive.ql.security.authorization.plugin.HiveRoleGrant;
 import org.apache.hadoop.hive.ql.security.authorization.plugin.SettableConfigUpdater;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.ranger.authorization.utils.StringUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public abstract class RangerHiveAuthorizerBase extends AbstractHiveAuthorizer {
 
-	private static final Log LOG = LogFactory.getLog(RangerHiveAuthorizerBase.class);
+	private static final Logger LOG = LoggerFactory.getLogger(RangerHiveAuthorizerBase.class);
 
 	private HiveMetastoreClientFactory mMetastoreClientFactory;
 	private HiveConf                   mHiveConf;
@@ -132,16 +131,6 @@ public abstract class RangerHiveAuthorizerBase extends AbstractHiveAuthorizer {
 		return null;
 	}
 
-	@Override
-	public List<HiveRoleGrant> getRoleGrantInfoForPrincipal(HivePrincipal principal)
-			throws HiveAuthzPluginException, HiveAccessControlException {
-		LOG.debug("RangerHiveAuthorizerBase.getRoleGrantInfoForPrincipal()");
-
-		throwNotImplementedException("getRoleGrantInfoForPrincipal");
-
-		return null;
-	}
-
 	@Override
 	public VERSION getVersion() {
 		return VERSION.V1;
