@@ -47,6 +47,24 @@ get_prop(){
 	echo $value
 }
 
+get_prop_or_default() {
+  validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
+
+  if test -z "$validateProperty" ;
+  then
+    value=$3 # default value
+  else
+    value=$(echo $validateProperty | cut -d "=" -f2-)
+  fi
+
+  if [[ $1 == *password* ]]
+  then
+    echo $value
+  else
+   echo $value | tr -d \'\"
+ fi
+}
+
 PYTHON_COMMAND_INVOKER=$(get_prop 'PYTHON_COMMAND_INVOKER' $PROPFILE)
 DB_FLAVOR=$(get_prop 'DB_FLAVOR' $PROPFILE)
 SQL_CONNECTOR_JAR=$(get_prop 'SQL_CONNECTOR_JAR' $PROPFILE)
@@ -80,6 +98,7 @@ LOGFILE=$(eval echo "$(get_prop 'LOGFILE' $PROPFILE)")
 JAVA_BIN=$(get_prop 'JAVA_BIN' $PROPFILE)
 JAVA_VERSION_REQUIRED=$(get_prop 'JAVA_VERSION_REQUIRED' $PROPFILE)
 JAVA_ORACLE=$(get_prop 'JAVA_ORACLE' $PROPFILE)
+java_opts=$(get_prop_or_default 'java_opts' $PROPFILE '')
 mysql_core_file=$(get_prop 'mysql_core_file' $PROPFILE)
 oracle_core_file=$(get_prop 'oracle_core_file' $PROPFILE)
 postgres_core_file=$(get_prop 'postgres_core_file' $PROPFILE)
@@ -114,6 +133,12 @@ AZURE_MASTER_KEY_TYPE=$(get_prop 'AZURE_MASTER_KEY_TYPE' $PROPFILE)
 ZONE_KEY_ENCRYPTION_ALGO=$(get_prop 'ZONE_KEY_ENCRYPTION_ALGO' $PROPFILE)
 AZURE_KEYVAULT_URL=$(get_prop 'AZURE_KEYVAULT_URL' $PROPFILE)
 
+AWS_KMS_ENABLED=$(get_prop 'AWS_KMS_ENABLED' $PROPFILE)
+AWS_KMS_MASTERKEY_ID=$(get_prop 'AWS_KMS_MASTERKEY_ID' $PROPFILE)
+AWS_CLIENT_ACCESSKEY=$(get_prop 'AWS_CLIENT_ACCESSKEY' $PROPFILE)
+AWS_CLIENT_SECRETKEY=$(get_prop 'AWS_CLIENT_SECRETKEY' $PROPFILE)
+AWS_CLIENT_REGION=$(get_prop 'AWS_CLIENT_REGION' $PROPFILE)
+
 IS_GCP_ENABLED=$(get_prop 'IS_GCP_ENABLED' $PROPFILE)
 GCP_KEYRING_ID=$(get_prop 'GCP_KEYRING_ID' $PROPFILE)
 GCP_CRED_JSON_FILE=$(get_prop 'GCP_CRED_JSON_FILE' $PROPFILE)
@@ -368,6 +393,7 @@ check_java_version() {
 		log "[E] The java version must be greater than or equal to $JAVA_VERSION_REQUIRED, the current java version is $version"
 		exit 1;
 	fi
+	if [[ ${JAVA_OPTS} == "" ]] ;then  export JAVA_OPTS="${java_opts}" ;fi
 }
 
 sanity_check_files() {
@@ -626,6 +652,9 @@ update_properties() {
 	AZURE_CLIENT_SEC="ranger.kms.azure.client.secret"
 	AZURE_CLIENT_SECRET_ALIAS="ranger.ks.azure.client.secret"
 
+	AWS_CLIENT_SEC="ranger.kms.aws.client.secretkey"
+	AWS_CLIENT_SECRET_ALIAS="ranger.ks.aws.client.secretkey"
+
 	TENCENT_CLIENT_SEC="ranger.kms.tencent.client.secret"
 	TENCENT_CLIENT_SECRET_ALIAS="ranger.ks.tencent.client.secret"
 
@@ -633,6 +662,7 @@ update_properties() {
         HSM_ENABLED=`echo $HSM_ENABLED | tr '[:lower:]' '[:upper:]'`
         KEYSECURE_ENABLED=`echo $KEYSECURE_ENABLED | tr '[:lower:]' '[:upper:]'`
 	AZURE_KEYVAULT_ENABLED=`echo $AZURE_KEYVAULT_ENABLED | tr '[:lower:]' '[:upper:]'`
+	AWS_KMS_ENABLED=`echo $AWS_KMS_ENABLED | tr '[:lower:]' '[:upper:]'`
 	IS_GCP_ENABLED=`echo $IS_GCP_ENABLED | tr '[:lower:]' '[:upper:]'`
 	TENCENT_KMS_ENABLED=`echo $TENCENT_KMS_ENABLED | tr '[:lower:]' '[:upper:]'`
 
@@ -688,6 +718,21 @@ update_properties() {
                         updatePropertyToFilePy $propertyName $newPropertyValue $to_file
                 fi
 
+    # if $AWS_CLIENT_ACCESSKEY is set, then $AWS_CLIENT_SECRETKEY must be set
+		if [ "$AWS_KMS_ENABLED" == "TRUE" -a -n "$AWS_CLIENT_ACCESSKEY" ]
+		then
+                        checkIfEmpty "$AWS_CLIENT_SECRETKEY" "AWS Client SecretKey"
+                        $PYTHON_COMMAND_INVOKER ranger_credential_helper.py -l "cred/lib/*" -f "$keystore" -k "${AWS_CLIENT_SECRET_ALIAS}" -v "${AWS_CLIENT_SECRETKEY}" -c 1
+
+                        propertyName=ranger.kms.aws.client.secretkey.alias
+                        newPropertyValue="${AWS_CLIENT_SECRET_ALIAS}"
+                        updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+
+                        propertyName=ranger.kms.aws.client.secretkey
+                        newPropertyValue="_"
+                        updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+		fi
+
 		if [ "$TENCENT_KMS_ENABLED" == "TRUE" ]
 		then
                         checkIfEmpty "$TENCENT_CLIENT_SECRET" "Tencent Client Secret"
@@ -742,6 +787,10 @@ update_properties() {
                 newPropertyValue="${AZURE_CLIENT_SECRET}"
                 updatePropertyToFilePy $propertyName $newPropertyValue $to_file
 
+		propertyName="${AWS_CLIENT_SEC}"
+                newPropertyValue="${AWS_CLIENT_SECRETKEY}"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+
 		propertyName="${TENCENT_CLIENT_SEC}"
                 newPropertyValue="${TENCENT_CLIENT_SECRET}"
                 updatePropertyToFilePy $propertyName $newPropertyValue $to_file
@@ -897,6 +946,32 @@ update_properties() {
 
         fi
 
+	########### AWS KEY VAULT #################
+
+
+        if [ "${AWS_KMS_ENABLED}" != "TRUE" ]
+        then
+                propertyName=ranger.kms.awskms.enabled
+                newPropertyValue="false"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+        else
+                propertyName=ranger.kms.awskms.enabled
+                newPropertyValue="true"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+
+                propertyName=ranger.kms.awskms.masterkey.id
+                newPropertyValue="${AWS_KMS_MASTERKEY_ID}"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+
+                propertyName=ranger.kms.aws.client.accesskey
+                newPropertyValue="${AWS_CLIENT_ACCESSKEY}"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+
+                propertyName=ranger.kms.aws.client.region
+                newPropertyValue="${AWS_CLIENT_REGION}"
+                updatePropertyToFilePy $propertyName $newPropertyValue $to_file
+        fi
+
 	########### RANGER GCP #################
 		if [ "${IS_GCP_ENABLED}" != "TRUE" ]
 		then
@@ -955,7 +1030,7 @@ update_properties() {
                 newPropertyValue="${TENCENT_CLIENT_ID}"
                 updatePropertyToFilePy $propertyName $newPropertyValue $to_file
 
-                propertyName=ranger.kms.client.region
+                propertyName=ranger.kms.tencent.client.region
                 newPropertyValue="${TENCENT_CLIENT_REGION}"
                 updatePropertyToFilePy $propertyName $newPropertyValue $to_file
 
