@@ -17,9 +17,8 @@
  */
 package org.apache.hadoop.crypto.key.kms.server;
 
-import com.google.common.base.Preconditions;
-import com.google.common.base.Stopwatch;
-
+import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.thirdparty.com.google.common.base.Stopwatch;
 import org.apache.hadoop.util.KMSUtil;
 import org.apache.commons.codec.binary.Base64;
 import org.apache.hadoop.classification.InterfaceAudience;
@@ -57,9 +56,14 @@ import java.util.ArrayList;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
+import java.util.concurrent.TimeUnit;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import org.apache.ranger.kms.metrics.KMSMetricWrapper;
+import org.apache.ranger.kms.metrics.KMSMetrics;
+import org.apache.ranger.kms.metrics.collector.KMSMetricsCollector;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import static org.apache.hadoop.util.KMSUtil.checkNotEmpty;
@@ -86,9 +90,12 @@ public class KMS {
   private final KeyProviderCryptoExtension provider;
   private final KMSAudit                   kmsAudit;
 
+  private KMSMetricsCollector kmsMetricsCollector;
+
   public KMS() throws Exception {
     provider = KMSWebApp.getKeyProvider();
     kmsAudit = KMSWebApp.getKMSAudit();
+    this.kmsMetricsCollector = KMSMetricWrapper.getInstance(KMSWebApp.isMetricCollectionThreadSafe()).getKmsMetricsCollector();
   }
 
   @POST
@@ -101,7 +108,10 @@ public class KMS {
       LOG.debug("==> createKey()");
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
+
 	try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.KEY_CREATE_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       final UserGroupInformation user = HttpUserGroupInformation.get();
@@ -175,6 +185,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.KEY_CREATE_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== createKey()");
       }
@@ -188,7 +199,9 @@ public class KMS {
       LOG.debug("==> deleteKey({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.DELETE_KEY_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       UserGroupInformation user = HttpUserGroupInformation.get();
@@ -213,6 +226,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.DELETE_KEY_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== deleteKey({})", name);
       }
@@ -228,7 +242,9 @@ public class KMS {
       LOG.debug("==> rolloverKey({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.ROLL_NEW_VERSION_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       UserGroupInformation user = HttpUserGroupInformation.get();
@@ -271,6 +287,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.ROLL_NEW_VERSION_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== rolloverKey({})", name);
       }
@@ -284,7 +301,9 @@ public class KMS {
       LOG.debug("==> invalidateCache({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.INVALIDATE_CACHE_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       checkNotEmpty(name, "name");
@@ -309,6 +328,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.INVALIDATE_CACHE_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== invalidateCache({})", name);
       }
@@ -323,7 +343,9 @@ public class KMS {
       LOG.debug("==> getKeysMetadata()");
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_KEYS_METADATA_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       final UserGroupInformation user     = HttpUserGroupInformation.get();
@@ -333,6 +355,7 @@ public class KMS {
 
       KeyProvider.Metadata[] keysMeta = user.doAs((PrivilegedExceptionAction<KeyProvider.Metadata[]>) () -> provider.getKeysMetadata(keyNames));
       Object                 json     = KMSServerJSONUtils.toJSON(keyNames, keysMeta);
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_KEYS_METADATA_KEYNAMES_COUNT, keyNames.length);
 
       kmsAudit.ok(user, KMSOp.GET_KEYS_METADATA, "");
 
@@ -342,6 +365,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_KEYS_METADATA_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getKeysMetadata()");
       }
@@ -356,7 +380,9 @@ public class KMS {
       LOG.debug("==> getKeyNames()");
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_KEYS_COUNT);
       KMSWebApp.getAdminCallsMeter().mark();
 
       UserGroupInformation user = HttpUserGroupInformation.get();
@@ -373,6 +399,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_KEYS_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getKeyNames()");
       }
@@ -407,13 +434,15 @@ public class KMS {
       LOG.debug("==> getMetadata({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_METADATA_COUNT);
+      KMSWebApp.getAdminCallsMeter().mark();
       UserGroupInformation user = HttpUserGroupInformation.get();
 
       checkNotEmpty(name, "name");
 
-      KMSWebApp.getAdminCallsMeter().mark();
-
       assertAccess(Type.GET_METADATA, user, KMSOp.GET_METADATA, name, request.getRemoteAddr());
 
       KeyProvider.Metadata metadata = user.doAs((PrivilegedExceptionAction<KeyProvider.Metadata>) () -> provider.getMetadata(name));
@@ -427,6 +456,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_METADATA_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getMetadata({})", name);
       }
@@ -441,13 +471,15 @@ public class KMS {
       LOG.debug("==> getCurrentVersion({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_CURRENT_KEY_COUNT);
+      KMSWebApp.getKeyCallsMeter().mark();
       UserGroupInformation user = HttpUserGroupInformation.get();
 
       checkNotEmpty(name, "name");
 
-      KMSWebApp.getKeyCallsMeter().mark();
-
       assertAccess(Type.GET, user, KMSOp.GET_CURRENT_KEY, name, request.getRemoteAddr());
 
       KeyVersion keyVersion = user.doAs((PrivilegedExceptionAction<KeyVersion>) () -> provider.getCurrentKey(name));
@@ -462,6 +494,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_CURRENT_KEY_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getCurrentVersion({})", name);
       }
@@ -476,13 +509,14 @@ public class KMS {
       LOG.debug("==> getKeyVersion({})", versionName);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_KEY_VERSION_COUNT);
+      KMSWebApp.getKeyCallsMeter().mark();
       UserGroupInformation user = HttpUserGroupInformation.get();
 
       checkNotEmpty(versionName, "versionName");
 
-      KMSWebApp.getKeyCallsMeter().mark();
-
       assertAccess(Type.GET, user, KMSOp.GET_KEY_VERSION, request.getRemoteAddr());
 
       KeyVersion keyVersion = user.doAs((PrivilegedExceptionAction<KeyVersion>) () -> provider.getKeyVersion(versionName));
@@ -499,6 +533,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_KEY_VERSION_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getKeyVersion({})", versionName);
       }
@@ -517,6 +552,7 @@ public class KMS {
       LOG.debug("==> generateEncryptedKeys(name={}, eekOp={}, numKeys={})", name, edekOp, numKeys);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
       UserGroupInformation user = HttpUserGroupInformation.get();
 
@@ -527,6 +563,7 @@ public class KMS {
       Object retJSON;
 
       if (edekOp.equals(KMSRESTConstants.EEK_GENERATE)) {
+        this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.EEK_GENERATE_COUNT);
         assertAccess(Type.GENERATE_EEK, user, KMSOp.GENERATE_EEK, name,request.getRemoteAddr());
 
         final List<EncryptedKeyVersion> retEdeks = new LinkedList<>();
@@ -543,6 +580,8 @@ public class KMS {
           LOG.error("Exception in generateEncryptedKeys:", e);
 
           throw new IOException(e);
+        }finally {
+          this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.EEK_GENERATE_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
         }
 
         kmsAudit.ok(user, KMSOp.GENERATE_EEK, name, "");
@@ -590,16 +629,17 @@ public class KMS {
       LOG.debug("==> reencryptEncryptedKeys(name={}, count={})", name, (jsonPayload != null ? jsonPayload.size() : 0));
     }
 
+    final Stopwatch sw = Stopwatch.createStarted();
+
     try {
-      final Stopwatch sw = Stopwatch.createStarted();
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.REENCRYPT_EEK_BATCH_COUNT);
+      KMSWebApp.getReencryptEEKBatchCallsMeter().mark();
 
       checkNotEmpty(name, "name");
       checkNotNull(jsonPayload, "jsonPayload");
 
       final UserGroupInformation user = HttpUserGroupInformation.get();
 
-      KMSWebApp.getReencryptEEKBatchCallsMeter().mark();
-
       if (jsonPayload.size() > MAX_NUM_PER_BATCH) {
         LOG.warn("Payload size {} too big for reencryptEncryptedKeys from" + " user {}.", jsonPayload.size(), user);
       }
@@ -626,6 +666,8 @@ public class KMS {
         retJSON.add(KMSUtil.toJSON(ekv));
       }
 
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.REENCRYPT_EEK_BATCH_KEYS_COUNT, ekvs.size());
+
       kmsAudit.ok(user, KMSOp.REENCRYPT_EEK_BATCH, name,"reencrypted " + ekvs.size() + " keys");
 
       if (LOG.isDebugEnabled()) {
@@ -638,6 +680,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.REENCRYPT_EEK_BATCH_ELAPSED_TIME, sw.elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== reencryptEncryptedKeys(name={}, count={})", name, (jsonPayload != null ? jsonPayload.size() : 0));
       }
@@ -655,6 +698,8 @@ public class KMS {
       LOG.debug("==> handleEncryptedKeyOp(versionName={}, eekOp={})", versionName, eekOp);
     }
 
+    Optional<KMSMetrics.KMSMetric> oprCode = Optional.empty();
+    final Stopwatch sw = Stopwatch.createStarted();
     try {
       UserGroupInformation user = HttpUserGroupInformation.get();
 
@@ -676,7 +721,9 @@ public class KMS {
       Object retJSON;
 
       if (eekOp.equals(KMSRESTConstants.EEK_DECRYPT)) {
+        oprCode = Optional.of(KMSMetrics.KMSMetric.EEK_DECRYPT_ELAPSED_TIME);
         KMSWebApp.getDecryptEEKCallsMeter().mark();
+        this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.EEK_DECRYPT_COUNT);
 
         assertAccess(Type.DECRYPT_EEK, user, KMSOp.DECRYPT_EEK, keyName, request.getRemoteAddr());
 
@@ -690,7 +737,9 @@ public class KMS {
 
         kmsAudit.ok(user, KMSOp.DECRYPT_EEK, keyName, "");
       } else if (eekOp.equals(KMSRESTConstants.EEK_REENCRYPT)) {
+        oprCode = Optional.of(KMSMetrics.KMSMetric.EEK_REENCRYPT_ELAPSED_TIME);
         KMSWebApp.getReencryptEEKCallsMeter().mark();
+        this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.EEK_REENCRYPT_COUNT);
 
         assertAccess(Type.GENERATE_EEK, user, KMSOp.REENCRYPT_EEK, keyName);
 
@@ -723,6 +772,7 @@ public class KMS {
 
       throw e;
     } finally {
+      oprCode.ifPresent( metric -> this.kmsMetricsCollector.updateMetric(metric, sw.stop().elapsed(TimeUnit.MILLISECONDS)));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== handleEncryptedKeyOp(versionName={}, eekOp={})", versionName, eekOp);
       }
@@ -737,13 +787,15 @@ public class KMS {
       LOG.debug("==> getKeyVersions({})", name);
     }
 
+    Stopwatch sw = Stopwatch.createStarted();
     try {
+
+      KMSWebApp.getKeyCallsMeter().mark();
+      this.kmsMetricsCollector.incrementCounter(KMSMetrics.KMSMetric.GET_KEY_VERSIONS_COUNT);
       UserGroupInformation user = HttpUserGroupInformation.get();
 
       checkNotEmpty(name, "name");
 
-      KMSWebApp.getKeyCallsMeter().mark();
-
       assertAccess(Type.GET, user, KMSOp.GET_KEY_VERSIONS, name, request.getRemoteAddr());
 
       List<KeyVersion> ret  = user.doAs((PrivilegedExceptionAction<List<KeyVersion>>) () -> provider.getKeyVersions(name));
@@ -757,6 +809,7 @@ public class KMS {
 
       throw e;
     } finally {
+      this.kmsMetricsCollector.updateMetric(KMSMetrics.KMSMetric.GET_KEY_VERSIONS_ELAPSED_TIME, sw.stop().elapsed(TimeUnit.MILLISECONDS));
       if (LOG.isDebugEnabled()) {
         LOG.debug("<== getKeyVersions({})", name);
       }
