@@ -71,32 +71,31 @@ public class KMSMDCFilter implements Filter {
 
   @Override
   public void doFilter(ServletRequest request, ServletResponse response,
-      FilterChain chain)
-      throws IOException, ServletException {
-    try {
-    	 String path = ((HttpServletRequest) request).getRequestURI();
-         HttpServletResponse resp = (HttpServletResponse) response;
-         resp.setHeader("Strict-Transport-Security", "max-age=31536000; includeSubDomains");
-    	    
-    	     if (path.startsWith(RANGER_KMS_REST_API_PATH)) {
-                 chain.doFilter(request, resp);
-    	      } else {
-			      DATA_TL.remove();
-			      UserGroupInformation ugi = HttpUserGroupInformation.get();
-			      String method = ((HttpServletRequest) request).getMethod();
-			      StringBuffer requestURL = ((HttpServletRequest) request).getRequestURL();
-			      String queryString = ((HttpServletRequest) request).getQueryString();
-			      if (queryString != null) {
-			        requestURL.append("?").append(queryString);
-			      }
-			      DATA_TL.set(new Data(ugi, method, requestURL.toString()));
-			      chain.doFilter(request, resp);
-    	    }
-    } finally {
-      DATA_TL.remove();
-      
-    }
-  }
+			FilterChain chain) throws IOException, ServletException {
+		try {
+			String path = ((HttpServletRequest) request).getRequestURI();
+			HttpServletResponse resp = (HttpServletResponse) response;
+			resp.setHeader("Strict-Transport-Security", "max-age=31536000; includeSubDomains; preload");
+
+			if (path.startsWith(RANGER_KMS_REST_API_PATH)) {
+				chain.doFilter(request, resp);
+			} else {
+				DATA_TL.remove();
+				UserGroupInformation ugi = HttpUserGroupInformation.get();
+				String method = ((HttpServletRequest) request).getMethod();
+				StringBuffer requestURL = ((HttpServletRequest) request).getRequestURL();
+				String queryString = ((HttpServletRequest) request).getQueryString();
+				if (queryString != null) {
+					requestURL.append("?").append(queryString);
+				}
+				DATA_TL.set(new Data(ugi, method, requestURL.toString()));
+				chain.doFilter(request, resp);
+			}
+		} finally {
+			DATA_TL.remove();
+
+		}
+	}
 
   @Override
   public void destroy() {
