@@ -48,6 +48,7 @@ import java.util.Map;
 public class NiFiRegistryConnectionMgr {
 
     private static final Logger LOG = LoggerFactory.getLogger(NiFiRegistryConnectionMgr.class);
+    private static final String SSL_ALGORITHM = "TLSv1.2";
 
     private static final String API_RESOURCES_PATH = "/nifi-registry-api/policies/resources";
     static final String INVALID_URL_MSG =  "NiFi Registry URL must be a valid URL of the form " +
@@ -112,7 +113,7 @@ public class NiFiRegistryConnectionMgr {
                         truststore.trim(),
                         truststorePassword.trim().toCharArray(),
                         truststoreType.trim(),
-                        "TLS");
+                        SSL_ALGORITHM);
             }
         }
 
