@@ -20,28 +20,28 @@
 
 package org.apache.ranger.authorization.yarn.authorizer;
 
-import java.util.Map;
+import java.util.List;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.authorize.AccessControlList;
-import org.apache.hadoop.yarn.security.AccessType;
-import org.apache.hadoop.yarn.security.PrivilegedEntity;
+import org.apache.hadoop.yarn.security.AccessRequest;
+import org.apache.hadoop.yarn.security.Permission;
 import org.apache.hadoop.yarn.security.YarnAuthorizationProvider;
 import org.apache.ranger.plugin.classloader.RangerPluginClassLoader;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 
 public class RangerYarnAuthorizer extends YarnAuthorizationProvider {
-	private static final Log LOG  = LogFactory.getLog(RangerYarnAuthorizer.class);
+	private static final Logger LOG  = LoggerFactory.getLogger(RangerYarnAuthorizer.class);
 
 	private static final String   RANGER_PLUGIN_TYPE                      = "yarn";
 	private static final String   RANGER_YARN_AUTHORIZER_IMPL_CLASSNAME   = "org.apache.ranger.authorization.yarn.authorizer.RangerYarnAuthorizer";
 
-	private YarnAuthorizationProvider 	 	yarnAuthorizationProviderImpl = null;
-	private static RangerPluginClassLoader  rangerPluginClassLoader  	  = null;
+	private YarnAuthorizationProvider yarnAuthorizationProviderImpl = null;
+	private RangerPluginClassLoader   rangerPluginClassLoader       = null;
 	
 	public RangerYarnAuthorizer() {
 		if(LOG.isDebugEnabled()) {
@@ -102,7 +102,7 @@ public class RangerYarnAuthorizer extends YarnAuthorizationProvider {
 	}
 
 	@Override
-	public boolean checkPermission(AccessType accessType, PrivilegedEntity target, UserGroupInformation user) {
+	public boolean checkPermission(AccessRequest accessRequest) {
 		
 		boolean ret = false;
 		
@@ -113,7 +113,7 @@ public class RangerYarnAuthorizer extends YarnAuthorizationProvider {
 		try {
 			activatePluginClassLoader();
 
-			ret = yarnAuthorizationProviderImpl.checkPermission(accessType, target, user);
+			ret = yarnAuthorizationProviderImpl.checkPermission(accessRequest);
 		} finally {
 			deactivatePluginClassLoader();
 		}
@@ -126,7 +126,7 @@ public class RangerYarnAuthorizer extends YarnAuthorizationProvider {
 	}
 
 	@Override
-	public void setPermission(PrivilegedEntity target,	Map<AccessType, AccessControlList> acls, UserGroupInformation ugi) {
+	public void setPermission(List<Permission> permissions, UserGroupInformation ugi) {
 		
 		if(LOG.isDebugEnabled()) {
 			LOG.debug("==> RangerYarnAuthorizer.setPermission()");
@@ -135,7 +135,7 @@ public class RangerYarnAuthorizer extends YarnAuthorizationProvider {
 		try {
 			activatePluginClassLoader();
 
-			yarnAuthorizationProviderImpl.setPermission(target, acls, ugi);
+			yarnAuthorizationProviderImpl.setPermission(permissions, ugi);
 		} finally {
 			deactivatePluginClassLoader();
 		}
