@@ -472,11 +472,11 @@ class OracleConf(BaseDB):
 						log("[I] User " + db_user + " created", "info")
 						log("[I] Granting permission to " + db_user, "info")
 						if is_unix:
-							query = get_cmd + " -c \; -query 'GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;'" % (db_user)
+							query = get_cmd + " -c \; -query 'GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;'" % (db_user)
 							jisql_log(query, db_root_password)
 							ret = subprocessCallWithRetry(shlex.split(query))
 						elif os_name == "WINDOWS":
-							query = get_cmd + " -query \"GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;\" -c ;" % (db_user)
+							query = get_cmd + " -query \"GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;\" -c ;" % (db_user)
 							jisql_log(query, db_root_password)
 							ret = subprocessCallWithRetry(query)
 						if ret == 0:
@@ -624,11 +624,11 @@ class OracleConf(BaseDB):
 		if dryMode == False:
 			get_cmd = self.get_jisql_cmd(root_user ,db_root_password)
 			if is_unix:
-				query = get_cmd + " -c \; -query 'GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;'" % (db_user)
+				query = get_cmd + " -c \; -query 'GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;'" % (db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(shlex.split(query))
 			elif os_name == "WINDOWS":
-				query = get_cmd + " -query \"GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;\" -c ;" % (db_user)
+				query = get_cmd + " -query \"GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;\" -c ;" % (db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(query)
 			if ret == 0:
@@ -638,7 +638,7 @@ class OracleConf(BaseDB):
 				log("[E] Granting Oracle user '" + db_user + "' failed..", "error")
 				sys.exit(1)
 		else:
-			logFile("GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;" % (db_user))
+			logFile("GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED Tablespace TO %s;" % (db_user))
 
 	def create_auditdb_user(self, xa_db_host , audit_db_host , db_name ,audit_db_name, xa_db_root_user, audit_db_root_user, db_user, audit_db_user, xa_db_root_password, audit_db_root_password, db_password, audit_db_password, DBA_MODE,dryMode):
 		if DBA_MODE == "TRUE":
@@ -713,7 +713,7 @@ class OracleConf(BaseDB):
 	def writeDrymodeCmd(self, xa_db_host, audit_db_host, xa_db_root_user, xa_db_root_password, db_user, db_password, db_name, audit_db_root_user, audit_db_root_password, audit_db_user, audit_db_password, audit_db_name):
 		logFile("# Login to ORACLE Server from a ORACLE dba user(i.e 'sys') to execute below sql statements.")
 		logFile('create user %s identified by "%s";'%(db_user, db_password))
-		logFile('GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE PUBLIC SYNONYM,CREATE ANY SYNONYM,CREATE TRIGGER,UNLIMITED TABLESPACE TO %s;'%(db_user))
+		logFile('GRANT CREATE SESSION,CREATE PROCEDURE,CREATE TABLE,CREATE VIEW,CREATE SEQUENCE,CREATE SYNONYM,CREATE TRIGGER,UNLIMITED TABLESPACE TO %s;'%(db_user))
 		logFile("create tablespace %s datafile '%s.dat' size 10M autoextend on;" %(db_name, db_name))
 		logFile('alter user %s DEFAULT tablespace %s;'%(db_user, db_name))
 		if not db_user == audit_db_user:
@@ -799,13 +799,13 @@ class PostgresConf(BaseDB):
 					log("[I] User does not exists, Creating user : " + db_user, "info")
 					get_cmd = self.get_jisql_cmd(root_user, db_root_password, 'postgres')
 					if is_unix:
-						query = get_cmd + " -query \"CREATE USER %s WITH LOGIN PASSWORD '%s';\"" %(db_user, db_password)
-						query_with_masked_pwd = get_cmd + " -query \"CREATE USER %s WITH LOGIN PASSWORD '%s';\"" %(db_user, masked_pwd_string)
+						query = get_cmd + " -query \"CREATE USER \\\"%s\\\" WITH LOGIN PASSWORD '%s';\"" %(db_user, db_password)
+						query_with_masked_pwd = get_cmd + " -query \"CREATE USER \\\"%s\\\" WITH LOGIN PASSWORD '%s';\"" %(db_user, masked_pwd_string)
 						jisql_log(query_with_masked_pwd, db_root_password)
 						ret = subprocessCallWithRetry(shlex.split(query))
 					elif os_name == "WINDOWS":
-						query = get_cmd + " -query \"CREATE USER %s WITH LOGIN PASSWORD '%s';\" -c ;" %(db_user, db_password)
-						query_with_masked_pwd = get_cmd + " -query \"CREATE USER %s WITH LOGIN PASSWORD '%s';\" -c ;" %(db_user, masked_pwd_string)
+						query = get_cmd + " -query \"CREATE USER \\\"%s\\\" WITH LOGIN PASSWORD '%s';\" -c ;" %(db_user, db_password)
+						query_with_masked_pwd = get_cmd + " -query \"CREATE USER \\\"%s\\\" WITH LOGIN PASSWORD '%s';\" -c ;" %(db_user, masked_pwd_string)
 						jisql_log(query_with_masked_pwd, db_root_password)
 						ret = subprocessCallWithRetry(query)
 					if self.verify_user(root_user, db_root_password, db_user,dryMode):
@@ -814,7 +814,7 @@ class PostgresConf(BaseDB):
 						log("[E] Postgres user " +db_user+" creation failed..", "error")
 						sys.exit(1)
 				else:
-					logFile("CREATE USER %s WITH LOGIN PASSWORD '%s';" %(db_user, db_password))
+					logFile("CREATE USER \"%s\" WITH LOGIN PASSWORD '%s';" %(db_user, db_password))
 
 	def verify_db(self, root_user, db_root_password, db_name,dryMode):
 		if dryMode == False:
@@ -841,11 +841,11 @@ class PostgresConf(BaseDB):
 				log("[I] Database does not exist, Creating database : " + db_name,"info")
 				get_cmd = self.get_jisql_cmd(root_user, db_root_password, 'postgres')
 				if is_unix:
-					query = get_cmd + " -query \"create database %s with OWNER %s;\"" %(db_name, db_user)
+					query = get_cmd + " -query \"create database \\\"%s\\\" with OWNER \\\"%s\\\";\"" %(db_name, db_user)
 					jisql_log(query, db_root_password)
 					ret = subprocessCallWithRetry(shlex.split(query))
 				elif os_name == "WINDOWS":
-					query = get_cmd + " -query \"create database %s with OWNER %s;\" -c ;" %(db_name, db_user)
+					query = get_cmd + " -query \"create database \\\"%s\\\" with OWNER \\\"%s\\\";\" -c ;" %(db_name, db_user)
 					jisql_log(query, db_root_password)
 					ret = subprocessCallWithRetry(query)
 				if self.verify_db(root_user, db_root_password, db_name,dryMode):
@@ -855,18 +855,18 @@ class PostgresConf(BaseDB):
 					log("[E] Database creation failed..","error")
 					sys.exit(1)
 			else:
-				logFile("CREATE DATABASE %s WITH OWNER %s;" %(db_name, db_user))
+				logFile("CREATE DATABASE \"%s\" WITH OWNER \"%s\";" %(db_name, db_user))
 
 	def grant_xa_db_user(self, root_user, db_name, db_user, db_password, db_root_password , is_revoke,dryMode):
 		if dryMode == False:
 			log("[I] Granting privileges TO user '"+db_user+"' on db '"+db_name+"'" , "info")
 			get_cmd = self.get_jisql_cmd(root_user, db_root_password, db_name)
 			if is_unix:
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON DATABASE %s to %s;\"" %(db_name, db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON DATABASE \\\"%s\\\" to \\\"%s\\\";\"" %(db_name, db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(shlex.split(query))
 			elif os_name == "WINDOWS":
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON DATABASE %s to %s;\" -c ;" %(db_name, db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON DATABASE \\\"%s\\\" to \\\"%s\\\";\" -c ;" %(db_name, db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(query)
 			if ret != 0:
@@ -874,11 +874,11 @@ class PostgresConf(BaseDB):
 				sys.exit(1)
 
 			if is_unix:
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SCHEMA public TO %s;\"" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SCHEMA public TO \\\"%s\\\";\"" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(shlex.split(query))
 			elif os_name == "WINDOWS":
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SCHEMA public TO %s;\" -c ;" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SCHEMA public TO \\\"%s\\\";\" -c ;" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(query)
 			if ret != 0:
@@ -886,11 +886,11 @@ class PostgresConf(BaseDB):
 				sys.exit(1)
 
 			if is_unix:
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO %s;\"" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO \\\"%s\\\";\"" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(shlex.split(query))
 			elif os_name == "WINDOWS":
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO %s;\" -c ;" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO \\\"%s\\\";\" -c ;" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(query)
 			if ret != 0:
@@ -909,14 +909,14 @@ class PostgresConf(BaseDB):
 						tablename , value = each_line.strip().split(" |",1)
 						tablename = tablename.strip()
 						if is_unix:
-							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON TABLE %s TO %s;\"" %(tablename,db_user)
+							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON TABLE %s TO \\\"%s\\\";\"" %(tablename,db_user)
 							jisql_log(query1, db_root_password)
 							ret = subprocessCallWithRetry(shlex.split(query1))
 							if ret != 0:
 								log("[E] Granting all privileges on tablename "+tablename+" to user "+db_user+" failed..", "error")
 								sys.exit(1)
 						elif os_name == "WINDOWS":
-							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON TABLE %s TO %s;\" -c ;" %(tablename,db_user)
+							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON TABLE %s TO \\\"%s\\\";\" -c ;" %(tablename,db_user)
 							jisql_log(query1, db_root_password)
 							ret = subprocessCallWithRetry(query1)
 							if ret != 0:
@@ -925,11 +925,11 @@ class PostgresConf(BaseDB):
 
 
 			if is_unix:
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO %s;\"" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO \\\"%s\\\";\"" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(shlex.split(query))
 			elif os_name == "WINDOWS":
-				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO %s;\" -c ;" %(db_user)
+				query = get_cmd + " -query \"GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO \\\"%s\\\";\" -c ;" %(db_user)
 				jisql_log(query, db_root_password)
 				ret = subprocessCallWithRetry(query)
 			if ret!=0:
@@ -947,14 +947,14 @@ class PostgresConf(BaseDB):
 						sequence_name , value = each_line.strip().split(" |",1)
 						sequence_name = sequence_name.strip()
 						if is_unix:
-							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SEQUENCE %s TO %s;\"" %(sequence_name,db_user)
+							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SEQUENCE %s TO \\\"%s\\\";\"" %(sequence_name,db_user)
 							jisql_log(query1, db_root_password)
 							ret = subprocessCallWithRetry(shlex.split(query1))
 							if ret != 0:
 								log("[E] Granting all privileges on sequence "+sequence_name+" to user "+db_user+" failed..", "error")
 								sys.exit(1)
 						elif os_name == "WINDOWS":
-							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SEQUENCE %s TO %s;\" -c ;" %(sequence_name,db_user)
+							query1 = get_cmd + " -query \"GRANT ALL PRIVILEGES ON SEQUENCE %s TO \\\"%s\\\";\" -c ;" %(sequence_name,db_user)
 							jisql_log(query1, db_root_password)
 							ret = subprocessCallWithRetry(query1)
 							if ret != 0:
@@ -963,10 +963,10 @@ class PostgresConf(BaseDB):
 
 			log("[I] Granting privileges TO user '"+db_user+"' on db '"+db_name+"' Done" , "info")
 		else:
-			logFile("GRANT ALL PRIVILEGES ON DATABASE %s to %s;" %(db_name, db_user))
-			logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO %s;" %( db_user))
-			logFile("GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO %s;" %(db_user))
-			logFile("GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO %s;" %(db_user))
+			logFile("GRANT ALL PRIVILEGES ON DATABASE %s to \"%s\";" %(db_name, db_user))
+			logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO \"%s\";" %( db_user))
+			logFile("GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO \"%s\";" %(db_user))
+			logFile("GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO \"%s\";" %(db_user))
 
 	def create_auditdb_user(self, xa_db_host, audit_db_host, db_name, audit_db_name, xa_db_root_user, audit_db_root_user, db_user, audit_db_user, xa_db_root_password, audit_db_root_password, db_password, audit_db_password, DBA_MODE,dryMode):
 		if DBA_MODE == "TRUE":
@@ -981,23 +981,23 @@ class PostgresConf(BaseDB):
 
 	def writeDrymodeCmd(self, xa_db_host, audit_db_host, xa_db_root_user, xa_db_root_password, db_user, db_password, db_name, audit_db_root_user, audit_db_root_password, audit_db_user, audit_db_password, audit_db_name):
 		logFile("# Login to POSTGRES Server from a POSTGRES dba user(i.e 'postgres') to execute below sql statements.")
-		logFile("CREATE USER %s WITH LOGIN PASSWORD '%s';" %(db_user, db_password))
-		logFile("CREATE DATABASE %s WITH OWNER %s;" %(db_name, db_user))
+		logFile("CREATE USER \"%s\" WITH LOGIN PASSWORD '%s';" %(db_user, db_password))
+		logFile("CREATE DATABASE \"%s\" WITH OWNER \"%s\";" %(db_name, db_user))
 		logFile("# Login to POSTGRES Server from a POSTGRES dba user(i.e 'postgres') on '%s' database to execute below sql statements."%(db_name))
-		logFile("GRANT ALL PRIVILEGES ON DATABASE %s TO %s;" %(db_name, db_user))
-		logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO %s;" %(db_user))
+		logFile("GRANT ALL PRIVILEGES ON DATABASE \"%s\" TO \"%s\";" %(db_name, db_user))
+		logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO \"%s\";" %(db_user))
 		if not db_user == audit_db_user:
 			logFile("# Login to POSTGRES Server from a POSTGRES dba user(i.e 'postgres') to execute below sql statements.")
-			logFile("CREATE USER %s WITH LOGIN PASSWORD '%s';" %(audit_db_user, audit_db_password))
+			logFile("CREATE USER \"%s\" WITH LOGIN PASSWORD \"%s\";" %(audit_db_user, audit_db_password))
 		if not db_name == audit_db_name:
 			if not db_user == audit_db_user:
 				pass
 			else:
 				logFile("# Login to POSTGRES Server from a POSTGRES dba user(i.e 'postgres') to execute below sql statements.")
-			logFile("CREATE DATABASE %s WITH OWNER %s;" %(audit_db_name, db_user))
+			logFile("CREATE DATABASE \"%s\" WITH OWNER \"%s\";" %(audit_db_name, db_user))
 			logFile("# Login to POSTGRES Server from a POSTGRES dba user(i.e 'postgres') on '%s' database to execute below sql statements."%(audit_db_name))
-			logFile("GRANT ALL PRIVILEGES ON DATABASE %s TO %s;" %(audit_db_name, db_user))
-			logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO %s;" %(db_user))
+			logFile("GRANT ALL PRIVILEGES ON DATABASE \"%s\" TO \"%s\";" %(audit_db_name, db_user))
+			logFile("GRANT ALL PRIVILEGES ON SCHEMA public TO \"%s\";" %(db_user))
 
 class SqlServerConf(BaseDB):
 	# Constructor
@@ -1729,8 +1729,6 @@ def main(argv):
 		xa_patch_file = os.path.join(RANGER_ADMIN_HOME,oracle_patches)
 
 	elif XA_DB_FLAVOR == "POSTGRES":
-		db_user=db_user.lower()
-		db_name=db_name.lower()
 		POSTGRES_CONNECTOR_JAR=CONNECTOR_JAR
 		xa_sqlObj = PostgresConf(xa_db_host, POSTGRES_CONNECTOR_JAR, JAVA_BIN,db_ssl_enabled,db_ssl_required,db_ssl_verifyServerCertificate,javax_net_ssl_keyStore,javax_net_ssl_keyStorePassword,javax_net_ssl_trustStore,javax_net_ssl_trustStorePassword,db_ssl_auth_type)
 		xa_db_version_file = os.path.join(RANGER_ADMIN_HOME,postgres_dbversion_catalog)
@@ -1772,8 +1770,6 @@ def main(argv):
 		audit_db_file = os.path.join(RANGER_ADMIN_HOME,oracle_audit_file)
 
 	elif AUDIT_DB_FLAVOR == "POSTGRES":
-		audit_db_user=audit_db_user.lower()
-		audit_db_name=audit_db_name.lower()
 		POSTGRES_CONNECTOR_JAR=CONNECTOR_JAR
 		audit_sqlObj = PostgresConf(audit_db_host,POSTGRES_CONNECTOR_JAR,JAVA_BIN,db_ssl_enabled,db_ssl_required,db_ssl_verifyServerCertificate,javax_net_ssl_keyStore,javax_net_ssl_keyStorePassword,javax_net_ssl_trustStore,javax_net_ssl_trustStorePassword,db_ssl_auth_type)
 		audit_db_file = os.path.join(RANGER_ADMIN_HOME,postgres_audit_file)
