@@ -26,6 +26,8 @@ from os.path import basename
 from subprocess import Popen,PIPE
 from datetime import date
 from datetime import datetime
+try: input = raw_input
+except NameError: pass
 globalDict = {}
 
 os_name = platform.system()
@@ -82,13 +84,13 @@ def logFile(msg):
 						f.write(msg+"\n")
 						f.close()
 				else:
-					print("Unable to open file "+logFileName+" in write mode, Check file permissions.")
+					log("[E] Unable to open file "+logFileName+" in write mode, Check file permissions.", "error")
 					sys.exit()
 			else:
-				print(logFileName+" is Invalid input file name! Provide valid file path to write DBA scripts:")
+				log("[E] "+ logFileName+" is Invalid input file name! Provide valid file path to write DBA scripts:", "error")
 				sys.exit()
 		else:
-			print("Invalid input! Provide file path to write DBA scripts:")
+			log("[E] Invalid input! Provide file path to write DBA scripts:", "error")
 			sys.exit()
 
 class BaseDB(object):
@@ -303,10 +305,10 @@ def main(argv):
 			JAVA_BIN = JAVA_BIN+'.exe'
 		if os.path.isfile(JAVA_BIN):
 			pass
-		else :	
+		else :
 			while os.path.isfile(JAVA_BIN) == False:
 				log("Enter java executable path: :","info")
-				JAVA_BIN=raw_input()
+				JAVA_BIN=input()
 	log("[I] Using Java:" + str(JAVA_BIN),"info")
 
 	if (quiteMode):
@@ -316,7 +318,7 @@ def main(argv):
 		XA_DB_FLAVOR=''
 		while XA_DB_FLAVOR == "":
 			log("Enter db flavour{MYSQL} :","info")
-			XA_DB_FLAVOR=raw_input()
+			XA_DB_FLAVOR=input()
 			AUDIT_DB_FLAVOR = XA_DB_FLAVOR
 			XA_DB_FLAVOR = XA_DB_FLAVOR.upper()
 			AUDIT_DB_FLAVOR = AUDIT_DB_FLAVOR.upper()
@@ -328,10 +330,10 @@ def main(argv):
 	else:
 		if XA_DB_FLAVOR == "MYSQL":
 			log("Enter JDBC connector file for :"+XA_DB_FLAVOR,"info")
-			CONNECTOR_JAR=raw_input()
+			CONNECTOR_JAR=input()
 			while os.path.isfile(CONNECTOR_JAR) == False:
 				log("JDBC connector file "+CONNECTOR_JAR+" does not exist, Please enter connector path :","error")
-				CONNECTOR_JAR=raw_input()
+				CONNECTOR_JAR=input()
 		else:
 			log("[E] ---------- NO SUCH SUPPORTED DB FLAVOUR.. ----------", "error")
 			sys.exit(1)
@@ -343,7 +345,7 @@ def main(argv):
 		xa_db_host=''
 		while xa_db_host == "":
 			log("Enter DB Host :","info")
-			xa_db_host=raw_input()
+			xa_db_host=input()
 			audit_db_host=xa_db_host
 	log("[I] DB Host:" + str(xa_db_host),"info")
 
@@ -354,7 +356,7 @@ def main(argv):
 		xa_db_root_user=''
 		while xa_db_root_user == "":
 			log("Enter db root user:","info")
-			xa_db_root_user=raw_input()
+			xa_db_root_user=input()
 			log("Enter db root password:","info")
 			xa_db_root_password = getpass.getpass("Enter db root password:")
 
@@ -364,7 +366,7 @@ def main(argv):
 		db_name = ''
 		while db_name == "":
 			log("Enter DB Name :","info")
-			db_name=raw_input()
+			db_name=input()
 
 	if (quiteMode):
 		db_user = globalDict['db_user']
@@ -372,7 +374,7 @@ def main(argv):
 		db_user=''
 		while db_user == "":
 			log("Enter db user name:","info")
-			db_user=raw_input()
+			db_user=input()
 
 	if (quiteMode):
 		db_password = globalDict['db_password']
@@ -388,7 +390,7 @@ def main(argv):
 		audit_db_name=''
 		while audit_db_name == "":
 			log("Enter audit db name:","info")
-			audit_db_name = raw_input()
+			audit_db_name = input()
 
 	if (quiteMode):
 		audit_db_user = globalDict['audit_db_user']
@@ -396,7 +398,7 @@ def main(argv):
 		audit_db_user=''
 		while audit_db_user == "":
 			log("Enter audit user name:","info")
-			audit_db_user = raw_input()
+			audit_db_user = input()
 
 	if (quiteMode):
 		audit_db_password = globalDict['audit_db_password']
