@@ -37,12 +37,10 @@ import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
 import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.ContextUtil;
 import org.apache.ranger.common.DateUtil;
-import org.apache.ranger.common.GUIDUtil;
 import org.apache.ranger.common.MessageEnums;
 import org.apache.ranger.common.PropertiesUtil;
 import org.apache.ranger.common.RESTErrorUtil;
 import org.apache.ranger.common.RangerCommonEnums;
-import org.apache.ranger.common.RangerConfigUtil;
 import org.apache.ranger.common.RangerConstants;
 import org.apache.ranger.common.SearchCriteria;
 import org.apache.ranger.common.SearchUtil;
@@ -52,7 +50,7 @@ import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.entity.XXGroupPermission;
 import org.apache.ranger.entity.XXPortalUser;
 import org.apache.ranger.entity.XXPortalUserRole;
-import org.apache.ranger.entity.XXTrxLog;
+import org.apache.ranger.entity.XXTrxLogV2;
 import org.apache.ranger.entity.XXUser;
 import org.apache.ranger.entity.XXUserPermission;
 import org.apache.ranger.service.XGroupPermissionService;
@@ -74,6 +72,8 @@ import org.springframework.transaction.annotation.Transactional;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.ranger.service.RangerBaseModelService.OPERATION_UPDATE_CONTEXT;
+
 @Component
 public class UserMgr {
 
@@ -91,17 +91,11 @@ public class UserMgr {
 	SearchUtil searchUtil;
 
 	@Autowired
-        RangerBizUtil rangerBizUtil;
+	RangerBizUtil rangerBizUtil;
 
 	@Autowired
 	SessionMgr sessionMgr;
 
-	@Autowired
-	DateUtil dateUtil;
-
-	@Autowired
-	RangerConfigUtil configUtil;
-
 	@Autowired
 	XPortalUserService xPortalUserService;
 
@@ -114,9 +108,6 @@ public class UserMgr {
 	@Autowired
 	XUserMgr xUserMgr;
 
-	@Autowired
-	GUIDUtil guidUtil;
-
 	private final boolean isFipsEnabled;
 	private static final int DEFAULT_PASSWORD_HISTORY_COUNT = 4;
 	private int passwordHistoryCount = PropertiesUtil.getIntProperty("ranger.password.history.count", DEFAULT_PASSWORD_HISTORY_COUNT);
@@ -206,127 +197,76 @@ public class UserMgr {
 	 * @return
 	 */
 	public XXPortalUser updateUser(VXPortalUser userProfile) {
-		XXPortalUser gjUser = daoManager.getXXPortalUser().getById(
-				userProfile.getId());
+		XXPortalUser gjUser = daoManager.getXXPortalUser().getById(userProfile.getId());
 
 		if (gjUser == null) {
-			logger.error("updateUser(). User not found. userProfile="
-					+ userProfile);
+			logger.error("updateUser(). User not found. userProfile=" + userProfile);
 			return null;
 		}
 
 		checkAccess(gjUser);
-                rangerBizUtil.blockAuditorRoleUser();
-		boolean updateUser = false;
+		rangerBizUtil.blockAuditorRoleUser();
+
+		VXPortalUser existing = xPortalUserService.populateViewBean(gjUser);
+
 		// Selectively update fields
 
-		// status
-		if (userProfile.getStatus() != gjUser.getStatus()) {
-			updateUser = true;
-		}
-
 		// Allowing email address update even when its set to empty.
-		// emailAddress
 		String emailAddress = userProfile.getEmailAddress();
 		if (stringUtil.isEmpty(emailAddress)) {
 			userProfile.setEmailAddress(null);
-			updateUser = true;
 		} else {
 			if (stringUtil.validateEmail(emailAddress)) {
-				XXPortalUser checkUser = daoManager.getXXPortalUser()
-						.findByEmailAddress(emailAddress);
+				XXPortalUser checkUser = daoManager.getXXPortalUser().findByEmailAddress(emailAddress);
 				if (checkUser != null) {
 					String loginId = userProfile.getLoginId();
 					if (loginId == null) {
 						throw restErrorUtil.createRESTException(
-								"Invalid user, please provide valid "
-										+ "username.",
-								MessageEnums.INVALID_INPUT_DATA);
+								"Invalid user, please provide valid username.", MessageEnums.INVALID_INPUT_DATA);
 					} else if (!loginId.equals(checkUser.getLoginId())) {
-						throw restErrorUtil
-								.createRESTException(
-										"The email address "
-												+ "you've provided already exists in system.",
-										MessageEnums.INVALID_INPUT_DATA);
+						throw restErrorUtil.createRESTException(
+								"The email address you've provided already exists in system.", MessageEnums.INVALID_INPUT_DATA);
 					} else {
 						userProfile.setEmailAddress(emailAddress);
-						updateUser = true;
 					}
 				} else {
 					userProfile.setEmailAddress(emailAddress);
-					updateUser = true;
 				}
 			} else {
-				throw restErrorUtil.createRESTException(
-						"Please provide valid email address.",
-						MessageEnums.INVALID_INPUT_DATA);
+				throw restErrorUtil.createRESTException("Please provide valid email address.", MessageEnums.INVALID_INPUT_DATA);
 			}
 		}
 
-		// loginId
-		// if (!stringUtil.isEmpty(userProfile.getLoginId())
-		// && !userProfile.getLoginId().equals(gjUser.getLoginId())) {
-		// gjUser.setLoginId(userProfile.getLoginId());
-		// updateUser = true;
-		// }
-
 		// firstName
 		if("null".equalsIgnoreCase(userProfile.getFirstName())){
 			userProfile.setFirstName("");
 		}
-		if (!stringUtil.isEmpty(userProfile.getFirstName())
-				&& !userProfile.getFirstName().equals(gjUser.getFirstName())) {
-			userProfile.setFirstName(stringUtil.toCamelCaseAllWords(userProfile
-					.getFirstName()));
-			updateUser = true;
+		if (!stringUtil.isEmpty(userProfile.getFirstName()) && !userProfile.getFirstName().equals(gjUser.getFirstName())) {
+			userProfile.setFirstName(stringUtil.toCamelCaseAllWords(userProfile.getFirstName()));
 		}
-
 		if("null".equalsIgnoreCase(userProfile.getLastName())){
 			userProfile.setLastName("");
 		}
-		if (!stringUtil.isEmpty(userProfile.getLastName())
-				&& !userProfile.getLastName().equals(gjUser.getLastName())) {
-			userProfile.setLastName(stringUtil.toCamelCaseAllWords(userProfile
-					.getLastName()));
-			updateUser = true;
+		if (!stringUtil.isEmpty(userProfile.getLastName()) && !userProfile.getLastName().equals(gjUser.getLastName())) {
+			userProfile.setLastName(stringUtil.toCamelCaseAllWords(userProfile.getLastName()));
 		}
 
 		// publicScreenName
-		if (userProfile.getFirstName() != null
-				&& userProfile.getLastName() != null
-				&& !userProfile.getFirstName().trim().isEmpty()
+		if (userProfile.getFirstName() != null && userProfile.getLastName() != null && !userProfile.getFirstName().trim().isEmpty()
 				&& !userProfile.getLastName().trim().isEmpty()) {
-			userProfile.setPublicScreenName(userProfile.getFirstName() + " "
-					+ userProfile.getLastName());
-			updateUser = true;
+			userProfile.setPublicScreenName(userProfile.getFirstName() + " " + userProfile.getLastName());
 		} else {
 			userProfile.setPublicScreenName(gjUser.getLoginId());
-			updateUser = true;
 		}
 
-		// notes
-		/*
-		 * if (!stringUtil.isEmpty(userProfile.getNotes()) &&
-		 * !userProfile.getNotes().equalsIgnoreCase(gjUser.getNotes())) {
-		 * updateUser = true; }
-		 */
-
 		// userRoleList
 		updateRoles(userProfile.getId(), userProfile.getUserRoleList());
 
-		if (updateUser) {
-
-			List<XXTrxLog> trxLogList = xPortalUserService.getTransactionLog(
-					userProfile, gjUser, "update");
-
-			userProfile.setPassword(gjUser.getPassword());
-			xPortalUserService.updateResource(userProfile);
-			sessionMgr.resetUserSessionForProfiles(ContextUtil
-					.getCurrentUserSession());
-
-                        rangerBizUtil.createTrxLog(trxLogList);
-		}
-
+		List<XXTrxLogV2> trxLogList = xPortalUserService.getTransactionLog(userProfile, existing, OPERATION_UPDATE_CONTEXT);
+		userProfile.setPassword(gjUser.getPassword());
+		xPortalUserService.updateResource(userProfile);
+		sessionMgr.resetUserSessionForProfiles(ContextUtil.getCurrentUserSession());
+		rangerBizUtil.createTrxLog(trxLogList);
 		return gjUser;
 	}
 
@@ -468,17 +408,8 @@ public class UserMgr {
 			}
 		}
 			if (isNewPasswordDifferent) {
-				List<XXTrxLog> trxLogList = new ArrayList<XXTrxLog>();
-				XXTrxLog xTrxLog = new XXTrxLog();
-				xTrxLog.setAttributeName("Password");
-				xTrxLog.setPreviousValue(currentPassword);
-				xTrxLog.setNewValue(encryptedNewPwd);
-				xTrxLog.setAction("password change");
-				xTrxLog.setObjectClassType(AppConstants.CLASS_TYPE_PASSWORD_CHANGE);
-				xTrxLog.setObjectId(pwdChange.getId());
-				xTrxLog.setObjectName(pwdChange.getLoginId());
-				trxLogList.add(xTrxLog);
-	                        rangerBizUtil.createTrxLog(trxLogList);
+				xPortalUserService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_PASSWORD_CHANGE, pwdChange.getId(), pwdChange.getLoginId(), "password change"), "Password", currentPassword, encryptedNewPwd);
+
 				gjUser.setPassword(encryptedNewPwd);
 				updateOldPasswords(gjUser, oldPasswords);
 				gjUser = daoManager.getXXPortalUser().update(gjUser);
@@ -504,56 +435,43 @@ public class UserMgr {
 	 * @param changeEmail
 	 * @return
 	 */
-	public VXPortalUser changeEmailAddress(XXPortalUser gjUser,
-			VXPasswordChange changeEmail) {
+	public VXPortalUser changeEmailAddress(XXPortalUser gjUser, VXPasswordChange changeEmail) {
 		checkAccessForUpdate(gjUser);
-                rangerBizUtil.blockAuditorRoleUser();
+		rangerBizUtil.blockAuditorRoleUser();
 		if (StringUtils.isEmpty(changeEmail.getEmailAddress())) {
 			changeEmail.setEmailAddress(null);
 		}
 
 		if (!StringUtils.isEmpty(changeEmail.getEmailAddress()) && !stringUtil.validateEmail(changeEmail.getEmailAddress())) {
 			logger.info("Invalid email address." + changeEmail);
-			throw restErrorUtil.createRESTException(
-					"serverMsg.userMgrInvalidEmail",
-					MessageEnums.INVALID_INPUT_DATA, changeEmail.getId(),
-					"emailAddress", changeEmail.toString());
-
+			throw restErrorUtil.createRESTException("serverMsg.userMgrInvalidEmail",
+					MessageEnums.INVALID_INPUT_DATA, changeEmail.getId(), "emailAddress", changeEmail.toString());
 		}
-		
+
 		if (this.isFipsEnabled) {
 			if (!isPasswordValid(changeEmail.getLoginId(), gjUser.getPassword(), changeEmail.getOldPassword())) {
-				logger.info("changeEmailAddress(). Invalid  password. changeEmail="
-								+ changeEmail);
-								throw restErrorUtil.createRESTException(
-											"serverMsg.userMgrWrongPassword",
-												MessageEnums.OPER_NO_PERMISSION, null, null, ""
-														+ changeEmail);
-					}
+				logger.info("changeEmailAddress(). Invalid  password. changeEmail=" + changeEmail);
+				throw restErrorUtil.createRESTException("serverMsg.userMgrWrongPassword",
+												MessageEnums.OPER_NO_PERMISSION, null, null, "" + changeEmail);
+			}
 		} else {
 			String encryptedOldPwd = encrypt(gjUser.getLoginId(), changeEmail.getOldPassword());
 			if (!stringUtil.equals(encryptedOldPwd, gjUser.getPassword())) {
-				encryptedOldPwd = encryptWithOlderAlgo(gjUser.getLoginId(), changeEmail.getOldPassword());
-				if (!stringUtil.equals(encryptedOldPwd, gjUser.getPassword())) {
-					logger.info("changeEmailAddress(). Invalid  password. changeEmail=" + changeEmail);
-					throw restErrorUtil.createRESTException("serverMsg.userMgrWrongPassword",
-							MessageEnums.OPER_NO_PERMISSION, null, null, "" + changeEmail);
-				}
+				logger.info("changeEmailAddress(). Invalid  password. changeEmail=" + changeEmail);
+				throw restErrorUtil.createRESTException("serverMsg.userMgrWrongPassword",
+						MessageEnums.OPER_NO_PERMISSION, null, null, "" + changeEmail);
 			}
 		}
 
 		// Normalize email. Make it lower case
-		gjUser.setEmailAddress(stringUtil.normalizeEmail(changeEmail
-				.getEmailAddress()));
+		gjUser.setEmailAddress(stringUtil.normalizeEmail(changeEmail.getEmailAddress()));
 
-		String saltEncodedpasswd = encrypt(gjUser.getLoginId(),
-				changeEmail.getOldPassword());
-        if (gjUser.getUserSource() == RangerCommonEnums.USER_APP) {
-		gjUser.setPassword(saltEncodedpasswd);
-       }
-        else if (gjUser.getUserSource() == RangerCommonEnums.USER_EXTERNAL) {
-                gjUser.setPassword(gjUser.getPassword());
-        }
+		String saltEncodedpasswd = encrypt(gjUser.getLoginId(), changeEmail.getOldPassword());
+		if (gjUser.getUserSource() == RangerCommonEnums.USER_APP) {
+			gjUser.setPassword(saltEncodedpasswd);
+		} else if (gjUser.getUserSource() == RangerCommonEnums.USER_EXTERNAL) {
+			gjUser.setPassword(gjUser.getPassword());
+		}
 		daoManager.getXXPortalUser().update(gjUser);
 		return mapXXPortalUserVXPortalUser(gjUser);
 	}
@@ -1406,19 +1324,7 @@ public class UserMgr {
                 if(xXPortalUser!=null && logAudits){
                         String dbNewPwd=xXPortalUser.getPassword();
                         if (!dbOldPwd.equals(dbNewPwd)) {
-                                List<XXTrxLog> trxLogList = new ArrayList<XXTrxLog>();
-                                XXTrxLog xTrxLog = new XXTrxLog();
-                                xTrxLog.setAttributeName("Password");
-                                xTrxLog.setPreviousValue(dbOldPwd);
-                                xTrxLog.setNewValue(dbNewPwd);
-                                xTrxLog.setAction("password change");
-                                xTrxLog.setObjectClassType(AppConstants.CLASS_TYPE_PASSWORD_CHANGE);
-                                xTrxLog.setObjectId(xXPortalUser.getId());
-                                xTrxLog.setObjectName(xXPortalUser.getLoginId());
-                                xTrxLog.setAddedByUserId(xXPortalUser.getId());
-                                xTrxLog.setUpdatedByUserId(xXPortalUser.getId());
-                                trxLogList.add(xTrxLog);
-                                rangerBizUtil.createTrxLog(trxLogList);
+							xPortalUserService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_PASSWORD_CHANGE, xXPortalUser.getId(), xXPortalUser.getLoginId(), "password change"), "Password", dbOldPwd, dbNewPwd);
                         }
                 }
 
@@ -1485,19 +1391,9 @@ public class UserMgr {
                     xXPortalUser.setPassword(xXPortalUser.getPassword());
                 }
                 xXPortalUser = daoManager.getXXPortalUser().update(xXPortalUser);
-                List<XXTrxLog> trxLogList = new ArrayList<XXTrxLog>();
-                XXTrxLog xTrxLog = new XXTrxLog();
-                xTrxLog.setAttributeName("User Name");
-                xTrxLog.setPreviousValue(userLoginId);
-                xTrxLog.setNewValue(newUserName);
-                xTrxLog.setAction("update");
-                xTrxLog.setObjectClassType(AppConstants.CLASS_TYPE_USER_PROFILE);
-                xTrxLog.setObjectId(xXPortalUser.getId());
-                xTrxLog.setObjectName(xXPortalUser.getLoginId());
-                xTrxLog.setAddedByUserId(xXPortalUser.getId());
-                xTrxLog.setUpdatedByUserId(xXPortalUser.getId());
-                trxLogList.add(xTrxLog);
-                rangerBizUtil.createTrxLog(trxLogList);
+
+				xPortalUserService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_USER_PROFILE, xXPortalUser.getId(), xXPortalUser.getLoginId(), "update"), "User Name", userLoginId, newUserName);
+
                 return xXPortalUser;
         }
         public boolean isPasswordValid(String loginId, String encodedPassword, String password) {
