@@ -22,6 +22,7 @@ package org.apache.ranger.common;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
+import org.apache.ranger.authorization.utils.JsonUtils;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.store.ServiceStore;
 
@@ -30,7 +31,12 @@ import org.apache.ranger.plugin.util.ServicePolicies;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.File;
+import java.io.FileFilter;
+import java.io.FileWriter;
+import java.io.Writer;
 import java.util.ArrayList;
+import java.util.Comparator;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
@@ -49,7 +55,6 @@ public class RangerServicePoliciesCache {
 
 	private final int     waitTimeInSeconds;
 	private final boolean dedupStrings;
-
 	private final Map<String, ServicePoliciesWrapper> servicePoliciesMap = new HashMap<>();
 
 	public static RangerServicePoliciesCache getInstance() {
@@ -130,7 +135,6 @@ public class RangerServicePoliciesCache {
 				LOG.error("getServicePolicies(" + serviceName + "): failed to get latest policies as service-store is null! Returning cached servicePolicies!");
 				ret = servicePoliciesWrapper.getServicePolicies();
 			}
-
 		} else {
 			LOG.error("getServicePolicies() failed to get policies as serviceName is null or blank and/or serviceId is null!");
 		}
@@ -186,6 +190,100 @@ public class RangerServicePoliciesCache {
         return ret;
     }
 
+	public void saveToCache(ServicePolicies policies) {
+		if(LOG.isDebugEnabled()) {
+			LOG.debug("==> RangerServicePoliciesCache(serviceName=" + policies.getServiceName() + ").saveToCache()");
+		}
+		if (policies != null) {
+			RangerAdminConfig config = RangerAdminConfig.getInstance();
+			boolean doSaveToDisk = config.getBoolean("ranger.admin.policy.save.to.disk", false);
+			int maxVersionsToSaveToDisk = config.getInt("ranger.admin.policy.max.versions.to.save.to.disk", 1);
+
+			if (doSaveToDisk) {
+				File cacheFile = null;
+
+				String cacheDir = config.get("ranger.admin.policy.cache.dir");
+				if (cacheDir != null) {
+					String appId = policies.getServiceDef().getName();
+					String serviceName = policies.getServiceName();
+					String cacheFileName = String.format("%s_%s.json", appId, serviceName);
+
+					cacheFileName = cacheFileName.replace(File.separatorChar, '_');
+					cacheFileName = cacheFileName.replace(File.pathSeparatorChar, '_');
+					cacheFileName = cacheFileName + "_" + policies.getPolicyVersion();
+
+					// Create the cacheDir if it doesn't already exist
+					File cacheDirTmp = new File(cacheDir);
+					if (cacheDirTmp.exists()) {
+						cacheFile = new File(cacheDir + File.separator + cacheFileName);
+					} else {
+						try {
+							cacheDirTmp.mkdirs();
+							cacheFile = new File(cacheDir + File.separator + cacheFileName);
+						} catch (SecurityException ex) {
+							LOG.error("Cannot create cache directory", ex);
+						}
+					}
+				}
+
+				if (cacheFile != null) {
+					try (Writer writer = new FileWriter(cacheFile)) {
+                        JsonUtils.objectToWriter(writer, policies);
+					} catch (Exception excp) {
+						LOG.error("failed to save policies to cache file '" + cacheFile.getAbsolutePath() + "'", excp);
+					}
+					String serviceDefName = policies.getServiceDef().getName();
+					String serviceName    = policies.getServiceName();
+
+					File parentFile = cacheFile.getParentFile();
+					FileFilter logFileFilter = (file) -> file.getName().matches(serviceDefName +"_.+json_.+");
+					File[] filesInParent = parentFile.listFiles(logFileFilter);
+					List<Long> policyVersions = new ArrayList<>();
+					if (filesInParent != null && filesInParent.length > 0) {
+						for (File f : filesInParent) {
+							String fileName = f.getName();
+							// Extract the part after json_
+							int policyVersionIdx = fileName.lastIndexOf("json_");
+							String policyVersionStr = fileName.substring(policyVersionIdx + 5);
+							Long policyVersion = Long.valueOf(policyVersionStr);
+							policyVersions.add(policyVersion);
+						}
+					} else {
+						LOG.info("No files matching '" + serviceDefName + "_.+json_*' found");
+					}
+					if (!policyVersions.isEmpty()) {
+						policyVersions.sort(new Comparator<Long>() {
+							@Override
+							public int compare(Long o1, Long o2) {
+								if (o1.equals(o2)) return 0;
+								return o1 < o2 ? -1 : 1;
+							}
+						});
+					}
+
+					if (policyVersions.size() > maxVersionsToSaveToDisk) {
+						String fileName = serviceDefName + "_" + serviceName + ".json_" + Long.toString(policyVersions.get(0));
+						String pathName = parentFile.getAbsolutePath() + File.separator + fileName;
+						File toDelete = new File(pathName);
+						if (toDelete.exists()) {
+							//LOG.info("Deleting file :[" + pathName + "]");
+							boolean isDeleted = toDelete.delete();
+							//LOG.info("file :[" + pathName + "] is deleted");
+						} else {
+							LOG.info("File: " + pathName + " does not exist!");
+						}
+					}
+				}
+			}
+		} else {
+			LOG.error("ServicePolicies is null object!!");
+		}
+
+		if(LOG.isDebugEnabled()) {
+			LOG.debug("<== RangerServicePoliciesCache(serviceName=" + policies.getServiceName() + ").saveToCache()");
+		}
+	}
+
 	private class ServicePoliciesWrapper {
 		final Long          serviceId;
 		ServicePolicies     servicePolicies;
@@ -229,6 +327,7 @@ public class RangerServicePoliciesCache {
 			}
 			ServicePolicies ret        = null;
 			boolean         lockResult = false;
+			boolean         doSaveToCache = false;
 
 			try {
 				final boolean isCacheReloadedByDQEvent;
@@ -238,49 +337,61 @@ public class RangerServicePoliciesCache {
 				if (lockResult) {
 					isCacheReloadedByDQEvent = getLatest(serviceName, serviceStore, lastKnownVersion);
 
-					if (isCacheReloadedByDQEvent) {
-						if (LOG.isDebugEnabled()) {
-							LOG.debug("ServicePolicies cache was completely loaded from database because of a disqualifying event - such as service-definition change!");
-						}
-					}
-
-					if (needsBackwardCompatibility || isCacheReloadedByDQEvent
-						|| lastKnownVersion == -1L || lastKnownVersion.equals(servicePolicies.getPolicyVersion())) {
-						// Looking for all policies, or Some disqualifying change encountered
-						if (LOG.isDebugEnabled()) {
-							LOG.debug("All policies were requested, returning cached ServicePolicies");
-						}
-						ret = this.servicePolicies;
-					} else {
-						boolean         isDeltaCacheReinitialized = false;
-						ServicePolicies servicePoliciesForDeltas  = this.deltaCache != null ? this.deltaCache.getServicePolicyDeltasFromVersion(lastKnownVersion) : null;
-
-						if (servicePoliciesForDeltas == null) {
-							servicePoliciesForDeltas  = serviceStore.getServicePolicyDeltas(serviceName, lastKnownVersion);
-							isDeltaCacheReinitialized = true;
-						}
-						if (servicePoliciesForDeltas != null && servicePoliciesForDeltas.getPolicyDeltas() != null) {
+					if (this.servicePolicies != null) {
+						if (isCacheReloadedByDQEvent) {
 							if (LOG.isDebugEnabled()) {
-								LOG.debug("Deltas were requested. Returning deltas from lastKnownVersion:[" + lastKnownVersion + "]");
+								LOG.debug("ServicePolicies cache was completely loaded from database because of a disqualifying event - such as service-definition change!");
 							}
-							if (isDeltaCacheReinitialized) {
-								this.deltaCache = new ServicePolicyDeltasCache(lastKnownVersion, servicePoliciesForDeltas);
-							}
-							ret = servicePoliciesForDeltas;
-						} else {
-							LOG.warn("Deltas were requested for service:[" + serviceName + "], but could not get them!! lastKnownVersion:[" + lastKnownVersion + "]; Returning cached ServicePolicies:[" + (servicePolicies != null ? servicePolicies.getPolicyVersion() : -1L) + "]");
-
-							this.deltaCache = null;
-							ret = this.servicePolicies;
 						}
+						if (!lastKnownVersion.equals(servicePolicies.getPolicyVersion()) || isCacheReloadedByDQEvent) {
+							doSaveToCache = true;
+						}
+
+						if (needsBackwardCompatibility || isCacheReloadedByDQEvent
+								|| lastKnownVersion == -1L || lastKnownVersion.equals(servicePolicies.getPolicyVersion())) {
+							// Looking for all policies, or Some disqualifying change encountered
+							if (LOG.isDebugEnabled()) {
+								LOG.debug("All policies were requested, returning cached ServicePolicies");
+							}
+							ret = this.servicePolicies;
+						} else {
+							boolean isDeltaCacheReinitialized = false;
+							ServicePolicies servicePoliciesForDeltas = this.deltaCache != null ? this.deltaCache.getServicePolicyDeltasFromVersion(lastKnownVersion) : null;
+
+							if (servicePoliciesForDeltas == null) {
+								servicePoliciesForDeltas = serviceStore.getServicePolicyDeltas(serviceName, lastKnownVersion, servicePolicies.getPolicyVersion());
+								isDeltaCacheReinitialized = true;
+							}
+							if (servicePoliciesForDeltas != null && servicePoliciesForDeltas.getPolicyDeltas() != null) {
+								if (LOG.isDebugEnabled()) {
+									LOG.debug("Deltas were requested. Returning deltas from lastKnownVersion:[" + lastKnownVersion + "]");
+								}
+								if (isDeltaCacheReinitialized) {
+									this.deltaCache = new ServicePolicyDeltasCache(lastKnownVersion, servicePoliciesForDeltas);
+								}
+								ret = servicePoliciesForDeltas;
+							} else {
+								LOG.warn("Deltas were requested for service:[" + serviceName + "], but could not get them!! lastKnownVersion:[" + lastKnownVersion + "]; Returning cached ServicePolicies:[" + (servicePolicies != null ? servicePolicies.getPolicyVersion() : -1L) + "]");
+
+								this.deltaCache = null;
+								ret = this.servicePolicies;
+							}
+						}
+					} else {
+						LOG.error("ServicePolicies object is null!");
 					}
 				} else {
 					LOG.error("Could not get lock in [" + waitTimeInSeconds + "] seconds, returning cached ServicePolicies and wait Queue Length:[" +lock.getQueueLength() + "], servicePolicies version:[" + (servicePolicies != null ? servicePolicies.getPolicyVersion() : -1L) + "]");
 					ret = this.servicePolicies;
+					doSaveToCache = true;
 				}
 			} catch (InterruptedException exception) {
 				LOG.error("getLatestOrCached:lock got interrupted..", exception);
 			} finally {
+				// Dump cached policies to disk
+				if (doSaveToCache) {
+					saveToCache(this.servicePolicies);
+				}
 				if (lockResult) {
 					lock.unlock();
 				}
@@ -322,9 +433,14 @@ public class RangerServicePoliciesCache {
 				if (dbLoadTime > longestDbLoadTimeInMs) {
 					longestDbLoadTimeInMs = dbLoadTime;
 				}
+
 				updateTime = new Date();
 
 				if (servicePoliciesFromDb != null) {
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("loading servicePolicies from database and it took:" + TimeUnit.MILLISECONDS.toSeconds(dbLoadTime) + " seconds");
+					}
+
 					if (dedupStrings) {
 						servicePoliciesFromDb.dedupStrings();
 					}
@@ -351,12 +467,12 @@ public class RangerServicePoliciesCache {
 						if (LOG.isDebugEnabled()) {
 							LOG.debug("Retrieved policy-deltas from database. These will be applied on top of ServicePolicy version:[" + cachedServicePoliciesVersion +"], policy-deltas:[" + servicePoliciesFromDb.getPolicyDeltas() + "]");
 						}
-						servicePolicies.setPolicyVersion(servicePoliciesFromDb.getPolicyVersion());
 
 						final List<RangerPolicy> policies = servicePolicies.getPolicies() == null ? new ArrayList<>() : servicePolicies.getPolicies();
 						final List<RangerPolicy> newPolicies = RangerPolicyDeltaUtil.applyDeltas(policies, servicePoliciesFromDb.getPolicyDeltas(), servicePolicies.getServiceDef().getName());
 
 						servicePolicies.setPolicies(newPolicies);
+						servicePolicies.setPolicyVersion(servicePoliciesFromDb.getPolicyVersion());
 
 						checkCacheSanity(serviceName, serviceStore, false);
 
