@@ -27,7 +27,6 @@ import java.util.Map.Entry;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
-import org.apache.log4j.Logger;
 import org.apache.ranger.biz.UserMgr;
 import org.apache.ranger.biz.XUserMgr;
 import org.apache.ranger.common.RangerConstants;
@@ -38,13 +37,15 @@ import org.apache.ranger.patch.BaseLoader;
 import org.apache.ranger.service.XUserService;
 import org.apache.ranger.util.CLIUtil;
 import org.apache.ranger.view.VXUser;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Component;
 
 @Component
 public class RoleBasedUserSearchUtil extends BaseLoader {
 
-        private static final Logger logger = Logger.getLogger(RoleBasedUserSearchUtil.class);
+        private static final Logger logger = LoggerFactory.getLogger(RoleBasedUserSearchUtil.class);
         @Autowired
         XUserService xUserService;
 
@@ -72,15 +73,9 @@ public class RoleBasedUserSearchUtil extends BaseLoader {
 	                currentPassword = args[1];
 	                if (args.length == 3) {
 	                    userRole = args[2];
-	                    List<String> roles = new ArrayList<String>();
-	                    roles.add(RangerConstants.ROLE_USER);
-	                    roles.add(RangerConstants.ROLE_SYS_ADMIN);
-	                    roles.add(RangerConstants.ROLE_KEY_ADMIN);
-                        roles.add(RangerConstants.ROLE_ADMIN_AUDITOR);
-                        roles.add(RangerConstants.ROLE_KEY_ADMIN_AUDITOR);
                         if (!StringUtils.isBlank(userRole)) {
 	                        userRole = userRole.toUpperCase();
-	                        if (!roles.contains(userRole)) {
+	                        if (!RangerConstants.VALID_USER_ROLE_LIST.contains(userRole)) {
 	                            System.out.println("Invalid UserRole. Exiting!!!");
 	                            logger.info("Invalid UserRole. Exiting!!!");
 	                            System.exit(1);
@@ -297,4 +292,4 @@ public class RoleBasedUserSearchUtil extends BaseLoader {
                 System.exit(1);
             }
         }
-}
\ No newline at end of file
+}
