@@ -23,9 +23,6 @@ import java.util.Map;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.log4j.Logger;
 import org.apache.ranger.authorization.utils.JsonUtils;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.apache.ranger.biz.PolicyRefUpdater;
@@ -46,6 +43,8 @@ import org.apache.ranger.plugin.model.RangerService;
 import org.apache.ranger.plugin.util.RangerPerfTracer;
 import org.apache.ranger.plugin.util.SearchFilter;
 import org.apache.ranger.util.CLIUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.beans.factory.annotation.Qualifier;
 import org.springframework.stereotype.Component;
@@ -61,7 +60,7 @@ import org.springframework.transaction.support.TransactionTemplate;
  */
 @Component
 public class UpdateUserAndGroupNamesInJson extends BaseLoader {
-	private static final Logger logger = Logger.getLogger(UpdateUserAndGroupNamesInJson.class);
+	private static final Logger logger = LoggerFactory.getLogger(UpdateUserAndGroupNamesInJson.class);
 
 	@Autowired
 	RangerDaoManager daoMgr;
@@ -300,8 +299,8 @@ public class UpdateUserAndGroupNamesInJson extends BaseLoader {
 	}
 
 	static private class RangerPolicyRetriever {
-		static final Log LOG = LogFactory.getLog(RangerPolicyRetriever.class);
-		static final Log PERF_LOG = RangerPerfTracer.getPerfLogger("db.RangerPolicyRetriever");
+		static final Logger LOG = LoggerFactory.getLogger(RangerPolicyRetriever.class);
+		static final Logger PERF_LOG = RangerPerfTracer.getPerfLogger("db.RangerPolicyRetriever");
 		private final RangerDaoManager daoMgr;
 		private final PlatformTransactionManager txManager;
 		private final TransactionTemplate txTemplate;
