@@ -19,6 +19,8 @@
 
 package org.apache.ranger.rest;
 
+import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
@@ -30,15 +32,18 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.ws.rs.*;
 import javax.ws.rs.core.Context;
+import javax.ws.rs.core.MediaType;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.io.IOUtils;
 import org.apache.ranger.admin.client.datatype.RESTResponse;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.apache.ranger.biz.AssetMgr;
 import org.apache.ranger.biz.RangerBizUtil;
 import org.apache.ranger.biz.RoleDBStore;
 import org.apache.ranger.biz.ServiceDBStore;
+import org.apache.ranger.biz.ServiceDBStore.JSON_FILE_NAME_TYPE;
 import org.apache.ranger.biz.XUserMgr;
 import org.apache.ranger.common.RESTErrorUtil;
 import org.apache.ranger.common.RangerSearchUtil;
@@ -46,10 +51,14 @@ import org.apache.ranger.common.RangerValidatorFactory;
 import org.apache.ranger.common.ServiceUtil;
 import org.apache.ranger.common.UserSessionBase;
 import org.apache.ranger.common.PropertiesUtil;
+import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.ContextUtil;
+import org.apache.ranger.common.MessageEnums;
 import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.entity.XXService;
 import org.apache.ranger.entity.XXServiceDef;
+import org.apache.ranger.entity.XXTrxLogV2;
+import org.apache.ranger.plugin.errors.ValidationErrorCode;
 import org.apache.ranger.plugin.model.RangerPluginInfo;
 import org.apache.ranger.plugin.model.RangerRole;
 import org.apache.ranger.plugin.model.RangerService;
@@ -58,20 +67,27 @@ import org.apache.ranger.plugin.model.validation.RangerValidator;
 import org.apache.ranger.plugin.policyengine.RangerPolicyEngine;
 import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;
 import org.apache.ranger.plugin.util.GrantRevokeRoleRequest;
+import org.apache.ranger.plugin.util.JsonUtilsV2;
 import org.apache.ranger.plugin.util.RangerRESTUtils;
 import org.apache.ranger.plugin.util.RangerRoles;
 import org.apache.ranger.plugin.util.SearchFilter;
+import org.apache.ranger.security.context.RangerContextHolder;
 import org.apache.ranger.service.RangerRoleService;
 import org.apache.ranger.service.XUserService;
+import org.apache.ranger.view.RangerExportRoleList;
 import org.apache.ranger.view.RangerRoleList;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.context.annotation.Scope;
+import org.springframework.security.access.prepost.PreAuthorize;
 import org.springframework.stereotype.Component;
 import org.springframework.transaction.annotation.Propagation;
 import org.springframework.transaction.annotation.Transactional;
 
+import com.sun.jersey.multipart.FormDataParam;
+import com.sun.jersey.core.header.FormDataContentDisposition;
+
 @Path("roles")
 @Component
 @Scope("request")
@@ -82,6 +98,8 @@ public class RoleREST {
     private static List<String> INVALID_USERS = new ArrayList<>();
 
     public static final String POLICY_DOWNLOAD_USERS = "policy.download.auth.users";
+    public static final String PARAM_ROLE_NAME = "roleName";
+    public static final String PARAM_IMPORT_IN_PROGRESS = "importInProgress";
 
     @Autowired
     RESTErrorUtil restErrorUtil;
@@ -269,7 +287,14 @@ public class RoleREST {
         } catch(Throwable excp) {
             LOG.error("deleteRole(" + roleId + ") failed", excp);
 
-            throw restErrorUtil.createRESTException(excp.getMessage());
+            if (excp.getMessage().contains(String.valueOf(ValidationErrorCode.ROLE_VALIDATION_ERR_INVALID_ROLE_ID.getErrorCode()))) {
+                throw restErrorUtil.createRESTException(
+                        "Data Not Found for given Id",
+                        MessageEnums.DATA_NOT_FOUND, roleId, null,
+                        "readResource : No Object found with given id.");
+            } else {
+                throw restErrorUtil.createRESTException(excp.getMessage());
+            }
         }
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== deleteRole(id=" + roleId + ")");
@@ -286,11 +311,14 @@ public class RoleREST {
     @Produces({ "application/json" })
     public RangerRole getRole(@QueryParam("serviceName") String serviceName, @QueryParam("execUser") String execUser, @PathParam("name") String roleName) {
         if (LOG.isDebugEnabled()) {
-            LOG.debug("==> getRole(name=" + roleName + ")");
+            LOG.debug("==> getRole(name=" + roleName + ", execUser=" + execUser + ")");
         }
         RangerRole ret;
+
         try {
-            ret = getRoleIfAccessible(roleName, serviceName, execUser, userMgr.getGroupsForUser(execUser));
+            Set<String> userGroups = StringUtils.isNotEmpty(execUser) ? userMgr.getGroupsForUser(execUser) : new HashSet<>();
+
+            ret = getRoleIfAccessible(roleName, serviceName, execUser, userGroups);
             if (ret == null) {
                 throw restErrorUtil.createRESTException("User doesn't have permissions to get details for " + roleName);
             }
@@ -298,12 +326,11 @@ public class RoleREST {
         } catch(WebApplicationException excp) {
             throw excp;
         } catch(Throwable excp) {
-            LOG.error("getRole(" + roleName + ") failed", excp);
-
+            LOG.error("getRole(name=" + roleName + ", execUser=" + execUser + ") failed", excp);
             throw restErrorUtil.createRESTException(excp.getMessage());
         }
         if (LOG.isDebugEnabled()) {
-            LOG.debug("<== getRole(name=" + roleName + "):" + ret);
+            LOG.debug("<== getRole(name=" + roleName + ", execUser=" + execUser + "):" + ret);
         }
         return ret;
     }
@@ -361,6 +388,181 @@ public class RoleREST {
         return ret;
     }
 
+	@GET
+	@Path("/roles/exportJson")
+	@Produces({ "application/json" })
+	@PreAuthorize("@rangerPreAuthSecurityHandler.isAdminRole()")
+	public void getRolesInJson(@Context HttpServletRequest request, @Context HttpServletResponse response) {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> getRolesInJson()");
+		}
+		try {
+			List<RangerRole> roleLists = getAllFilteredRoleList(request);
+
+			if (CollectionUtils.isNotEmpty(roleLists)) {
+				svcStore.getObjectInJson(roleLists, response, JSON_FILE_NAME_TYPE.ROLE);
+			} else {
+				response.setStatus(HttpServletResponse.SC_NO_CONTENT);
+				LOG.error("There is no Role to Export!!");
+			}
+
+		} catch (WebApplicationException excp) {
+			throw excp;
+		} catch (Throwable excp) {
+			LOG.error("Error while exporting policy file!!", excp);
+			throw restErrorUtil.createRESTException(excp.getMessage());
+		}
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("<== getRolesInJson()");
+		}
+	}
+
+	@POST
+	@Path("/roles/importRolesFromFile")
+	@Consumes({ MediaType.MULTIPART_FORM_DATA, MediaType.APPLICATION_JSON })
+	@Produces({ "application/json", "application/xml" })
+	@PreAuthorize("@rangerPreAuthSecurityHandler.isAdminRole()")
+	public RESTResponse importRolesFromFile(@Context HttpServletRequest request,
+			@FormDataParam("file") InputStream uploadedInputStream,
+			@FormDataParam("file") FormDataContentDisposition fileDetail,
+			@QueryParam("updateIfExists") Boolean updateIfExists,
+			@DefaultValue("false") @QueryParam("createNonExistUserGroupRole") Boolean createNonExistUserGroupRole) {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> RoleREST.importRolesFromFile()");
+		}
+		RESTResponse ret = new RESTResponse();
+
+		RangerContextHolder.getOrCreateOpContext().setBulkModeContext(true);
+
+		String metaDataInfo = null;
+		request.setAttribute(PARAM_IMPORT_IN_PROGRESS, true);
+
+		try {
+			roleService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_RANGER_ROLE, null, null, "IMPORT START"), "Import", "IMPORT START", null);
+
+			if (updateIfExists == null) {
+				updateIfExists = false;
+			}
+			List<String> roleNameList = new ArrayList<String>();
+
+			roleNameList = getRoleNameList(request, roleNameList);
+
+			String fileName = fileDetail.getFileName();
+			int totalRoleCreate = 0;
+			int totalRoleUpdate = 0;
+			int totalRoleUnchange = 0;
+			String msg;
+
+			if (fileName.endsWith("json")) {
+				try {
+					RangerExportRoleList rangerExportRoleList = null;
+					List<RangerRole> roles = null;
+					rangerExportRoleList = processRoleInputJsonForMetaData(uploadedInputStream, rangerExportRoleList);
+
+					if (rangerExportRoleList != null
+							&& !CollectionUtils.sizeIsEmpty(rangerExportRoleList.getMetaDataInfo())) {
+						metaDataInfo = JsonUtilsV2.mapToJson(rangerExportRoleList.getMetaDataInfo());
+					} else {
+						LOG.info("metadata info is not provided!!");
+					}
+					roles = getRolesFromProvidedJson(rangerExportRoleList);
+
+					if (roles != null && !CollectionUtils.sizeIsEmpty(roles)) {
+						for (RangerRole roleInJson : roles) {
+
+							if (roleInJson != null && StringUtils.isNotEmpty(roleInJson.getName().trim())) {
+								String roleNameInJson = roleInJson.getName().trim();
+								if (CollectionUtils.isNotEmpty(roleNameList) && roleNameList.contains(roleNameInJson)) {
+
+									// check updateIfExists
+									if (updateIfExists) {
+										try {
+											RangerRole exitingRole = roleStore.getRole(roleNameInJson);
+											if (!exitingRole.getId().equals(roleInJson.getId())) {
+												roleInJson.setId(exitingRole.getId());
+											}
+											if(exitingRole.equals(roleInJson)){
+												totalRoleUnchange++;
+												if (LOG.isDebugEnabled()) {
+													LOG.debug("Ignoring Roles from provided role in Json file... "+ roleNameInJson);
+												}
+											}
+											else {
+												roleStore.updateRole(roleInJson, createNonExistUserGroupRole);
+												totalRoleUpdate++;
+											}
+										} catch (WebApplicationException excp) {
+											throw excp;
+										} catch (Throwable excp) {
+											LOG.error("updateRole(" + roleInJson + ") failed", excp);
+
+											throw restErrorUtil.createRESTException(excp.getMessage());
+										}
+									} else {
+										totalRoleUnchange++;
+										if (LOG.isDebugEnabled()) {
+											LOG.debug("Ignoring Roles from provided role in Json file... " + roleNameInJson);
+										}
+									}
+									ret.setStatusCode(RESTResponse.STATUS_SUCCESS);
+								} else if (!roleNameList.contains(roleNameInJson) && (!roleNameInJson.isEmpty())) {
+									try {
+										roleStore.createRole(roleInJson, createNonExistUserGroupRole);
+									} catch (WebApplicationException excp) {
+										throw excp;
+									} catch (Throwable excp) {
+										LOG.error("createRole(" + roleInJson + ") failed", excp);
+
+										throw restErrorUtil.createRESTException(excp.getMessage());
+									}
+									totalRoleCreate++;
+									ret.setStatusCode(RESTResponse.STATUS_SUCCESS);
+								}
+							}
+						}
+					} else {
+						LOG.error("Json File does not contain any role.");
+						throw restErrorUtil.createRESTException("Json File does not contain any role.");
+					}
+					if (updateIfExists) {
+						msg = "Total Role Created = " + totalRoleCreate + " , Total Role Updated = " + totalRoleUpdate + " , Total Role Unchanged = " + totalRoleUnchange;
+						ret.setMsgDesc(msg);
+					} else {
+						msg = "Total Role Created = " + totalRoleCreate + " , Total Role Unchanged = " + totalRoleUnchange;
+						ret.setMsgDesc(msg);
+					}
+
+				} catch (IOException e) {
+					LOG.error(e.getMessage());
+					throw restErrorUtil.createRESTException(e.getMessage());
+				}
+			} else {
+				LOG.error("Provided file format is not supported!!");
+				throw restErrorUtil.createRESTException("Provided file format is not supported!!");
+			}
+		} catch (WebApplicationException excp) {
+			LOG.error("Error while importing role from file!!", excp);
+
+			roleService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_RANGER_ROLE, null, null, "IMPORT ERROR"), "Import failed", StringUtils.isNotEmpty(metaDataInfo) ? metaDataInfo : null, null);
+
+			throw excp;
+		} catch (Throwable excp) {
+			LOG.error("Error while importing role from file!!", excp);
+
+			roleService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_RANGER_ROLE, null, null, "IMPORT ERROR"), "Import failed", StringUtils.isNotEmpty(metaDataInfo) ? metaDataInfo : null, null);
+
+			throw restErrorUtil.createRESTException(excp.getMessage());
+		} finally {
+			roleService.createTransactionLog(new XXTrxLogV2(AppConstants.CLASS_TYPE_RANGER_ROLE, null, null, "IMPORT END"), "IMPORT END", StringUtils.isNotEmpty(metaDataInfo) ? metaDataInfo : null, null);
+
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("<== RoleREST.importRolesFromFile()");
+			}
+		}
+
+		return ret;
+	}
+
     @GET
     @Path("/lookup/roles")
     @Produces({ "application/json" })
@@ -1321,4 +1523,83 @@ public class RoleREST {
             request.setRoles(new HashSet<>());
         }
     }
+
+	protected List<RangerRole> getAllFilteredRoleList(HttpServletRequest request) throws Exception {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> getAllFilteredRoleList()");
+		}
+		String roleNames = null;
+		List<String> roleNameList = null;
+		List<RangerRole> roleLists = new ArrayList<>();
+
+		if (request.getParameter(PARAM_ROLE_NAME) != null) {
+			roleNames = request.getParameter(PARAM_ROLE_NAME);
+		}
+		if (StringUtils.isNotEmpty(roleNames)) {
+			roleNameList = new ArrayList<String>(Arrays.asList(roleNames.split(",")));
+		}
+
+		List<RangerRole> rangerRoleList = new ArrayList<RangerRole>();
+		SearchFilter filter = new SearchFilter();
+
+		rangerRoleList = roleStore.getRoles(filter);
+
+		if (!CollectionUtils.isEmpty(rangerRoleList)) {
+			for (RangerRole role : rangerRoleList) {
+				if (role != null) {
+					if (CollectionUtils.isNotEmpty(roleNameList)) {
+						if (roleNameList.contains(role.getName())) {
+							// set createTime & updateTime Time as null since exported Roles don't need this
+							role.setCreateTime(null);
+							role.setUpdateTime(null);
+							roleLists.add(role);
+							roleNameList.remove(role.getName());
+							if (roleNameList.size() == 0) {
+								break;
+							}
+						}
+					} else {
+						// set createTime & updateTime Time as null since exported Roles don't need this
+						role.setCreateTime(null);
+						role.setUpdateTime(null);
+						roleLists.add(role);
+					}
+				}
+			}
+		}
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("<== getAllFilteredRoleList()" + roleLists.size());
+		}
+		return roleLists;
+	}
+
+	private List<String> getRoleNameList(HttpServletRequest request, List<String> roleNameList) throws Exception {
+		SearchFilter filter = searchUtil.getSearchFilter(request, roleService.sortFields);
+		roleNameList = roleStore.getRoleNames(filter);
+		return roleNameList;
+	}
+
+	private RangerExportRoleList processRoleInputJsonForMetaData(InputStream uploadedInputStream,
+			RangerExportRoleList rangerExportRoleList) throws Exception {
+		String rolesString = IOUtils.toString(uploadedInputStream);
+		rolesString = rolesString.trim();
+		if (StringUtils.isNotEmpty(rolesString)) {
+			rangerExportRoleList = JsonUtilsV2.jsonToObj(rolesString, RangerExportRoleList.class);
+		} else {
+			LOG.error("Provided json file is empty!!");
+			throw restErrorUtil.createRESTException("Provided json file is empty!!");
+		}
+		return rangerExportRoleList;
+	}
+
+	private List<RangerRole> getRolesFromProvidedJson(RangerExportRoleList rangerExportRoleList) {
+		List<RangerRole> roles = null;
+		if (rangerExportRoleList != null && !CollectionUtils.sizeIsEmpty(rangerExportRoleList.getSecurityRoles())) {
+			roles = rangerExportRoleList.getSecurityRoles();
+		} else {
+			LOG.error("Provided json file does not contain any role!!");
+			throw restErrorUtil.createRESTException("Provided json file does not contain any role!!");
+		}
+		return roles;
+	}
 }
