@@ -151,6 +151,7 @@ public class RangerAPIList {
 	public static final String SEARCH_X_USERS = "XUserREST.searchXUsers";
 	public static final String GET_USERS_LOOKUP = "XUserREST.getUsersLookup";
 	public static final String GET_GROUPS_LOOKUP = "XUserREST.getGroupsLookup";
+	public static final String GET_PRINCIPALS_LOOKUP = "XUserREST.getPrincipalsLookup";
 	public static final String COUNT_X_USERS = "XUserREST.countXUsers";
 	public static final String GET_X_GROUP_USER = "XUserREST.getXGroupUser";
 	public static final String CREATE_X_GROUP_USER = "XUserREST.createXGroupUser";
@@ -209,4 +210,6 @@ public class RangerAPIList {
 	public static final String SET_USER_ROLES_BY_NAME="XUserREST.setUserRolesByName";
 	public static final String GET_USER_ROLES_BY_ID="XUserREST.getUserRolesByID";
 	public static final String GET_USER_ROLES_BY_NAME="XUserREST.getUserRolesByName";
+	public static final String FORCE_DELETE_EXTERNAL_USERS = "XUserREST.forceDeleteExternalUsers";
+	public static final String FORCE_DELETE_EXTERNAL_GROUPS = "XUserREST.forceDeleteExternalGroups";
 }
