@@ -132,12 +132,15 @@ public class RangerSecurityContextFormationFilter extends GenericFilterBean {
 
 				context.setUserSession(userSession);
 			}
+
+			setupAdminOpContext(request);
+
 			HttpServletResponse res = (HttpServletResponse)response;
 			res.setHeader("Cache-Control", "no-cache, no-store, max-age=0, must-revalidate");
 			res.setHeader("X-Frame-Options", "DENY" );
 			res.setHeader("X-XSS-Protection", "1; mode=block");
 			res.setHeader("Strict-Transport-Security", "max-age=31536000; includeSubDomains; preload");
-			res.setHeader("Content-Security-Policy", "default-src 'none'; script-src 'self' 'unsafe-inline' 'unsafe-eval'; connect-src 'self'; img-src 'self'; style-src 'self' 'unsafe-inline';font-src 'self'");
+			res.setHeader("Content-Security-Policy", "default-src 'none'; script-src 'self' 'unsafe-inline' 'unsafe-eval'; connect-src 'self'; img-src 'self' data:; style-src 'self' 'unsafe-inline';font-src 'self'");
 			res.setHeader("X-Permitted-Cross-Domain-Policies", "none");
 			chain.doFilter(request, res);
 
@@ -148,6 +151,14 @@ public class RangerSecurityContextFormationFilter extends GenericFilterBean {
 		}
 	}
 
+	private void setupAdminOpContext(ServletRequest request) {
+		Object attrCreatePrincipalsIfAbsent = request.getParameter("createPrincipalsIfAbsent");
+
+		if (attrCreatePrincipalsIfAbsent != null) {
+			RangerContextHolder.getOrCreateOpContext().setCreatePrincipalsIfAbsent(Boolean.parseBoolean(attrCreatePrincipalsIfAbsent.toString()));
+		}
+	}
+
 	private int getAuthType(HttpServletRequest request) {
 		int authType;
 		Object ssoEnabledObj = request.getAttribute("ssoEnabled");
