@@ -18,9 +18,10 @@
 package org.apache.ranger.service;
 
 
-import java.lang.reflect.Field;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -29,59 +30,61 @@ import java.util.Set;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
+import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
+import org.apache.ranger.authorization.utils.StringUtil;
+import org.apache.ranger.authorization.utils.JsonUtils;
 import org.apache.ranger.biz.ServiceDBStore;
-import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.view.VTrxLogAttr;
 import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.entity.XXSecurityZone;
 import org.apache.ranger.entity.XXServiceVersionInfo;
-import org.apache.ranger.entity.XXTrxLog;
-import org.apache.ranger.entity.XXUser;
 import org.apache.ranger.plugin.model.RangerPolicyDelta;
 import org.apache.ranger.plugin.model.RangerSecurityZone;
-import org.apache.ranger.util.RangerEnumUtil;
+import org.apache.ranger.plugin.model.RangerSecurityZone.RangerSecurityZoneService;
+import org.apache.ranger.plugin.util.JsonUtilsV2;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.context.annotation.Scope;
 import org.springframework.stereotype.Service;
 
-import com.google.gson.Gson;
-import com.google.gson.GsonBuilder;
+import javax.annotation.PostConstruct;
 
 @Service
 @Scope("singleton")
 public class RangerSecurityZoneServiceService extends RangerSecurityZoneServiceBase<XXSecurityZone, RangerSecurityZone> {
-
-	@Autowired
-	RangerEnumUtil xaEnumUtil;
-
 	@Autowired
 	ServiceDBStore serviceDBStore;
 
+    boolean compressJsonData = false;
+
     private static final Logger logger = LoggerFactory.getLogger(RangerSecurityZoneServiceService.class);
-    private static final Gson gsonBuilder = new GsonBuilder().setDateFormat("yyyyMMdd-HH:mm:ss.SSS-Z").create();
 
     private Map<Long, Set<String>> serviceNamesInZones = new HashMap<>();
     private Map<Long, Set<String>> tagServiceNamesInZones = new HashMap<>();
 
-    static HashMap<String, VTrxLogAttr> trxLogAttrs = new HashMap<String, VTrxLogAttr>();
-
-    static {
-		trxLogAttrs.put("name", new VTrxLogAttr("name", "Zone Name", false));
-		trxLogAttrs.put("services", new VTrxLogAttr("services", "Zone Services", false));
-		trxLogAttrs.put("adminUsers", new VTrxLogAttr("adminUsers", "Zone Admin Users", false));
-		trxLogAttrs.put("adminUserGroups", new VTrxLogAttr("adminUserGroups", "Zone Admin User Groups", false));
-		trxLogAttrs.put("auditUsers", new VTrxLogAttr("auditUsers", "Zone Audit Users", false));
-		trxLogAttrs.put("auditUserGroups", new VTrxLogAttr("auditUserGroups", "Zone Audit User Groups", false));
-		trxLogAttrs.put("description", new VTrxLogAttr("description", "Zone Description", false));
-                trxLogAttrs.put("tagServices", new VTrxLogAttr("tagServices", "Zone Tag Services", false));
-	}
 
     public RangerSecurityZoneServiceService() {
         super();
     }
 
+    @PostConstruct
+    public void initService() {
+        if (logger.isDebugEnabled()) {
+            logger.debug("==> RangerSecurityZoneServiceService.initService()");
+        }
+
+        RangerAdminConfig config = RangerAdminConfig.getInstance();
+
+        compressJsonData = config.getBoolean("ranger.admin.store.security.zone.compress.json_data", compressJsonData);
+
+        logger.info("ranger.admin.store.security.zone.compress.json_data={}", compressJsonData);
+
+        if (logger.isDebugEnabled()) {
+            logger.debug("<== RangerSecurityZoneServiceService.initService()");
+        }
+    }
+
     @Override
     protected void validateForCreate(RangerSecurityZone vObj) {
     }
@@ -99,26 +102,55 @@ public class RangerSecurityZoneServiceService extends RangerSecurityZoneServiceB
     protected XXSecurityZone mapViewToEntityBean(RangerSecurityZone securityZone, XXSecurityZone xxSecurityZone, int OPERATION_CONTEXT) {
         XXSecurityZone ret = super.mapViewToEntityBean(securityZone, xxSecurityZone, OPERATION_CONTEXT);
 
-        ret.setJsonData(gsonBuilder.toJson(securityZone));
+        String json = JsonUtils.objectToJson(securityZone);
+
+        if (StringUtils.isNotEmpty(json) && compressJsonData) {
+            try {
+                ret.setJsonData(null);
+                ret.setGzJsonData(StringUtil.gzipCompress(json));
+            } catch (IOException excp) {
+                logger.error("mapViewToEntityBean(): json compression failed (length="+json.length()+"). Will save uncompressed json", excp);
+
+                ret.setJsonData(json);
+                ret.setGzJsonData(null);
+            }
+        } else {
+            ret.setJsonData(json);
+            ret.setGzJsonData(null);
+        }
 
         return ret;
     }
     @Override
     protected RangerSecurityZone mapEntityToViewBean(RangerSecurityZone securityZone, XXSecurityZone xxSecurityZone) {
-        RangerSecurityZone ret = super.mapEntityToViewBean(securityZone, xxSecurityZone);
+        RangerSecurityZone ret    = super.mapEntityToViewBean(securityZone, xxSecurityZone);
+        byte[]             gzJson = xxSecurityZone.getGzJsonData();
+        String             json;
 
-        if (StringUtils.isNotEmpty(xxSecurityZone.getJsonData())) {
-            RangerSecurityZone zoneFromJsonData = gsonBuilder.fromJson(xxSecurityZone.getJsonData(), RangerSecurityZone.class);
+        if (gzJson != null) {
+            try {
+                json = StringUtil.gzipDecompress(gzJson);
+            } catch (IOException excp) {
+                json = xxSecurityZone.getJsonData();
 
-            if (zoneFromJsonData == null) {
-                logger.info("Cannot read jsonData into RangerSecurityZone object in [" + xxSecurityZone.getJsonData() + "]!!");
-            } else {
+                logger.error("mapEntityToViewBean(): decompression of x_security_zone.gz_jsonData failed (length={}). Will use contents of x_security_zone.jsonData (length={})", gzJson.length, (json != null ? json.length() : 0), excp);
+            }
+        } else {
+            json = xxSecurityZone.getJsonData();
+        }
+
+        if (StringUtils.isNotEmpty(json)) {
+            RangerSecurityZone zoneFromJsonData = JsonUtils.jsonToObject(json, RangerSecurityZone.class);
+
+            if (zoneFromJsonData != null) {
                 ret.setName(zoneFromJsonData.getName());
                 ret.setServices(zoneFromJsonData.getServices());
                 ret.setAdminUsers(zoneFromJsonData.getAdminUsers());
                 ret.setAdminUserGroups(zoneFromJsonData.getAdminUserGroups());
+                ret.setAdminRoles(zoneFromJsonData.getAdminRoles());
                 ret.setAuditUsers(zoneFromJsonData.getAuditUsers());
                 ret.setAuditUserGroups(zoneFromJsonData.getAuditUserGroups());
+                ret.setAuditRoles(zoneFromJsonData.getAuditRoles());
                 ret.setTagServices(zoneFromJsonData.getTagServices());
             }
         } else {
@@ -135,10 +167,15 @@ public class RangerSecurityZoneServiceService extends RangerSecurityZoneServiceB
         RangerSecurityZone ret = super.postCreate(xObj);
         Set<String> serviceNames = ret.getServices().keySet();
 
+        List<String> tagServiceNames = ret.getTagServices();
+
         // Create default zone policies
         try {
             serviceDBStore.createZoneDefaultPolicies(serviceNames, ret);
             updateServiceInfos(serviceNames);
+
+            serviceDBStore.createZoneDefaultPolicies(tagServiceNames, ret);
+            updateServiceInfos(tagServiceNames);
         } catch (Exception exception) {
             logger.error("postCreate processing failed for security-zone:[" + ret + "]", exception);
             ret = null;
@@ -200,6 +237,40 @@ public class RangerSecurityZoneServiceService extends RangerSecurityZoneServiceB
         return ret;
     }
 
+    @Override
+    public String getTrxLogAttrValue(RangerSecurityZone obj, VTrxLogAttr trxLogAttr) {
+        final String ret;
+
+        if (compressJsonData && obj != null && "services".equalsIgnoreCase(trxLogAttr.getAttribName())) {
+            Map<String, RangerSecurityZoneService> servicesSummary = new HashMap<>();
+
+            for (Map.Entry<String, RangerSecurityZoneService> entry : obj.getServices().entrySet()) {
+                String                    serviceName    = entry.getKey();
+                RangerSecurityZoneService service        = entry.getValue();
+                int                       resourceCount  = service != null && service.getResources() != null ? service.getResources().size() : 0;
+                RangerSecurityZoneService serviceSummary = new RangerSecurityZoneService();
+
+                serviceSummary.getResources().add((new HashMap<String, List<String>>() {{ put("resourceCount", Collections.singletonList(Integer.toString(resourceCount))); }}));
+
+                servicesSummary.put(serviceName, serviceSummary);
+            }
+
+            String summaryJson = null;
+
+            try {
+                summaryJson = JsonUtilsV2.mapToJson(servicesSummary);
+            } catch (Exception excp) {
+                logger.error("getFieldValue(): failed to convert services to JSON", excp);
+            }
+
+            ret = summaryJson;
+        } else {
+            ret = super.getTrxLogAttrValue(obj, trxLogAttr);
+        }
+
+        return ret;
+    }
+
     private void updateServiceInfos(Collection<String> services) {
         if(CollectionUtils.isEmpty(services)) {
             return;
@@ -222,97 +293,4 @@ public class RangerSecurityZoneServiceService extends RangerSecurityZoneServiceB
         }
 
     }
-
-	public List<XXTrxLog> getTransactionLog(RangerSecurityZone vSecurityZone, RangerSecurityZone securityZoneDB, String action) {
-		if (vSecurityZone == null || action == null  || ("update".equalsIgnoreCase(action) && securityZoneDB == null)) {
-			return null;
-		}
-		List<XXTrxLog> trxLogList = new ArrayList<XXTrxLog>();
-		Field[] fields = vSecurityZone.getClass().getDeclaredFields();
-
-		try {
-			Field nameField = vSecurityZone.getClass().getDeclaredField("name");
-			nameField.setAccessible(true);
-			String objectName = "" + nameField.get(vSecurityZone);
-
-			for (Field field : fields) {
-				String fieldName = field.getName();
-				if (!trxLogAttrs.containsKey(fieldName)) {
-					continue;
-				}
-				field.setAccessible(true);
-				VTrxLogAttr vTrxLogAttr = trxLogAttrs.get(fieldName);
-				XXTrxLog xTrxLog = new XXTrxLog();
-				xTrxLog.setAttributeName(vTrxLogAttr
-						.getAttribUserFriendlyName());
-				xTrxLog.setAction(action);
-				xTrxLog.setObjectId(vSecurityZone.getId());
-				xTrxLog.setObjectClassType(AppConstants.CLASS_TYPE_RANGER_SECURITY_ZONE);
-				xTrxLog.setObjectName(objectName);
-
-				String value = null;
-				if (vTrxLogAttr.isEnum()) {
-					String enumName = XXUser.getEnumName(fieldName);
-					int enumValue = field.get(vSecurityZone) == null ? 0 : Integer
-							.parseInt("" + field.get(vSecurityZone));
-					value = xaEnumUtil.getLabel(enumName, enumValue);
-				} else {
-					value = "" + field.get(vSecurityZone);
-					if ((value == null || "null".equalsIgnoreCase(value))
-							&& !"update".equalsIgnoreCase(action)) {
-						continue;
-					}
-				}
-				if("services".equalsIgnoreCase(fieldName)) {
-					Gson gson = new Gson();
-					value = gson.toJson(vSecurityZone.getServices(), HashMap.class);
-				}
-				if ("create".equalsIgnoreCase(action)) {
-					xTrxLog.setNewValue(value);
-					trxLogList.add(xTrxLog);
-				}
-				else if ("delete".equalsIgnoreCase(action)) {
-					xTrxLog.setPreviousValue(value);
-					trxLogList.add(xTrxLog);
-				}
-				else if ("update".equalsIgnoreCase(action)) {
-					String oldValue = null;
-					Field[] mFields = vSecurityZone.getClass().getDeclaredFields();
-					for (Field mField : mFields) {
-						mField.setAccessible(true);
-						String mFieldName = mField.getName();
-						if (fieldName.equalsIgnoreCase(mFieldName)) {
-							if("services".equalsIgnoreCase(mFieldName)) {
-								Gson gson = new Gson();
-								oldValue = gson.toJson(securityZoneDB.getServices(), HashMap.class);
-							}
-							else {
-								oldValue = mField.get(securityZoneDB) + "";
-							}
-							break;
-						}
-					}
-					if (oldValue == null || oldValue.equalsIgnoreCase(value)) {
-						continue;
-					}
-					xTrxLog.setPreviousValue(oldValue);
-					xTrxLog.setNewValue(value);
-					trxLogList.add(xTrxLog);
-				}
-			}
-			if (trxLogList.isEmpty()) {
-				XXTrxLog xTrxLog = new XXTrxLog();
-				xTrxLog.setAction(action);
-				xTrxLog.setObjectClassType(AppConstants.CLASS_TYPE_RANGER_SECURITY_ZONE);
-				xTrxLog.setObjectId(vSecurityZone.getId());
-				xTrxLog.setObjectName(objectName);
-				trxLogList.add(xTrxLog);
-			}
-		} catch (IllegalAccessException e) {
-			logger.error("Transaction log failure.", e);
-		} catch (NoSuchFieldException e) {
-			logger.error("Transaction log failure.", e);
-		}
-		return trxLogList;
-	}
 }
