@@ -213,7 +213,7 @@ define(function(require) {
 		$.notify({
 			icon: 'fa-fw fa fa-exclamation-circle',
 			title: '<strong>Info!</strong>',
-			message: text
+			message: _.escape(text)
 		});
 	};
 
@@ -231,7 +231,7 @@ define(function(require) {
 		$.notify({
 			icon: 'fa-fw fa fa-exclamation-triangle',
 			title: '<strong>Error!</strong>',
-			message: text
+			message: _.escape(text)
 		},{
 			type: 'danger',
 		});
@@ -251,7 +251,7 @@ define(function(require) {
 		$.notify({
 			icon: 'fa-fw fa fa-check-circle',
 			title: '<strong>Success!</strong>',
-			message: text
+			message: _.escape(text)
 		},{
 			type: 'success'
 		});
@@ -497,7 +497,7 @@ define(function(require) {
 		} else
 			return '--';
 	};
-        XAUtils.showGroupsOrUsersForPolicy = function(rawValue, model, showType, rangerServiceDefModel) {
+        XAUtils.showGroupsOrUsersForPolicy = function(model, showType, rangerServiceDefModel) {
 		var showMoreLess = false, groupArr = [], items = [];
 		var itemList = ['policyItems','allowExceptions','denyPolicyItems','denyExceptions','dataMaskPolicyItems','rowFilterPolicyItems']
 		if(!_.isUndefined(rangerServiceDefModel)){
@@ -628,6 +628,12 @@ define(function(require) {
 			} else {
 				window.location = 'login.jsp?sessionTimeout=true';
 			}
+		}else if(error.status == 400 && error.responseJSON && error.responseJSON.messageList && error.responseJSON.messageList[0].name) {
+			if(error.responseJSON.messageList[0].name == "DATA_NOT_FOUND" || error.responseJSON.messageList[0].name == "INVALID_INPUT_DATA"){
+				App.rContent.show(new vError({
+					status : error.status
+				}));
+			}
 		}
 	};
 	XAUtils.select2Focus = function(event) {
@@ -1667,6 +1673,7 @@ define(function(require) {
         return {
             closeOnSelect : true,
             placeholder   : placeholder,
+            separator : "@-undefined-@",
             tags : true,
             width : width,
             initSelection: function(element, callback) {
@@ -1719,8 +1726,14 @@ define(function(require) {
                         //remove selected values
                         if(that.collection && that.collection.models){
                             _.filter(that.collection.models, function(model){
-                                if(model && !_.isUndefined(model.get('name'))){
-                                    selectedVals.push(model.get('name'));
+                                if(auditFilter !== "auditFilter"){
+                                    if(model && !_.isUndefined(model.get('name'))){
+                                        selectedVals.push(model.get('name'));
+                                    }
+                                } else {
+                                    if(model && !_.isUndefined(model.get($select))){
+                                        selectedVals = model.get($select);
+                                    }
                                 }
                             })
                         }
@@ -1916,57 +1929,66 @@ define(function(require) {
         }
     }
 
-    XAUtils.setIdealActivityTime = function() {
+    XAUtils.setIdleActivityTime = function() {
         var App = require('App');
         var INACTIVITY_TIME_OUT = 900;
         if (App.userProfile && App.userProfile.get('configProperties') && App.userProfile.get('configProperties').inactivityTimeout) {
             INACTIVITY_TIME_OUT = parseInt(App.userProfile.get('configProperties').inactivityTimeout);
         }
         INACTIVITY_TIME_OUT *= 1000;
-        localStorage.setItem('idealTimeOut', moment().add(INACTIVITY_TIME_OUT, 'milliseconds').valueOf());
+        localStorage.setItem('idleTimeOut', moment().add(INACTIVITY_TIME_OUT, 'milliseconds').valueOf());
         localStorage.setItem('idleTimerLoggedOut', false);
-        XAUtils.setIdealActivityTime = function () {
-            localStorage.setItem('idealTimeOut', moment().add(INACTIVITY_TIME_OUT, 'milliseconds').valueOf());
+        XAUtils.setIdleActivityTime = function () {
+            localStorage.setItem('idleTimeOut', moment().add(INACTIVITY_TIME_OUT, 'milliseconds').valueOf());
             var isLoggedOut = localStorage.getItem('idleTimerLoggedOut') == "true";
             if (isLoggedOut) {
                 localStorage.setItem('idleTimerLoggedOut', false);
             }
             XAUtils.startIdealActivityInterval()
         }
+        XAUtils.setIdleActivityTime();
     };
 
     XAUtils.startIdealActivityInterval = function () {
         clearInterval(XAUtils.activityIntervalID)
         XAUtils.activityIntervalID = setInterval(function() {
-            var idealTimeVal = parseInt(localStorage.getItem('idealTimeOut'));
-            if(moment().isAfter(moment(idealTimeVal))) {
+            var idleTimeVal = parseInt(localStorage.getItem('idleTimeOut'));
+            if(moment().isAfter(moment(idleTimeVal))) {
                 clearInterval(XAUtils.activityIntervalID)
                 var isLoggedOut = localStorage.getItem('idleTimerLoggedOut') == "true";
                 if(isLoggedOut) {
                     localStorage.setItem('idleTimerLoggedOut', 'false');
-                    XAUtils.idealActivityLogout();
+                    XAUtils.idleActivityLogout();
                 } else {
-                    XAUtils.idelTimePopup();
+                    XAUtils.idleTimePopup();
                 }
             }
         }, 2000);
     };
 
-    XAUtils.idelTimePopup = function() {
+    XAUtils.idleTimePopup = function() {
         var timeLeft = 15;
         var $elem = '<div id="Timer"></div>';
 
         function countdown() {
-            if (timeLeft == 0) {
+            var idleTimeVal = parseInt(localStorage.getItem('idleTimeOut'));
+            if (timeLeft == 0 ) {
                 clearTimeout(timerId);
-                localStorage.setItem('idleTimerLoggedOut', 'false');
-                XAUtils.idealActivityLogout();
+                if (!moment().isAfter(moment(idleTimeVal))) {
+                    bootbox.hideAll()
+                } else {
+                    localStorage.setItem('idleTimerLoggedOut', 'false');
+                    XAUtils.idleActivityLogout();
+                }
             } else {
                 var isLoggedOut = localStorage.getItem('idleTimerLoggedOut') == "true";
                 if(isLoggedOut) {
                     clearTimeout(timerId);
                     localStorage.setItem('idleTimerLoggedOut', 'false');
-                    XAUtils.idealActivityLogout();
+                    XAUtils.idleActivityLogout();
+                } else if (!moment().isAfter(moment(idleTimeVal))) {
+                    bootbox.hideAll()
+                    clearTimeout(timerId);
                 } else {
                     $.find('#Timer')[0].innerHTML ='Time left : '+ timeLeft + ' seconds remaining';
                     timeLeft--;
@@ -1975,15 +1997,15 @@ define(function(require) {
         }
         bootbox.dialog({
             title: 'Session Expiration Warning',
-            message: '<span class="inline-block">' + localization.tt('dialogMsg.idelTimeOutMsg') +'<br>'+ $elem + '</span>',
+            message: '<span class="inline-block">' + localization.tt('dialogMsg.idleTimeOutMsg') +'<br>'+ $elem + '</span>',
             closeButton: false,
             buttons: {
                 noclose: {
                     "label" : localization.tt('btn.stayLoggdedIn'),
-                    "className" : "btn-success btn-sm",
+                    "className" : "btn-success btn-sm stayLoggdedIn-popup",
                     "callback" : function() {
                         clearTimeout(timerId);
-                        XAUtils.setIdealActivityTime()
+                        XAUtils.setIdleActivityTime()
                     }
                 },
                 cancel: {
@@ -1991,7 +2013,7 @@ define(function(require) {
                     "className" : "btn-danger btn-sm",
                     "callback" : function() {
                         localStorage.setItem('idleTimerLoggedOut', 'false');
-                        XAUtils.idealActivityLogout();
+                        XAUtils.idleActivityLogout();
                     }
                 }
             }
@@ -2002,7 +2024,7 @@ define(function(require) {
         return false;
     };
 
-    XAUtils.idealActivityLogout = function () {
+    XAUtils.idleActivityLogout = function () {
         var App = require('App');
         // localStorage.setItem('idleTimerLoggedOut', true);
         if(localStorage.getItem('idleTimerLoggedOut') == "false") {
