@@ -136,8 +136,7 @@ define(function(require){
 					editable: false,
 					sortable : false
 				},
-				groupPermList : {
-					reName : 'groupPermList',
+				groupNameList : {
 					cell	: Backgrid.HtmlCell.extend({className: 'cellWidth-1'}),
 					label : localization.tt("lbl.group"),
 					formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
@@ -153,8 +152,7 @@ define(function(require){
 					sortable : false
 				},
 				//Hack for backgrid plugin doesn't allow to have same column name
-				userPermList : {
-					reName : 'userPermList',
+				userNameList : {
 					cell	: Backgrid.HtmlCell.extend({className: 'cellWidth-1'}),
 					label : localization.tt("lbl.users"),
 					formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
@@ -174,7 +172,7 @@ define(function(require){
 				label : localization.tt("lbl.action"),
 				formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
 					fromRaw: function (rawValue,model) {
-						return '<a href="#!/permissions/'+model.id+'/edit" class="btn btn-mini" title="Edit"><i class="icon-edit icon-large"></i></a>';
+						return '<a href="#!/permissions/'+model.id+'/edit" class="btn btn-sm" title="Edit"><i class="fa-fw fa fa-edit fa-fw fa fa-large"></i></a>';
 					}
 				}),
 				editable: false,
