@@ -16,9 +16,12 @@
  */
 package org.apache.ranger.rest;
 
+import static org.mockito.ArgumentMatchers.eq;
+
 import java.util.ArrayList;
 import java.util.List;
 
+import javax.servlet.http.HttpServletRequest;
 import javax.ws.rs.WebApplicationException;
 
 import org.apache.ranger.biz.AssetMgr;
@@ -26,6 +29,7 @@ import org.apache.ranger.biz.RangerBizUtil;
 import org.apache.ranger.biz.ServiceDBStore;
 import org.apache.ranger.biz.TagDBStore;
 import org.apache.ranger.common.RESTErrorUtil;
+import org.apache.ranger.common.RangerSearchUtil;
 import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.db.XXServiceDao;
 import org.apache.ranger.db.XXServiceDefDao;
@@ -33,13 +37,20 @@ import org.apache.ranger.entity.XXService;
 import org.apache.ranger.entity.XXServiceDef;
 import org.apache.ranger.plugin.model.RangerService;
 import org.apache.ranger.plugin.model.RangerServiceResource;
+import org.apache.ranger.plugin.model.RangerServiceResourceWithTags;
 import org.apache.ranger.plugin.model.RangerTag;
 import org.apache.ranger.plugin.model.RangerTagDef;
 import org.apache.ranger.plugin.model.RangerTagResourceMap;
+import org.apache.ranger.plugin.store.PList;
 import org.apache.ranger.plugin.store.TagValidator;
 import org.apache.ranger.plugin.util.RangerPluginCapability;
 import org.apache.ranger.plugin.util.SearchFilter;
 import org.apache.ranger.plugin.util.ServiceTags;
+import org.apache.ranger.service.RangerServiceResourceService;
+import org.apache.ranger.service.RangerServiceResourceWithTagsService;
+import org.apache.ranger.service.RangerTagDefService;
+import org.apache.ranger.service.RangerTagService;
+import org.apache.ranger.view.RangerServiceResourceWithTagsList;
 import org.junit.Assert;
 import org.junit.FixMethodOrder;
 import org.junit.Rule;
@@ -90,6 +101,21 @@ public class TestTagREST {
 	@Mock
 	AssetMgr assetMgr;
 
+	@Mock
+	RangerSearchUtil searchUtil;
+
+	@Mock
+	RangerTagDefService tagDefService;
+
+	@Mock
+	RangerTagService tagService;
+
+	@Mock
+	RangerServiceResourceService resourceService;
+
+	@Mock
+	RangerServiceResourceWithTagsService serviceResourceWithTagsService;
+
 	@Rule
 	public ExpectedException thrown = ExpectedException.none();
 
@@ -420,7 +446,39 @@ public class TestTagREST {
 		} catch (Exception e) {
 		}
 	}
-	
+
+	@Test
+	public void test62getTagDefs() {
+		HttpServletRequest  request      = Mockito.mock(HttpServletRequest.class);
+		PList<RangerTagDef> ret          = new PList<RangerTagDef>();
+		List<RangerTagDef>  tagDefList   = new ArrayList<RangerTagDef>();
+		SearchFilter        searchFilter = new SearchFilter();
+		RangerTagDef        rangerTagDef = new RangerTagDef();
+
+		rangerTagDef.setId(id);
+		rangerTagDef.setVersion(5L);
+		tagDefList.add(rangerTagDef);
+		ret.setList(tagDefList);
+
+		Mockito.when(searchUtil.getSearchFilter(Mockito.any(HttpServletRequest.class), eq(tagDefService.sortFields)))
+		       .thenReturn(searchFilter);
+
+		try {
+			Mockito.when(tagStore.getPaginatedTagDefs((SearchFilter) Mockito.any())).thenReturn(ret);
+		} catch (Exception e) {
+		}
+		PList<RangerTagDef> result = tagREST.getTagDefs(request);
+
+		Assert.assertNotNull(result);
+		Assert.assertEquals(result.getList().get(0).getId(), tagDefList.get(0).getId());
+		Assert.assertEquals(result.getList().get(0).getVersion(), tagDefList.get(0).getVersion());
+
+		try {
+			Mockito.verify(tagStore).getPaginatedTagDefs((SearchFilter) Mockito.any());
+		} catch (Exception e) {
+		}
+	}
+
 	@Test
 	public void test15getAllTagDefs() {
 		try {
@@ -742,6 +800,38 @@ public class TestTagREST {
 		}
 	}
 	
+	@Test
+	public void test63getTags() {
+		HttpServletRequest request      = Mockito.mock(HttpServletRequest.class);
+		SearchFilter       searchFilter = new SearchFilter();
+		String             testTagType  = "TAG-TYPE";
+		PList<RangerTag>   ret          = new PList<RangerTag>();
+		List<RangerTag>    tagList      = new ArrayList<RangerTag>();
+		RangerTag          tag          = new RangerTag();
+
+		tag.setType(testTagType);
+		tagList.add(tag);
+		ret.setList(tagList);
+
+		Mockito.when(searchUtil.getSearchFilter(Mockito.any(HttpServletRequest.class), eq(tagService.sortFields)))
+		       .thenReturn(searchFilter);
+
+		try {
+			Mockito.when(tagStore.getPaginatedTags(searchFilter)).thenReturn(ret);
+		} catch (Exception e) {
+		}
+
+		PList<RangerTag> result = tagREST.getTags(request);
+
+		Assert.assertNotNull(result);
+		Assert.assertEquals(result.getList().get(0).getType(), tagList.get(0).getType());
+
+		try {
+			Mockito.verify(tagStore).getPaginatedTags((SearchFilter) Mockito.any());
+		} catch (Exception e) {
+		}
+	}
+
 	@Test
 	public void test27createServiceResource() {
 		RangerServiceResource oldRSR = null;
@@ -1048,11 +1138,54 @@ public class TestTagREST {
 		} catch (Exception e) {
 		}
 	}
-	
+
+	@Test
+	public void test64getServiceResourcesWithTags() {
+		HttpServletRequest                   request               = Mockito.mock(HttpServletRequest.class);
+		SearchFilter                         searchFilter          = new SearchFilter();
+		RangerServiceResourceWithTagsList    ret                   = new RangerServiceResourceWithTagsList();
+		List<RangerServiceResourceWithTags>  serviceResourceList   = new ArrayList<RangerServiceResourceWithTags>();
+		RangerServiceResourceWithTags        rangerServiceResource = new RangerServiceResourceWithTags();
+		List<RangerTag>                      associatedTags        = new ArrayList<RangerTag>();
+		RangerTag                            rangerTag             = new RangerTag();
+
+		rangerTag.setId(id);
+		rangerTag.setGuid(gId);
+		rangerTag.setType(name);
+		associatedTags.add(rangerTag);
+
+		rangerServiceResource.setId(id);
+		rangerServiceResource.setServiceName(serviceName);
+		rangerServiceResource.setAssociatedTags(associatedTags);
+		serviceResourceList.add(rangerServiceResource);
+		ret.setResourceList(serviceResourceList);
+
+		Mockito.when(searchUtil.getSearchFilter(Mockito.any(HttpServletRequest.class), eq(resourceService.sortFields))).thenReturn(searchFilter);
+
+		try {
+			Mockito.when(tagStore.getPaginatedServiceResourcesWithTags(Mockito.any())).thenReturn(ret);
+		} catch (Exception e) {
+		}
+
+		RangerServiceResourceWithTagsList result = tagREST.getServiceResourcesWithTags(request);
+
+		Assert.assertNotNull(result.getResourceList().get(0).getId());
+		Assert.assertEquals(result.getResourceList().get(0).getId(), serviceResourceList.get(0).getId());
+		Assert.assertEquals(result.getResourceList().get(0).getServiceName(), serviceResourceList.get(0).getServiceName());
+		Assert.assertEquals(result.getResourceList().get(0).getAssociatedTags().size(), 1);
+		Assert.assertEquals(result.getResourceList().get(0).getAssociatedTags().get(0).getType(), name);
+
+		try {
+			Mockito.verify(tagStore).getPaginatedServiceResourcesWithTags((SearchFilter) Mockito.any());
+		} catch (Exception e) {
+		}
+	}
+
 	@Test
 	public void test38createTagResourceMap() {
 		RangerTagResourceMap oldTagResourceMap = null;
 		RangerTagResourceMap newTagResourceMap = new RangerTagResourceMap();
+
 		newTagResourceMap.setTagId(id);
 		newTagResourceMap.setResourceId(id);
 		
@@ -1070,6 +1203,7 @@ public class TestTagREST {
 		}
 		
 		RangerTagResourceMap rangerTagResourceMap = tagREST.createTagResourceMap(tagGuid, resourceGuid, false);
+
 		Assert.assertEquals(rangerTagResourceMap.getTagId(), newTagResourceMap.getTagId());
 		Assert.assertEquals(rangerTagResourceMap.getResourceId(), newTagResourceMap.getResourceId());
 		
@@ -1077,10 +1211,12 @@ public class TestTagREST {
 			Mockito.verify(tagStore).getTagResourceMapForTagAndResourceGuid(tagGuid, resourceGuid);
 		} catch (Exception e) {
 		}
+
 		try {
 			Mockito.verify(validator).preCreateTagResourceMap(tagGuid, resourceGuid);
 		} catch (Exception e) {
 		}
+
 		try {
 			Mockito.verify(tagStore).createTagResourceMap(newTagResourceMap);
 		} catch (Exception e) {
@@ -1095,7 +1231,9 @@ public class TestTagREST {
 			Mockito.when(tagStore.getTagResourceMapForTagAndResourceGuid(tagGuid, resourceGuid)).thenReturn(oldTagResourceMap);
 		} catch (Exception e) {
 		}
+
 		Mockito.when(restErrorUtil.createRESTException(Mockito.anyInt(),Mockito.anyString(), Mockito.anyBoolean())).thenThrow(new WebApplicationException());
+
 		thrown.expect(WebApplicationException.class);
 		tagREST.createTagResourceMap(tagGuid, resourceGuid, false);
 		
